"""Module for Account Account and Accoun tMove models."""
from odoo import fields, models


class AccountAccount(models.Model):
    """Agrega campos necesarios para aplicar/machear en impuestos."""

    _inherit = "account.account"

    account_result = fields.Boolean("Account Result")
    account_end_result = fields.Boolean("Account End Result")


class AccountMove(models.Model):
    """Agrega campos y overridea metodos"""

    _inherit = "account.move"

    date_result = fields.Date("Date Result")

    def _check_fiscalyear_lock_date(self):
        moves_without_date_result = self.filtered(lambda move: not move.date_result)
        if moves_without_date_result:
            return super(AccountMove, self)._check_fiscalyear_lock_date()
        return True

    def _get_violated_lock_dates(self, invoice_date, has_tax):
        """
        Get all the lock dates affecting the current invoice_date.
        :param invoice_date: The invoice date
        :param has_tax: If any taxes are involved in the lines of the invoice
        :return: a list of tuples containing the lock dates affecting this move, ordered chronologically.
        """
        locks = []
        if not self.date_result:
            locks = super(AccountMove, self)._get_violated_lock_dates(
                invoice_date, has_tax
            )
        return locks

    def _get_lock_date_message(self, invoice_date, has_tax):
        """
        Get a message describing the latest lock date affecting the specified date.
        :param invoice_date: The date to be checked
        :param has_tax: If any taxes are involved in the lines of the invoice
        :return: a message describing the latest lock date affecting this move and the date it will be
                 accounted on if posted, or False if no lock dates affect this move.
        """
        res = False
        if not self.date_result:
            res = super(AccountMove, self)._get_lock_date_message(invoice_date, has_tax)
        return res
