from odoo import _, api, fields, models


class ProductItems(models.Model):
    _name = "product.family"
    _order = "name"
    _description = "Items and Product Variants"

    name = fields.Char(string="Name", required=True)

    @api.constrains("name")
    def _check_unique_name(self):
        for record in self:
            # Search for other records with the same name, ignoring case
            duplicate_records = self.search(
                [("name", "=ilike", record.name), ("id", "!=", record.id)]
            )
            if duplicate_records:
                raise models.ValidationError(_("Cannot create a duplicate item."))


class ProductSubFamily(models.Model):
    _name = "product.subfamily"
    _description = "SubRubros"
    _order = "family_id, name"

    # Definition of fields

    _inherit = ["mail.thread", "mail.activity.mixin"]

    name = fields.Char(string="Name", required=True)
    family_id = fields.Many2one("product.family", string="Rubro", required=True)

    _sql_constraints = [
        (
            "unique_family_subfamily",
            "UNIQUE(family_id, name)",
            "El Rubro y el SubRubro deben ser únicos.",
        ),
    ]

    @api.constrains("family_id", "name")
    def _check_unique_family_subfamily(self):
        for record in self:
            # Search for other records with the same combination of Item and Sub Item
            duplicate_records = self.search(
                [
                    ("family_id", "=", record.family_id.id),
                    ("name", "=ilike", record.name),
                    ("id", "!=", record.id),
                ]
            )
            if duplicate_records:
                raise models.ValidationError(
                    _("El Rubro y el SubRubro deben ser únicos.")
                )
