"""class what contains the definition of the Product Spare Parts model."""
from odoo import _, api, fields, models
from odoo.exceptions import ValidationError


class ProductSpareParts(models.Model):
    """This class contains the definition of the Product Spare Parts model."""

    _name = "product.spare.parts"
    _description = "Product Spare Parts"

    name = fields.Char(string="Nombre")
    product_template_id = fields.Many2one(
        "product.template", string="Producto", required=True
    )

    product_id = fields.Many2one(
        "product.product",
        string="Producto",
        domain="[('product_tmpl_id', '!=', product_template_id),('product_tmpl_id.detailed_type', '=', 'product')]",
    )
    product_tag_ids = fields.Many2many(
        related="product_id.product_tag_ids", string="Etiquetas de Producto"
    )
    sequence = fields.Integer(string="Secuencia", default=10)
    active = fields.Boolean(
        string="Activo", default=True, compute="_compute_active", store=True
    )

    _sql_constraints = [
        (
            "unique_product_template_product",
            "UNIQUE(product_template_id, product_id)",
            "No es posible relacionar el mismo repuesto más de una vez al formulario.",
        ),
    ]

    @api.depends("product_id.active")
    def _compute_active(self):
        """Compute the active field based on the product_id active field."""
        for record in self:
            record.active = record.product_id.active

    @api.constrains("product_id")
    def _check_product_id_not_empty(self):
        """Check that the product_id field is not empty."""
        for record in self:
            if not record.product_id:
                raise ValidationError(
                    _(
                        "No se permite grabar valor de repuesto vacío, debe seleccionar un repuesto."
                    )
                )


class ProductMachinery(models.Model):
    """This class contains the definition of the Product Machinery model."""

    _name = "product.machinery"

    name = fields.Char(string="Nombre")
    product_template_id = fields.Many2one(
        "product.template", string="Maquinarias asociadas", required=True
    )
    product_id = fields.Many2one(
        "product.product",
        string="Maquinarias asociadas",
        required=True,
        domain="[('product_tmpl_id', '!=', product_template_id),('product_tmpl_id.detailed_type', '=', 'product')]",
    )
    alternative_product_ids = fields.One2many(
        "product.spare.parts", "product_id", string="Repuestos"
    )
