from odoo import _, api, models


class PurchaseOrderLine(models.Model):
    _inherit = "purchase.order.line"

    @api.onchange("product_id")
    def onchange_product_id_warning(self):
        if not self.product_id or not self.env.user.has_group(
            "purchase.group_warning_purchase"
        ):
            return {}

        warning = {}
        title = False
        message = False
        product_info = self.product_id

        # Check for purchase line warning
        if product_info.purchase_line_warn != "no-message":
            title = _("Warning for %s", product_info.name)
            message = product_info.purchase_line_warn_msg
            warning["title"] = f"Aviso para producto {self.product_id.display_name}"
            warning["message"] = message

        # Check if optional products are available
        if self.product_id.optional_product_ids:
            optional_product_info = "\n".join(
                f"{[product.default_code]} - {product.name}"
                for product in self.product_id.optional_product_ids
            )
            optional_warning_message = f"El producto seleccionado {self.product_id.display_name} tiene los siguientes productos complementarios:\n{optional_product_info}"
            if warning.get("message"):
                # Append the optional warning to the existing warning message
                warning["message"] += f"\n\n{optional_warning_message}"
            else:
                warning["title"] = f"Aviso para producto {self.product_id.display_name}"
                warning["message"] = optional_warning_message

        if product_info.purchase_line_warn != "no-message":
            if product_info.purchase_line_warn == "block":
                self.product_id = False

        if warning:
            return {"warning": warning}

        return {}
