from odoo import _, api, fields, models
from odoo.exceptions import ValidationError


class AssociatedMachines(models.Model):
    _name = "associated.machines"
    _description = "Associated machines"

    # Submodel fields
    partner_id = fields.Many2one("res.partner", string="Cliente", invisible=True)
    product_id = fields.Many2one(
        "product.product",
        domain="[('type', '=', 'product')]",
        string="Associated machines",
        help="Name of the machine owned by the client",
    )

    # Restrict selection of duplicate records
    _sql_constraints = [
        (
            "unique_partner_product",
            "unique(partner_id, product_id)",
            "You cannot select the same duplicate product record.",
        )
    ]

    # New constraint to check if product_id is not empty
    @api.constrains("product_id")
    def _check_product_id_not_empty(self):
        for record in self:
            if not record.product_id:
                raise ValidationError(
                    _(
                        "The machine associated with that client is not valid, modify or delete the null record."
                    )
                )


class AssociatedBrands(models.Model):
    _name = "associated.brands"
    _description = "Supplier-Associated Brands"

    brand_ids = fields.Many2one(
        "product.brand2", string="Brand", help="Marcas vendidas por el proveedor."
    )
    partner_id = fields.Many2one("res.partner", string="Supplier", ondelete="cascade")

    _sql_constraints = [
        (
            "unique_brand_partner",
            "unique(brand_ids, partner_id)",
            "No se puede seleccionar el mismo registro de Marca duplicado.",
        ),
    ]

    # New constraint to check if product_id is not empty
    @api.constrains("brand_ids")
    def _check_product_id_not_empty(self):
        for record in self:
            if not record.brand_ids:
                raise ValidationError(
                    _(
                        "La marca asociada a ese Proveedor no es válida, modificar o eliminar el registro nulo."
                    )
                )


class ResPartner(models.Model):
    _inherit = "res.partner"

    # Relationship field with the new submodel
    associated_machines = fields.One2many(
        "associated.machines", "partner_id", string="Associated machines"
    )
    associated_brands = fields.One2many(
        "associated.brands", "partner_id", string="brands", view_mode="list"
    )
