"""This module adds a field to the account.bank.statement.line model to store the coupon number"""
import logging

from odoo import fields, models

_logger = logging.getLogger(__name__)


class AccountMoveLine(models.Model):
    """Extends to add field coupon_number"""

    _inherit = "account.move.line"

    payment_id = fields.Many2one("account.payment", string="Payment Group")
    coupon_number = fields.Integer(
        related="payment_id.tiket_number", string="Coupon Number", store=True
    )


class AccountBankStatementLine(models.Model):
    """Extends to add field coupon_number"""

    _inherit = "account.bank.statement.line"

    coupon_number = fields.Char(string="Coupon Number")


class BankRecWidget(models.Model):
    """Extends the bank reconciliation widget to update the state of the payment to Settled when the bank statement is validated."""

    _inherit = "bank.rec.widget"

    st_line_id = fields.Many2one(
        "account.bank.statement.line", string="Línea de Extracto"
    )
    coupon_number = fields.Char(
        string="Número de Cupón", related="st_line_id.coupon_number"
    )

    def button_validate(self):
        """Override to update the state of the payment to Settled when the bank statement is validated."""
        res = super(BankRecWidget, self).button_validate()
        statement_line = self.st_line_id
        if statement_line:
            # Try to find the coupon_number from related records.
            for line in self.line_ids.filtered(
                lambda x: x.source_aml_move_id.payment_id
            ):
                coupon_number = line.source_aml_move_id.payment_id.tiket_number
                if coupon_number:
                    payment = self.env["credit.card.reconciliation"].search(
                        [
                            ("tiket_number", "=", coupon_number),
                            ("state_reconcile", "=", "Pending"),
                        ],
                        limit=1,
                    )
                    if payment:
                        payment.write({"state_reconcile": "Settled"})
                        _logger.info(
                            f"Updated payment (ID: {payment.id}) state to Settled for coupon number: {coupon_number}"
                        )
                    else:
                        _logger.warning(
                            f"No pending payment found for coupon number: {coupon_number}"
                        )
        return res
