"""Test cases for the account.payment model"""
from odoo.exceptions import ValidationError
from odoo.tests.common import TransactionCase


class TestAccountPayment(TransactionCase):
    """Test cases for the account.payment model"""

    def setUp(self):
        """Set up necessary test data"""
        super().setUp()
        # Create a test partner
        self.partner = self.env["res.partner"].create({"name": "Test Partner"})

        # Create a test journal
        self.journal = self.env["account.journal"].create(
            {"name": "Test Journal", "type": "bank", "code": "TEST"}
        )

        # Select any existing currency
        self.currency = self.env["res.currency"].search([], limit=1)

        # Create a test record for account.payment
        self.account_payment = self.env["account.payment"].create(
            {
                "name": "Test Account Payment",
                "payment_date": "2021-01-01",
                "payment_type": "inbound",
                "partner_type": "customer",
                "partner_id": self.partner.id,
                "journal_id": self.journal.id,
                "company_id": self.env.company.id,
                "currency_id": self.currency.id,
                "amount": 100.0,
                "payment_reference": "Test Payment Reference",
                "communication": "Test Communication",
                "state": "draft",
                "card_id": self.env["payment.card"]
                .create({"name": "Test Card"})
                .id,  # Assuming a card object is used
                "card_number": "1234",
                "tiket_number": "5678",
                "lot_number": "91011",
            }
        )

    def test_validate_tar_data(self):
        """Test that duplicate card data raises a ValidationError"""
        # Create a duplicate payment to trigger the validation error
        with self.assertRaises(ValidationError):
            self.env["account.payment"].create(
                {
                    "name": "Duplicate Account Payment",
                    "payment_date": "2021-01-01",
                    "payment_type": "inbound",
                    "partner_type": "customer",
                    "partner_id": self.partner.id,
                    "journal_id": self.journal.id,
                    "company_id": self.env.company.id,
                    "currency_id": self.currency.id,
                    "amount": 100.0,
                    "payment_reference": "Duplicate Payment Reference",
                    "communication": "Duplicate Communication",
                    "state": "draft",
                    "card_id": self.account_payment.card_id.id,  # Same card ID
                    "card_number": "1234",  # Same card number
                    "tiket_number": "5678",  # Same tiket number
                    "lot_number": "91011",  # Same lot number
                }
            )
