"""This module is used to modify the account.payment model to allow creation from the fast payment wizard"""
from odoo import _, api, models
from odoo.exceptions import ValidationError


class AccountPayment(models.Model):
    """Inherit account.payment to allow creation from fast payment wizard"""

    _inherit = "account.payment"

    @api.model_create_multi
    def create(self, vals_list):
        """Modified to allow creation from fast payment wizard"""
        # Check if we're coming from the fast payment wizard
        if self.env.context.get("fast_payment_wizard"):
            return super(AccountPayment, self).create(vals_list)

        recs = super().create(vals_list)
        for rec in recs.filtered(
            lambda x: not x.payment_group_id and not x.is_internal_transfer
        ).with_context(created_automatically=True):
            if not rec.partner_id:
                raise ValidationError(
                    _(
                        "Manual payments should not be created manually but created from Customer Receipts / Supplier Payments menus"
                    )
                )
            rec.payment_group_id = self.env["account.payment.group"].create(
                {
                    "company_id": rec.company_id.id,
                    "partner_type": rec.partner_type,
                    "partner_id": rec.partner_id.id,
                    "payment_date": rec.date,
                    "communication": rec.ref,
                }
            )
            rec.payment_group_id.post()
        return recs
