import xml.etree.cElementTree as ET

from odoo import _, api, fields, models
from odoo.exceptions import ValidationError


class AccountPayment(models.Model):
    """Add necessary fields to apply/match taxes."""

    _inherit = "account.payment"

    comment = fields.Char("coment")


class AccountPaymentGroup(models.Model):
    """ " Add necessary fields to apply/match taxes."""

    _inherit = "account.payment.group"

    retencion_iva = fields.Selection(
        [
            ("imposibilidadret", "Imposibilidad de Retención"),
            ("noaplica", "No Aplica"),
            ("nroregimen", "Nro Regimen"),
        ],
        string="Retención de Iva",
    )
    tabla_iva = fields.Selection(
        [
            (
                "15",
                "15 - USUARIOS DE SISTEMAS DE PAGO ELECTRONICO - INSCRIPTOS EN IVA - PAGOS EFECTUADOS POR CUALQUIER MEDIO EXCEPTO TARJETAS DE CREDITO",
            ),
            (
                "16",
                "16 - USUARIOS DE SISTEMAS DE PAGO ELECTRONICO - INSCRIPTOS EN IVA - PAGOS EFECTUADOS CON TARJETAS DE CREDITO ? SUJETOS ANEXO I RG N? 2854 Y ESTACIONES DE SERVICIO",
            ),
            (
                "17",
                "17 - USUARIOS DE SISTEMAS DE PAGO ELECTRONICO - INSCRIPTOS EN IVA - PAGOS EFECTUADOS CON TARJETAS DE CREDITO ? SUJETOS NO COMPRENDIDOS EN ANEXO I RGN? 2854",
            ),
            (
                "18",
                "18 - USUARIOS DE SISTEMAS DE PAGO ELECTRONICO - NO ACREDITEN CONDICION FRENTE AL IVA",
            ),
            (
                "208",
                "208 - INGRESOS OBTENIDOS POR LA PRODUCCION, REALIZACION Y DISTRIBUCION DE PROGRAMAS, PELICULAS Y/O GRABACIONES - ART. 8 INC. B) Y ART. 28 INC. F) DE INGRESOS OBTENIDOS POR LA PRODUCCION, REALIZACION Y DISTRIBUCION DE PROGRAMAS",
            ),
            (
                "209",
                "209 - INGRESOS OBTENIDOS POR LA COMERCIALIZACION DE ESPACIOS PUBLICITARIOS - ART. 8, INC. B) Y ART. 28 INC. G) DE LA LEY DE IVA.",
            ),
            (
                "210",
                "210 - SERVICIOS DE TRANSPORTE DE PASAJEROS, TERRESTRES, ACUATICOS O AEREOS - ART. 8 INC.B) Y ART. 28 INC. H) DE LA LEY DE IVA.",
            ),
            (
                "211",
                "211 - SERVICIOS DE ASISTENCIA SANITARIA MEDICA Y PARAMEDICA - ART. 8 INC. B) Y ART 28 INC. I) DE LA LEY DE IVA.",
            ),
            (
                "212",
                "212 - LOCACIONES O PRESTACIONES DE SERVICIOS, NO COMPRENDIDAS EN EL ART. 8, INC. B).",
            ),
            (
                "214",
                "214 - COMPRAVENTA DE COSAS MUEBLES Y LOCACIONES COMPRENDIDAS EN EL INCISO COMPRAVENTA DE COSAS MUEBLES Y LOCACIONES COMPRENDIDAS EN EL INCISO C) DEL ARTICULO 3? DE LA LEY DE IVA - ART. 8>, INC. A) C) DEL ARTICULO 3? DE LA LEY DE IVA - ART. 8>, INC. A)",
            ),
            (
                "215",
                "215 - LOCACIONES VINCULADAS A LA OBTENCION DE CARNES Y DESPOJOS COMESTIBLES DE BOVINOS Y FRUTAS - ART. 8, INC. B) Y ART. 28, INC A) DE LA LEY DE IVA.",
            ),
            (
                "216",
                "216 - OBRAS, LOCACIONES Y PRESTACIONES OBTENCION DE ANIMALES VIVOS, FRUTAS, LEGUMBRES Y HORTALIZAS ART.8> INC.B Y ART.28> INC.B DE LA LEY DE IVA.",
            ),
            (
                "217",
                "217 - TRABAJOS SOBRE INMUEBLE AJENO - ART. 8 INC. B) Y ART. 28 INC. C) DE LA LEY DE IVA",
            ),
            (
                "218",
                "218 - INTERESES Y COMISIONES DE PRESTAMOS - ART. 8 INC. B) Y 28 INC. D) DE LA LEY DE IVA.",
            ),
            (
                "219",
                "219 - INGRESOS OBTENIDOS POR LAS EMPRESAS DE SERVICIOS COMPLEMENTARIOS (LEY 22.285) ART. 8 INC. B) Y ART 28 INC. E) DE LA LEY DE IVA",
            ),
            (
                "225",
                "225 - LOCACIONES VINCULADAS A LA OBTENCION DE CARNES Y DESPOJOS COMESTIBLES DE BOVINOS Y FRUTAS- ART. 8C)",
            ),
            (
                "234",
                "234 - IVA-RETENCION-OPERACIONES DE COMPRA-VENTA DE CHEQUES DE PAGO DIFERIDO Y CERTIFICADOS DE AVAL.",
            ),
            ("280", "280 - PAGO DE HONORARIOS PROFESIONALES"),
            (
                "499",
                "499 - FACTURA M-IVA-EMISION DE COMPROBANTES CON DISCRIMINACION DEL GRAVAMEN",
            ),
            ("689", "689 - VALES DE ALMUERZO"),
            ("690", "690 - VALES DE ALMUERZO Y ALIMENTOS"),
            (
                "777",
                "777 - IVA -Regimen de Retencion a Sujetos adheridos al Regimen Simplificado para Pequeños Contribuyentes.",
            ),
            (
                "831",
                "831 - Locacion de obras, locaciones y prestaciones de servicios realizados por empresas de limpieza de edificios, investigacion y seguridad, y recoleccion de residuos domiciliarios.",
            ),
            (
                "263",
                "263 - SISTEMA DE TARJETA DE CREDITO Y/O COMPRA ART. 4 INC. A PUNTO 2",
            ),
            (
                "264",
                "264 - SISTEMA DE TARJETA DE CREDITO Y/O COMPRA ART. 4 INC. A PUNTO 1",
            ),
            (
                "966",
                "966 - FACTURA A CON LEYENDA OPERACION SUJETA A RETENCION - IVA - EMISION DE COMPROBANTES CON DISCRIMINACION DEL GRAVAMEN",
            ),
        ]
    )
    is_show = fields.Boolean("Show", default=False)

    @api.onchange("retencion_iva")
    def _compute_retencion(self):
        """Depending on the value in retencion_iva, it shows or does not show fields necessary for exemption."""
        if self.retencion_iva == "nroregimen":
            self.is_show = True
        else:
            self.is_show = False


class AccountTax(models.Model):
    """Add necessary fields to apply/match taxes."""

    _inherit = "account.tax"

    tipo_txt = fields.Selection(
        [
            ("sifereperc", "Sifere Percepciones Sufridas"),
            ("sifereret", "Sifere Retenciones Sufridas"),
            ("sicoreiva", "Sicore de Iva"),
            ("perceivasufrida", "Percepciones sufridas IVA"),
            ("reteivasufrida", "Retenciones sufridas IVA"),
        ],
        "Tipo Generación TXT",
    )
    siapre_txt = fields.Selection(
        [
            ("tucumanpercsufrida", "Percepción Tucuman Sufrida"),
            ("tucumanretsufrida", "Retención Tucuman Sufrida"),
        ],
        "Tucuman - Aplicativo SIAPRE",
    )
    tucuman_datos_txt = fields.Boolean("TXT Datos", default=False)
    tucuman_retper_txt = fields.Boolean("TXT RetPer", default=False)
    tucuman_ncfact_txt = fields.Boolean("TXT NCFact", default=False)
    state_id = fields.Many2one(
        "res.country.state", "Provincia", domain="[('country_id', '=', country_id)]"
    )
    tipo_list = fields.Selection(
        [
            ("Retenciones SUSS", "Retenciones SUSS"),
            ("Retenciones DREI", "Retenciones DREI"),
            ("Retención Ganancias Sufrida", "Retención Ganancias Sufrida"),
            ("Retención Ganancias Aplicada", "Retención Ganancias Aplicada"),
            ("Retenciones IVA", "Retenciones IVA"),
            ("Retenciones Ingresos brutos", "Retenciones Ingresos brutos"),
            ("Percepción Ganancias Sufrida", "Percepción Ganancias Sufrida"),
            ("Percepción IVA Sufrida", "Percepción IVA Sufrida"),
            (
                "Percepción Ingresos brutos Sufrida",
                "Percepción Ingresos brutos Sufrida",
            ),
        ],
        "Tipo",
    )
    provisional = fields.Boolean("Provisional")

    @api.model
    def get_view(self, view_id=None, view_type="form", **options):
        res = super().get_view(view_id, view_type, **options)
        group_user = self.env.user.has_group(
            "nybble_internal_voucher.group_voucher_user"
        )
        group_manager = self.env.user.has_group(
            "nybble_internal_voucher.group_voucher_admin"
        )
        if not group_manager and (group_user):
            if view_type in ["form"]:
                root = ET.fromstring(res["arch"])
                for el in root.iter("field"):
                    if el.attrib.get("name") in ["provisional"]:
                        el.attrib.update(
                            {
                                "modifiers": '{"invisible": [["id", "!=", 0]]}',
                            }
                        )
                res.update({"arch": ET.tostring(root, encoding="utf8", method="xml")})
        return res

    @api.onchange("state_id")
    def _compute_exemption(self):
        pass

    def create_payment_withholdings(self, payment_group):
        """
        Extendemos y calculamos retencion de Iva para Facturas M.
        :param payment_group:
        :return:
        """
        inv = ""
        for tax in self.filtered(lambda x: x.withholding_type != "none"):
            payment_withholding = self.env["account.payment"].search(
                [
                    ("payment_group_id", "=", payment_group.id),
                    ("tax_withholding_id", "=", tax.id),
                    ("automatic", "=", True),
                ],
                limit=1,
            )
            if tax.withholding_user_error_message and tax.withholding_user_error_domain:
                try:
                    domain = literal_eval(tax.withholding_user_error_domain)
                except Exception as e:
                    raise ValidationError(
                        _(
                            'Could not eval rule domain "%s".\n'
                            "This is what we get:\n%s"
                            % (tax.withholding_user_error_domain, e)
                        )
                    )
                domain.append(("id", "=", payment_group.id))

                if payment_group.search(domain):
                    raise ValidationError(tax.withholding_user_error_message)
            vals = tax.get_withholding_vals(payment_group)
            # we set computed_withholding_amount, hacemos round porque
            # si no puede pasarse un valor con mas decimales del que se ve
            # y terminar dando error en el asiento por debitos y creditos no
            # son iguales, algo parecido hace odoo en el compute_all de taxes
            currency = payment_group.currency_id
            period_withholding_amount = currency.round(
                vals.get("period_withholding_amount", 0.0)
            )
            previous_withholding_amount = currency.round(
                vals.get("previous_withholding_amount")
            )
            # withholding can not be negative
            computed_withholding_amount = max(
                0, (period_withholding_amount - previous_withholding_amount)
            )
            # we copy withholdable_base_amount on base_amount
            # al final vimos con varios clientes que este monto base
            # debe ser la base imponible de lo que se está pagando en este
            # voucher
            if tax.name != "Retenciones de IVA (Fc M)":
                vals["withholding_base_amount"] = vals.get(
                    "withholdable_advanced_amount"
                ) + vals.get("withholdable_invoiced_amount")
                vals["amount"] = computed_withholding_amount
                vals["computed_withholding_amount"] = computed_withholding_amount
                if payment_withholding:
                    payment_withholding.write(vals)
                else:
                    # TODO implementar devoluciones de retenciones
                    payment_method = self.env.ref(
                        "account_withholding." "account_payment_method_out_withholding"
                    )
                    journal = self.env["account.journal"].search(
                        [
                            ("company_id", "=", tax.company_id.id),
                            (
                                "outbound_payment_method_line_ids.payment_method_id",
                                "=",
                                payment_method.id,
                            ),
                            ("type", "in", ["cash", "bank"]),
                        ],
                        limit=1,
                    )
                    if not journal:
                        raise UserError(
                            _("No journal for withholdings found on company %s")
                            % (tax.company_id.name)
                        )
                    vals["journal_id"] = journal.id
                    vals["payment_method_id"] = payment_method.id
                    vals["payment_type"] = "outbound"
                    vals["partner_type"] = payment_group.partner_type
                    vals["partner_id"] = payment_group.partner_id.id
                    payment_withholding = payment_withholding.create(vals)
            else:
                payment_withholding22 = self.env["account.payment"].search(
                    [
                        ("payment_group_id", "=", payment_group.id),
                    ]
                )
                # Factura a la cual tiene asociada pago retencion
                for payment_withholding2 in payment_withholding22:
                    if payment_withholding2.ref:
                        invoice = self.env["account.move"].search(
                            [
                                ("name", "=", payment_withholding2.ref),
                            ]
                        )
                        inv = "M"
                        # total de iva de la factura, el cual tomamos como valor para la retención.
                        if invoice.l10n_latam_document_type_id.name == "FACTURAS M":
                            amount_iva = 0
                            for imp in invoice.line_ids.filtered(
                                lambda r: r.name == "IVA 21%"
                            ):
                                amount_iva = imp.debit
                            vals["withholding_base_amount"] = vals.get(
                                "withholdable_advanced_amount"
                            ) + vals.get("withholdable_invoiced_amount")
                            vals["amount"] = amount_iva
                            vals["computed_withholding_amount"] = amount_iva
                            # vals.pop('comment')
                            if payment_withholding:
                                payment_withholding.write(vals)
                            else:
                                # TODO implementar devoluciones de retenciones
                                payment_method = self.env.ref(
                                    "account_withholding."
                                    "account_payment_method_out_withholding"
                                )
                                journal = self.env["account.journal"].search(
                                    [
                                        ("company_id", "=", tax.company_id.id),
                                        (
                                            "outbound_payment_method_line_ids.payment_method_id",
                                            "=",
                                            payment_method.id,
                                        ),
                                        ("type", "in", ["cash", "bank"]),
                                    ],
                                    limit=1,
                                )
                                if not journal:
                                    raise UserError(
                                        _(
                                            "No journal for withholdings found on company %s"
                                        )
                                        % (tax.company_id.name)
                                    )
                                vals["journal_id"] = journal.id
                                vals["payment_method_id"] = payment_method.id
                                vals["payment_type"] = "outbound"
                                vals["partner_type"] = payment_group.partner_type
                                vals["partner_id"] = payment_group.partner_id.id
                                payment_withholding = payment_withholding.create(vals)
                    else:
                        # Calcula retencion sobre Facturas M, al efectuar pago con multiples FM asociadas. payment_group.to_pay_move_line_ids 306034
                        for inv in payment_group.to_pay_move_line_ids.move_id:
                            invoice = self.env["account.move"].search(
                                [
                                    (
                                        "l10n_latam_document_type_id.name",
                                        "=",
                                        "FACTURAS M",
                                    ),
                                    ("id", "=", inv.id),
                                ]
                            )
                            if invoice:
                                amount_iva = 0
                                for imp in invoice.line_ids.filtered(
                                    lambda r: r.name == "IVA 21%"
                                ):
                                    amount_iva += imp.debit
                                vals["withholding_base_amount"] = vals.get(
                                    "withholdable_advanced_amount"
                                ) + vals.get("withholdable_invoiced_amount")
                                vals["amount"] = amount_iva
                                vals["computed_withholding_amount"] = amount_iva
                                # vals.pop('comment')
                                if payment_withholding:
                                    payment_withholding.write(vals)
                                else:
                                    # TODO implementar devoluciones de retenciones
                                    payment_method = self.env.ref(
                                        "account_withholding."
                                        "account_payment_method_out_withholding"
                                    )
                                    journal = self.env["account.journal"].search(
                                        [
                                            ("company_id", "=", tax.company_id.id),
                                            (
                                                "outbound_payment_method_line_ids.payment_method_id",
                                                "=",
                                                payment_method.id,
                                            ),
                                            ("type", "in", ["cash", "bank"]),
                                        ],
                                        limit=1,
                                    )
                                    if not journal:
                                        raise UserError(
                                            _(
                                                "No journal for withholdings found on company %s"
                                            )
                                            % (tax.company_id.name)
                                        )
                                    vals["journal_id"] = journal.id
                                    vals["payment_method_id"] = payment_method.id
                                    vals["payment_type"] = "outbound"
                                    vals["partner_type"] = payment_group.partner_type
                                    vals["partner_id"] = payment_group.partner_id.id
                                    payment_withholding = payment_withholding.create(
                                        vals
                                    )

        return True


class Country(models.Model):
    """Agrega campo country_code_afip en vista tree para Provincias."""

    _inherit = "res.country.state"

    country_code_afip = fields.Char("Jurisdicción")


class Followers(models.Model):
    _inherit = "mail.followers"

    @api.model
    def create(self, vals):
        if "res_model" in vals and "res_id" in vals and "partner_id" in vals:
            dups = self.env["mail.followers"].search(
                [
                    ("res_model", "=", vals.get("res_model")),
                    ("res_id", "=", vals.get("res_id")),
                    ("partner_id", "=", vals.get("partner_id")),
                ]
            )
            if len(dups):
                for p in dups:
                    p.unlink()
        res = super(Followers, self).create(vals)
        return res
