# Part of Odoo. See LICENSE file for full copyright and licensing details.

from odoo import api, fields, models


class ResPartnerArbaAlicuot(models.Model):
    """Agrega campos para   validar exencion en caso de impuestos para Santa Fe."""

    _inherit = "res.partner.arba_alicuot"

    def _year_selection(self):
        """Despliega años comenzando desde el 2000 a 2060 ."""
        year = 2000
        year_list = []
        while year != 2060:
            year_list.append((str(year), str(year)))
            year += 1
        return year_list

    @api.onchange("type_exemption")
    def _compute_exemption(self):
        """Dependiendo del valor en type_exemption muestra o no campos necesarios para exención."""
        if self.type_exemption != "unexencion":
            self.is_show = True
        else:
            self.is_show = False

    @api.onchange("tag_id")
    def _compute_tag(self):
        """Dependiendo del valor en tag_id, si es Santa Fe, muestra para ingresar exención."""
        if self.tag_id.name == "Jur: 921 - Santa Fe":
            self.is_active = True
        else:
            self.is_active = False
            self.is_show = False

    type_exemption = fields.Selection(
        [
            ("unexencion", "Sin exención"),
            ("cert_unretencion", "Certificado de no retención"),
            ("cert_exencion", "Certificado de exención"),
            ("cert_unpercepcion", "Certificado de no percepción"),
            ("articule3", "Art 3ro Resol 015/97 y modificaciones"),
        ],
        "Tipo de exención",
    )
    year_exemption = fields.Selection(
        _year_selection,
        string="Año exención",
        default="2022",  # as a default value it would be 2022
    )
    number_certificate = fields.Char("Número de certificado propio")
    is_show = fields.Boolean("Show", default=False)
    tag_id = fields.Many2one(
        "account.account.tag",
        domain=[("applicability", "=", "taxes")],
        required=True,
        change_default=True,
    )
    is_active = fields.Boolean("Active", default=False)


class ResPartner(models.Model):
    """Agrega campos para validar exencion en caso de impuestos para Santa Fe."""

    _inherit = "res.partner"

    retencion_iva = fields.Selection(
        [
            ("imposibilidadret", "Imposibilidad de Retención"),
            ("noaplica", "No Aplica"),
            ("nroregimen", "Nro Regimen"),
        ],
        string="Retención de Iva",
    )
    tabla_iva = fields.Selection(
        [
            (
                "15",
                "15 - USUARIOS DE SISTEMAS DE PAGO ELECTRONICO - INSCRIPTOS EN IVA - PAGOS EFECTUADOS POR CUALQUIER MEDIO EXCEPTO TARJETAS DE CREDITO",
            ),
            (
                "16",
                "16 - USUARIOS DE SISTEMAS DE PAGO ELECTRONICO - INSCRIPTOS EN IVA - PAGOS EFECTUADOS CON TARJETAS DE CREDITO ? SUJETOS ANEXO I RG N? 2854 Y ESTACIONES DE SERVICIO",
            ),
            (
                "17",
                "17 - USUARIOS DE SISTEMAS DE PAGO ELECTRONICO - INSCRIPTOS EN IVA - PAGOS EFECTUADOS CON TARJETAS DE CREDITO ? SUJETOS NO COMPRENDIDOS EN ANEXO I RGN? 2854",
            ),
            (
                "18",
                "18 - USUARIOS DE SISTEMAS DE PAGO ELECTRONICO - NO ACREDITEN CONDICION FRENTE AL IVA",
            ),
            (
                "208",
                "208 - INGRESOS OBTENIDOS POR LA PRODUCCION, REALIZACION Y DISTRIBUCION DE PROGRAMAS, PELICULAS Y/O GRABACIONES - ART. 8 INC. B) Y ART. 28 INC. F) DE INGRESOS OBTENIDOS POR LA PRODUCCION, REALIZACION Y DISTRIBUCION DE PROGRAMAS",
            ),
            (
                "209",
                "209 - INGRESOS OBTENIDOS POR LA COMERCIALIZACION DE ESPACIOS PUBLICITARIOS - ART. 8, INC. B) Y ART. 28 INC. G) DE LA LEY DE IVA.",
            ),
            (
                "210",
                "210 - SERVICIOS DE TRANSPORTE DE PASAJEROS, TERRESTRES, ACUATICOS O AEREOS - ART. 8 INC.B) Y ART. 28 INC. H) DE LA LEY DE IVA.",
            ),
            (
                "211",
                "211 - SERVICIOS DE ASISTENCIA SANITARIA MEDICA Y PARAMEDICA - ART. 8 INC. B) Y ART 28 INC. I) DE LA LEY DE IVA.",
            ),
            (
                "212",
                "212 - LOCACIONES O PRESTACIONES DE SERVICIOS, NO COMPRENDIDAS EN EL ART. 8, INC. B).",
            ),
            (
                "214",
                "214 - COMPRAVENTA DE COSAS MUEBLES Y LOCACIONES COMPRENDIDAS EN EL INCISO COMPRAVENTA DE COSAS MUEBLES Y LOCACIONES COMPRENDIDAS EN EL INCISO C) DEL ARTICULO 3? DE LA LEY DE IVA - ART. 8>, INC. A) C) DEL ARTICULO 3? DE LA LEY DE IVA - ART. 8>, INC. A)",
            ),
            (
                "215",
                "215 - LOCACIONES VINCULADAS A LA OBTENCION DE CARNES Y DESPOJOS COMESTIBLES DE BOVINOS Y FRUTAS - ART. 8, INC. B) Y ART. 28, INC A) DE LA LEY DE IVA.",
            ),
            (
                "216",
                "216 - OBRAS, LOCACIONES Y PRESTACIONES OBTENCION DE ANIMALES VIVOS, FRUTAS, LEGUMBRES Y HORTALIZAS ART.8> INC.B Y ART.28> INC.B DE LA LEY DE IVA.",
            ),
            (
                "217",
                "217 - TRABAJOS SOBRE INMUEBLE AJENO - ART. 8 INC. B) Y ART. 28 INC. C) DE LA LEY DE IVA",
            ),
            (
                "218",
                "218 - INTERESES Y COMISIONES DE PRESTAMOS - ART. 8 INC. B) Y 28 INC. D) DE LA LEY DE IVA.",
            ),
            (
                "219",
                "219 - INGRESOS OBTENIDOS POR LAS EMPRESAS DE SERVICIOS COMPLEMENTARIOS (LEY 22.285) ART. 8 INC. B) Y ART 28 INC. E) DE LA LEY DE IVA",
            ),
            (
                "225",
                "225 - LOCACIONES VINCULADAS A LA OBTENCION DE CARNES Y DESPOJOS COMESTIBLES DE BOVINOS Y FRUTAS- ART. 8C)",
            ),
            (
                "234",
                "234 - IVA-RETENCION-OPERACIONES DE COMPRA-VENTA DE CHEQUES DE PAGO DIFERIDO Y CERTIFICADOS DE AVAL.",
            ),
            ("280", "280 - PAGO DE HONORARIOS PROFESIONALES"),
            (
                "499",
                "499 - FACTURA M-IVA-EMISION DE COMPROBANTES CON DISCRIMINACION DEL GRAVAMEN",
            ),
            ("689", "689 - VALES DE ALMUERZO"),
            ("690", "690 - VALES DE ALMUERZO Y ALIMENTOS"),
            (
                "777",
                "777 - IVA -Regimen de Retencion a Sujetos adheridos al Regimen Simplificado para Pequeños Contribuyentes.",
            ),
            (
                "831",
                "831 - Locacion de obras, locaciones y prestaciones de servicios realizados por empresas de limpieza de edificios, investigacion y seguridad, y recoleccion de residuos domiciliarios.",
            ),
            (
                "263",
                "263 - SISTEMA DE TARJETA DE CREDITO Y/O COMPRA ART. 4 INC. A PUNTO 2",
            ),
            (
                "264",
                "264 - SISTEMA DE TARJETA DE CREDITO Y/O COMPRA ART. 4 INC. A PUNTO 1",
            ),
            (
                "966",
                "966 - FACTURA A CON LEYENDA OPERACION SUJETA A RETENCION - IVA - EMISION DE COMPROBANTES CON DISCRIMINACION DEL GRAVAMEN",
            ),
        ]
    )
    is_show = fields.Boolean("Show", default=False)
    percepcion_iva = fields.Selection(
        [
            ("imposibilidadret", "Imposibilidad de Retención"),
            ("noaplica", "No Aplica"),
            ("nroregimen", "Nro Regimen"),
        ],
        string="Percepción de Iva",
    )
    tabla_percepcion_iva = fields.Many2one(
        "afip.tabla_percepciones.iva", "Tabla Percepciones de Iva"
    )
    is_show2 = fields.Boolean("Show", default=False)

    @api.onchange("retencion_iva")
    def _compute_retencion_iva(self):
        """Dependiendo del valor en type_exemption muestra o no campos necesarios para exención."""
        if self.retencion_iva == "nroregimen":
            self.is_show = True
        else:
            self.is_show = False

    @api.onchange("percepcion_iva")
    def _compute_percepcion_iva(self):
        """Dependiendo del valor en type_exemption muestra o no campos necesarios para exención."""
        if self.percepcion_iva == "nroregimen":
            self.is_show2 = True
        else:
            self.is_show2 = False
