"""Create a wizard to generate tax files by period"""
import base64
import logging
import math

from odoo import _, api, fields, models
from odoo.exceptions import UserError

_logger = logging.getLogger(__name__)


class WzdImpuestosXeriodo(models.TransientModel):
    """Wizard for generating tax files by period"""

    _name = "nybble.wzd_impuestos_x_periodo"
    _description = "Wizard para generar archivos de impuestos por período"

    name = fields.Char("Descripción", required=True)
    fecha_desde = fields.Date("Fecha Desde", required=True)
    fecha_hasta = fields.Date("Fecha Hasta", required=True)
    tipo_txt = fields.Selection(
        [
            ("sicore", "Sicore"),
            ("sifereperc", "Sifere Percepciones Sufridas"),
            ("sifereret", "Sifere Retenciones Sufridas"),
            ("sicoreiva", "Sicore de Iva"),
            ("perceivasufrida", "Percepciones sufridas IVA"),
            ("reteivasufrida", "Retenciones sufridas IVA"),
        ],
        "Tipo",
    )
    export_data = fields.Text(string="Contenido archivo")
    export_file = fields.Binary("Archivo")
    export_filename = fields.Char("Archivo nombre")
    NAME_FILE = None

    @api.model
    def _tax_piibbarbaa(self, apuntes):
        """Create TXT, for presenting IIBB ARBA Applied"""
        nombre = "PIIBBARBAA.txt"
        global NAME_FILE
        NAME_FILE = nombre
        windows_line_ending = "\r" + "\n"
        string = ""
        move_id = 0
        for apunte in apuntes.filtered(
            lambda r: r.move_id.state == "posted"
            and r.move_id.l10n_latam_document_type_id.l10n_ar_letter
        ):
            # Solo comprobantes Fiscales if apunte.move_id.name[:1] != 'C':
            amount_perc = 0
            monto3 = ""
            for imp in apunte.move_id.line_ids.filtered(
                lambda r: r.tax_ids.filtered(lambda i: i.tipo_txt == "piibbarbaa")
            ):
                if apunte.move_id.name[:2] != "NC":
                    amount_perc = imp.credit
                else:
                    amount_perc = imp.debit
            parte_decimal, parte_entera = math.modf(amount_perc)
            parte_entera = str(parte_entera).split(".")[0]
            parte_decimal = (str(round(parte_decimal, 2)).split(".")[1][:2]).ljust(
                2, "0"
            )
            monto3 += (parte_entera + "," + parte_decimal).zfill(11)
            if monto3 != "00000000,00" and apunte.move_id != move_id:
                # 1er campo: cuit separado por guiones
                string += (
                    apunte.partner_id.vat[:2]
                    + "-"
                    + apunte.partner_id.vat[2:-1].zfill(8)
                    + "-"
                    + apunte.partner_id.vat[-1:]
                )
                # 2do campo: fecha formato dd/mm/yyyy
                d = str(apunte.date.day).zfill(2)
                m = str(apunte.date.month).zfill(2)
                string += str(d) + "/" + str(m) + "/" + str(apunte.date.year)
                # 3er campo: tipo de comprobante (F=Factura, R=Recibo, C=Nota Crédito, D =Nota Debito, V=Nota de Venta)
                # 4to campo: letra de comprobante (ABC o "blanco")
                if (
                    apunte.move_id.l10n_latam_document_type_id.doc_code_prefix[0:2]
                    == "FA"
                ):
                    comprob = (
                        "F" + apunte.move_id.l10n_latam_document_type_id.l10n_ar_letter
                    )  # _id.name
                elif (
                    apunte.move_id.l10n_latam_document_type_id.doc_code_prefix[0:2]
                    == "ND"
                ):
                    comprob = (
                        "D" + apunte.move_id.l10n_latam_document_type_id.l10n_ar_letter
                    )
                elif (
                    apunte.move_id.l10n_latam_document_type_id.doc_code_prefix[0:2]
                    == "NC"
                ):
                    comprob = (
                        "C" + apunte.move_id.l10n_latam_document_type_id.l10n_ar_letter
                    )
                else:
                    comprob = "O "
                string += comprob
                # 5to campo: numero de sucursal, 4 digitos (nro pto vta)
                # 6to campo: numero de emision 8 digitos (num factura) apunte.move_id.l10n_latam_document_number
                comp = str(apunte.move_id.name)[5:]
                numcompro = str(comp).split("-")
                ptoventa = str(apunte.move_id.l10n_latam_document_number)[1:5]
                numcom = str(numcompro[1].zfill(8))
                numcompro = str(ptoventa + numcom)
                string += numcompro
                # 7mo campo: base imponible 12 digitos con la coma
                parte_decimal, parte_entera = math.modf(apunte.move_id.amount_untaxed)
                parte_entera = str(parte_entera).split(".")[0]
                parte_decimal = (str(round(parte_decimal, 2)).split(".")[1][:2]).ljust(
                    2, "0"
                )
                if comprob[:1] != "C":
                    string += (parte_entera + "," + parte_decimal).zfill(12)
                else:
                    string += "-" + (parte_entera + "," + parte_decimal).zfill(11)
                if comprob[:1] != "C":
                    parte_decimal, parte_entera = math.modf(apunte.credit)
                else:
                    parte_decimal, parte_entera = math.modf(apunte.debit)
                # 8mo campo: importe de la percepción 12 digitos con la coma
                amount_perc = 0
                for imp in apunte.move_id.line_ids.filtered(
                    lambda r: r.tax_ids.filtered(lambda i: i.tipo_txt == "piibbarbaa")
                ):
                    if apunte.move_id.name[:2] != "NC":
                        amount_perc = imp.credit
                    else:
                        amount_perc = imp.debit
                parte_decimal, parte_entera = math.modf(amount_perc)
                parte_entera = str(parte_entera).split(".")[0]
                parte_decimal = (str(round(parte_decimal, 2)).split(".")[1][:2]).ljust(
                    2, "0"
                )
                if comprob[:1] != "C":
                    string += (parte_entera + "," + parte_decimal).zfill(11)
                else:
                    string += "-" + (parte_entera + "," + parte_decimal).zfill(10)
                string += "A"
                string += windows_line_ending
                move_id = apunte.move_id
        return string

    @api.model
    def _tax_riibbarbaa(self, apuntes):
        """Create TXT, for presenting IIBB ARBA Retained"""
        nombre = "RIIBBARBAA.txt"
        global NAME_FILE
        NAME_FILE = nombre
        windows_line_ending = "\r" + "\n"
        string = ""
        for apunte in apuntes:
            # Solo comprobantes Fiscales
            # 1er campo: cuit separado por guiones
            string += (
                apunte.partner_id.vat[:2]
                + "-"
                + apunte.partner_id.vat[2:-1].zfill(8)
                + "-"
                + apunte.partner_id.vat[-1:]
            )
            # 2do campo: fecha formato dd/mm/yyyy
            d = str(apunte.date.day).zfill(2)
            m = str(apunte.date.month).zfill(2)
            string += str(d) + "/" + str(m) + "/" + str(apunte.date.year)
            # 3er campo: 4 digitos para numero de sucursal 0001 ???? Chequear
            string += "0001"
            # 4to campo: 8 digitos para numero de retención ???? Chequear
            string += apunte.withholding_number.zfill(8)
            # 5to campo: importe de la retención 11 digitos incluida la coma
            parte_decimal, parte_entera = math.modf(apunte.computed_withholding_amount)
            parte_entera = str(parte_entera).split(".")[0]
            parte_decimal = (str(round(parte_decimal, 2)).split(".")[1][:2]).ljust(
                2, "0"
            )
            string += (parte_entera + "," + parte_decimal).zfill(11)
            # 5to campo: Letra "A" ?????
            string += "A"
            string += windows_line_ending
        return string

    @api.model
    def _tax_misionesret(self, apuntes):
        """Genera TXT, para presentar impuesto Retención Misiones"""
        nombre = "MISIONESRET.txt"
        global NAME_FILE
        NAME_FILE = nombre
        windows_line_ending = "\r" + "\n"
        string = ""
        for apunte in apuntes:
            imp_misones = apunte.computed_withholding_amount
            parte_decimala, parte_enteraa = math.modf(imp_misones)
            parte_enteraa = str(parte_enteraa).split(".")[0]
            parte_decimala = (str(round(parte_decimala, 2)).split(".")[1][:2]).ljust(
                2, "0"
            )
            alicuot = parte_enteraa + "." + parte_decimala
            if alicuot != "0.00":
                # Solo comprobantes Fiscales
                #    8) + '-' + apunte.partner_id.vat[-1:]
                # 1ro campo: fecha formato dd-mm-yyyy
                d = str(apunte.date.day).zfill(2)
                m = str(apunte.date.month).zfill(2)
                string += str(d) + "-" + str(m) + "-" + str(apunte.date.year)
                # 2do campo:  6 digitos para numero de retención ???? Chequear
                if len(apunte.partner_id.name) < 6:
                    string += "," + apunte.withholding_number
                else:
                    string += "," + apunte.withholding_number.zfill(6)
                # 3ro: Partner
                if len(apunte.partner_id.name) < 100:
                    string += "," + apunte.partner_id.name
                else:
                    string += "," + apunte.partner_id.name.zfill(100)
                # 4to campo: Domicilio
                if apunte.partner_id.street != False:
                    if len(apunte.partner_id.name) < 100:
                        string += "," + apunte.partner_id.street
                    else:
                        string += "," + apunte.partner_id.street.zfill(100)
                else:
                    string += "," + "00000"
                # 5to campo: cuit separado por guiones
                string += (
                    ","
                    + apunte.partner_id.vat[:2]
                    + "-"
                    + apunte.partner_id.vat[2:-1].zfill(8)
                    + "-"
                    + apunte.partner_id.vat[-1:]
                )
                # 5to campo: importe de la retención 17 digitos incluida la coma.   apunte.withholding_base_amount
                parte_decimal, parte_entera = math.modf(apunte.withholding_base_amount)
                parte_entera = str(parte_entera).split(".")[0]
                parte_decimal = (str(round(parte_decimal, 2)).split(".")[1][:2]).ljust(
                    2, "0"
                )
                imp_ret = parte_entera + "." + parte_decimal
                if len(imp_ret) < 17:
                    string += "," + imp_ret
                else:
                    string += "," + imp_ret.zfill(17)
                # 6to campo: Alicuota
                alic = 0
                for (
                    arba_alicuot_id
                ) in apunte.payment_group_id.partner_id.arba_alicuot_ids.filtered(
                    lambda v: v.tag_id.jurisdiction_code == "914"
                    and v.alicuota_retencion != "0.00"
                    and v.from_date
                    and v.to_date
                    and v.from_date <= self.fecha_desde
                    and v.to_date >= self.fecha_hasta
                ):
                    alic = arba_alicuot_id.alicuota_retencion
                parte_decimala, parte_enteraa = math.modf(alic)
                parte_enteraa = str(parte_enteraa).split(".")[0]
                parte_decimala = (
                    str(round(parte_decimala, 2)).split(".")[1][:2]
                ).ljust(2, "0")
                alicuot = parte_enteraa + "." + parte_decimala
                if len(alicuot) < 17:
                    string += "," + alicuot.zfill(5)
                else:
                    string += "," + alicuot.zfill(5)
                string += windows_line_ending
        return string

    @api.model
    def _tax_piibbcabaanc(self, apuntes):
        """Genera TXT, para presentar impuesto Percepción IIBB CABA Aplicada (Notas de Crédito)"""
        nombre = "PIIBBCABAANCRED.txt"
        global NAME_FILE
        NAME_FILE = nombre
        windows_line_ending = "\r" + "\n"
        string = ""
        fecha_desde = self.fecha_desde
        fecha_hasta = self.fecha_hasta
        # CABAAA Operación 2
        for apunte in apuntes.filtered(
            lambda r: r.move_id.l10n_latam_document_type_id.doc_code_prefix[0:2] == "NC"
            and r.move_id.reversed_entry_id != False
            and r.move_id.l10n_latam_document_type_id.l10n_ar_letter == "A"
        ):
            # Solo comprobantes Fiscales
            tipo_operacion = "2"
            imp_caba2 = 0.0
            if tipo_operacion == "2":
                for impuesto2 in apunte.move_id.line_ids.filtered(
                    lambda r: r.tax_ids.filtered(lambda i: i.tipo_txt == "piibbcabaa")
                ):
                    imp_caba2 += impuesto2.debit
            parte_decimal2, parte_entera2 = math.modf(imp_caba2)
            parte_entera2 = str(parte_entera2).split(".")[0]
            parte_decimal2 = (str(round(parte_decimal2, 2)).split(".")[1][:2]).ljust(
                2, "0"
            )
            monto3 = (parte_entera2 + "," + parte_decimal2).zfill(16)
            if monto3 != "0000000000000,00":
                # 1  campo: largo 01 tipo operación (1:retencion 2:percepcion)
                string += tipo_operacion
                # 2  campo: largo 12 nro nota de credito
                if tipo_operacion == "2":
                    num_cbte = apunte.move_id.l10n_latam_document_number.replace(
                        "-", ""
                    ).zfill(12)
                else:
                    num_cbte = "NNNNNNNNNNNNNNNN"
                if len(num_cbte) == 13:
                    num_cbte = num_cbte[1:13]
                string += num_cbte
                # 3  campo: largo 10 fech a de nota de credito
                # string+=apunte.date[-2:]+'/'+apunte.date[5:-3]+'/'+apunte.date[:4]
                d = str(apunte.date.day).zfill(2)
                m = str(apunte.date.month).zfill(2)
                string += str(d) + "/" + str(m) + "/" + str(apunte.date.year)
                # 4 campo: largo 16 Monto nota de credito
                if tipo_operacion == "2":
                    parte_decimal, parte_entera = math.modf(
                        apunte.move_id.amount_untaxed
                    )
                    parte_entera = str(parte_entera).split(".")[0]
                    parte_decimal = (
                        str(round(parte_decimal, 2)).split(".")[1][:2]
                    ).ljust(2, "0")
                    monto = (parte_entera + "," + parte_decimal).zfill(16)
                else:
                    monto = "NNNNNNNNNNNNNNNN"
                string += monto
                # 5  campo: largo 16 nro de cert propio (blancos o nro cert)
                string += " " * 16
                # 6  campo: largo 02 tipo cbte origen de la ret (si tipo op =1 :01 fac 02 ndeb 03 ordenpago 04 boleta de dep 05 liq de pago 06 cert de obra 07 recibo 08 con de loc de serv 09 otro comb si tipo op =2 01 factura 09 otro comp)
                # 7  campo: largo 01 letra del comprobante (A,M,B,C o espacio blanco)
                if tipo_operacion == "2":
                    comprob = (
                        "01" + apunte.move_id.l10n_latam_document_type_id.l10n_ar_letter
                    )
                else:
                    comprob = "NNN"
                string += comprob
                # 8  campo: largo 16 nro de comprobante
                if tipo_operacion == "2":
                    num_cbte = apunte.move_id.reversed_entry_id.l10n_latam_document_number.replace(
                        "-", ""
                    ).zfill(
                        16
                    )
                else:
                    num_cbte = "NNNNNNNNNNNNNNNN"
                string += num_cbte
                # 9 campo: largo 11 nro de documento del retenido
                string += apunte.partner_id.vat.zfill(11)
                # 10 campo: largo 03 codigo de norma (segun tipo de operacion)
                string += "029"
                # 11 campo: largo 10 fecha de retencion/percepcion
                # string+=apunte.invoice_id.refund_invoice_id.date_invoice[-2:]+'/'+apunte.invoice_id.refund_invoice_id.date_invoice[5:-3]+'/'+apunte.invoice_id.refund_invoice_id.date_invoice[:4]
                d = str(apunte.move_id.reversed_entry_id.invoice_date.day).zfill(2)
                m = str(apunte.move_id.reversed_entry_id.invoice_date.month).zfill(2)
                y = str(apunte.move_id.reversed_entry_id.invoice_date.year).zfill(4)
                string += d + "/" + m + "/" + y
                # 12 campo: largo 16 REt/percep a deducir
                imp_caba = 0.0
                if tipo_operacion == "2":
                    for impuesto in apunte.move_id.line_ids.filtered(
                        lambda r: r.tax_ids.filtered(
                            lambda i: i.tipo_txt == "piibbcabaa"
                        )
                    ):
                        imp_caba += impuesto.debit
                parte_decimal, parte_entera = math.modf(imp_caba)
                parte_entera = str(parte_entera).split(".")[0]
                parte_decimal = (str(round(parte_decimal, 2)).split(".")[1][:2]).ljust(
                    2, "0"
                )
                monto = (parte_entera + "," + parte_decimal).zfill(16)
                string += monto
                # 13 campo: largo 05 Alícuota .tag_id.jurisdiction_code == 901 and r.move_id.partner_id.arba_alicuot_ids.tag_id.jurisdiction_code == 902 and r.move_id.partner_id.arba_alicuot_ids.from_date == self.fecha_desde and r.move_id.partner_id.arba_alicuot_ids.to_date == self.fecha_hasta and r.move_id.partner_id.arba_alicuot_ids.alicuota_percepcion != '0.00')
                alicuota = 0.0
                for alic in apunte.move_id.partner_id.arba_alicuot_ids.filtered(
                    lambda v: v.from_date <= fecha_desde
                    and v.to_date <= fecha_hasta
                    and v.tag_id.jurisdiction_code == "901"
                ):
                    if alic.alicuota_percepcion != 0.0:
                        alicuota = float(alic.alicuota_percepcion)
                    else:
                        alicuota = 0.0
                alicuota = str(alicuota).replace(",", ".")
                parte_decimal, parte_entera = math.modf(float(alicuota))
                parte_entera = str(parte_entera).split(".")[0]
                parte_decimal = (str(round(parte_decimal, 2)).split(".")[1][:2]).ljust(
                    2, "0"
                )
                monto = (parte_entera + "," + parte_decimal).zfill(5)
                string += monto
                string += windows_line_ending
        string = string.encode("utf-8")
        return string

    @api.model
    def _tax_priibbcabaa(self, apuntes2):
        """Genera TXT, para presentar impuesto Percepción/Retención IIBB CABA Aplicada"""
        nombre = "PRIIBBCABAA.txt"
        global NAME_FILE
        NAME_FILE = nombre
        windows_line_ending = "\r" + "\n"
        string = ""
        imp_caba = 0.00
        apunte = ""
        for apunte in apuntes2:
            if apunte.tax_line_id.type_tax_use == "supplier":
                tipo_operacion = "1"
            elif apunte.tax_line_id.type_tax_use == "sale":
                tipo_operacion = "2"
            imp_caba2 = 0.0
            if tipo_operacion == "2":
                for impuesto2 in apunte.move_id.line_ids.filtered(
                    lambda r: r.tax_ids.filtered(lambda i: i.tipo_txt == "piibbcabaa")
                ):
                    imp_caba2 += impuesto2.credit
            elif tipo_operacion == "1":
                for impuesto2 in apunte.payment_id:
                    imp_caba2 += impuesto2.amount
            parte_decimal2, parte_entera2 = math.modf(imp_caba2)
            parte_entera2 = str(parte_entera2).split(".")[0]
            parte_decimal2 = (str(round(parte_decimal2, 2)).split(".")[1][:2]).ljust(
                2, "0"
            )
            monto3 = (parte_entera2 + "," + parte_decimal2).zfill(16)
            if monto3 != "0000000000000,00":
                # 1  campo: largo 01 tipo operación (1:retencion 2:percepcion)
                if apunte.tax_line_id.type_tax_use == "supplier":
                    tipo_operacion = "1"
                elif apunte.tax_line_id.type_tax_use == "sale":
                    tipo_operacion = "2"
                    if (
                        apunte.move_id.l10n_latam_document_type_id.doc_code_prefix[0:2]
                        == "NC"
                    ):
                        continue
                else:
                    tipo_operacion = "0"
                if tipo_operacion == "1":
                    payment = apunte.payment_id
                    pay = apunte.payment_id
                    string += tipo_operacion
                    # 2  campo: largo 03 codigo de norma (segun tipo de operacion)
                    string += "029"
                    # 3  campo: largo 10 fecha de retencion/percepcion
                    d = str(pay.payment_group_id.payment_date.day).zfill(2)
                    m = str(pay.payment_group_id.payment_date.month).zfill(2)
                    string += (
                        str(d)
                        + "/"
                        + str(m)
                        + "/"
                        + str(pay.payment_group_id.payment_date.year)
                    )
                    # 4  campo: largo 02 tipo cbte origen de la ret (si tipo op =1 :01 fac 02 ndeb 03 ordenpago 04 boleta de dep 05 liq de pago 06 cert de obra 07 recibo 08 con de loc de serv 09 otro comb si tipo op =2 01 factura 09 otro comp)
                    # 5  campo: largo 01 letra del comprobante (A,M,B,C o espacio blanco)
                    comprob = "03 "
                    string += comprob
                    # 6  campo: largo 16 nro de comprobante
                    num_cbte = pay.payment_group_id.name.zfill(16)
                    string += num_cbte
                    # 7  campo: largo 10 fecha
                    d = str(pay.payment_group_id.payment_date.day).zfill(2)
                    m = str(pay.payment_group_id.payment_date.month).zfill(2)
                    string += (
                        str(d)
                        + "/"
                        + str(m)
                        + "/"
                        + str(pay.payment_group_id.payment_date.year)
                    )
                    # DIVIDO EL CODIGO tipo de operación
                    # CALCULO TOTALES(Monto,Importe otros conceptos,Importe IVA,Monto Sujeto a Retención/ Percepción,Alícuota,Retención/Percepción Practicada)
                    imp_caba = pay.computed_withholding_amount
                    # Calculo Alícuota payment.payment_group_id.matched_move_line_ids.move_id.amount_untaxed
                    alicuota = round((imp_caba / pay.amount * 100), 2)
                    # Calculo monto sujeto
                    # 8  campo: largo 16 monto del comprobante
                    # string += monto_total
                    monto_sujeto = str(payment.withholding_base_amount)
                    parte_decimal, parte_entera = math.modf(float(monto_sujeto))
                    parte_entera = str(parte_entera).split(".")[0]
                    parte_decimal = (
                        str(round(parte_decimal, 2)).split(".")[1][:2]
                    ).ljust(2, "0")
                    string += (parte_entera + "," + parte_decimal).zfill(16)
                    ######################################################################
                    # 9  campo: largo 16 nro de cert propio (blancos o nro cert)
                    name = pay.payment_group_id.name.zfill(16)
                    if len(name) <= 16:
                        string += str(name)
                    else:
                        string += str(name)[0:16]
                    # 10 campo: largo 01 tipo de doc del retenido (1: CDI 2: CUIL 3:CUIT)
                    if pay.partner_id.l10n_latam_identification_type_id.name == "CDI":
                        tipo_doc = "1"
                    elif (
                        pay.partner_id.l10n_latam_identification_type_id.name == "CUIL"
                    ):
                        tipo_doc = "2"
                    elif (
                        pay.partner_id.l10n_latam_identification_type_id.name == "CUIT"
                    ):
                        tipo_doc = "3"
                    else:
                        tipo_doc = "N"
                    string += tipo_doc
                    # 11 campo: largo 11 nro de documento del retenido
                    string += pay.partner_id.vat.zfill(11)
                    # 12 campo: largo 01 situacion IB del retenido (1:local 2:convenio multilateral 4:no inscripto 5: reg. simplificado) si tipo doc 1 o 2 =4
                    if tipo_doc in ["1", "4"]:
                        string += "4"
                    else:
                        string += "4"
                    # 13 campo: largo 11 nro inscripcion ID del Retenido  Si Situación IB del Retenido=4 : 00000000000
                    string += "0" * 11
                    # 14 campo: largo 01 situacion frente al iva del retenido (1 -RI 2- Exento 3-monotributo)
                    if pay.partner_id.l10n_ar_afip_responsibility_type_id.id in [
                        1,
                        2,
                        12,
                    ]:
                        string += "1"
                    elif pay.partner_id.l10n_ar_afip_responsibility_type_id.id in [5]:
                        string += "2"
                    elif pay.partner_id.l10n_ar_afip_responsibility_type_id.id in [
                        7,
                        14,
                    ]:
                        string += "3"
                    else:
                        string += "N"
                    # 15 campo: largo 30 Razon social del retenido
                    string += (
                        pay.partner_id.name[0:25]
                        .replace("ñ", "n")
                        .replace("Ñ", "N")
                        .replace("\n", " ")
                        .replace("\r", " ")
                        .ljust(30)
                    )
                    # 16 campo: largo 16 importe otros conceptos
                    string += "0000000000000,00"
                    # 17 campo: largo 16 importe iva (completar si letra comp =A,M)
                    string += "0000000000000,00"
                    # 18 campo: largo 16 Monto sujeto a retencion/ recepcion (Monto Sujeto a Retención/ Percepción=(Monto del comprobante - Importe Iva -Importe otros conceptos))
                    monto_sujeto = str(pay.withholding_base_amount)
                    parte_decimal, parte_entera = math.modf(float(monto_sujeto))
                    parte_entera = str(parte_entera).split(".")[0]
                    parte_decimal = (
                        str(round(parte_decimal, 2)).split(".")[1][:2]
                    ).ljust(2, "0")
                    string += (parte_entera + "," + parte_decimal).zfill(16)
                    # 19 campo: largo 05 Alícuota
                    alic = 0
                    for arba_alicuot_id in pay.partner_id.arba_alicuot_ids.filtered(
                        lambda v: v.tag_id.jurisdiction_code == "901"
                        and v.alicuota_retencion != "0.00"
                        and v.from_date
                        and v.to_date
                        and v.from_date <= self.fecha_desde
                        and v.to_date >= self.fecha_hasta
                    ):
                        alic = arba_alicuot_id.alicuota_retencion
                    parte_decimal, parte_entera = math.modf(alic)
                    parte_entera = str(parte_entera).split(".")[0]
                    parte_decimal = (
                        str(round(parte_decimal, 2)).split(".")[1][:2]
                    ).ljust(2, "0")
                    monto = (parte_entera + "," + parte_decimal).zfill(5)
                    string += str(monto)
                    # 20 campo: largo 16 Retención/Percepción Practicada
                    parte_decimal, parte_entera = math.modf(
                        float(payment.computed_withholding_amount)
                    )
                    parte_entera = str(parte_entera).split(".")[0]
                    parte_decimal = (
                        str(round(parte_decimal, 2)).split(".")[1][:2]
                    ).ljust(2, "0")
                    amount = (parte_entera + "," + parte_decimal).zfill(16)
                    string += str(amount)
                    # 21 campo: largo 16 Monto Total Retenido/Percibido (igual a campo 20)
                    string += str(amount)
                elif tipo_operacion == "2":
                    string += tipo_operacion
                    # 2  campo: largo 03 codigo de norma (segun tipo de operacion)
                    string += "029"
                    # 3  campo: largo 10 fecha de retencion/percepcion
                    d = str(apunte.date.day).zfill(2)
                    m = str(apunte.date.month).zfill(2)
                    string += str(d) + "/" + str(m) + "/" + str(apunte.date.year)
                    # 4  campo: largo 02 tipo cbte origen de la ret (si tipo op =1 :01 fac 02 ndeb 03 ordenpago 04 boleta de dep 05 liq de pago 06 cert de obra 07 recibo 08 con de loc de serv 09 otro comb si tipo op =2 01 factura 09 otro comp)
                    # 5  campo: largo 01 letra del comprobante (A,M,B,C o espacio blanco)
                    if tipo_operacion == "1":
                        comprob = "03 "
                    elif tipo_operacion == "2":
                        comprob = (
                            "01"
                            + apunte.move_id.l10n_latam_document_type_id.l10n_ar_letter
                        )
                    else:
                        comprob = "04 "
                    string += comprob
                    # 6  campo: largo 16 nro de comprobante
                    if tipo_operacion == "1":
                        num_cbte = ("0001" + apunte.name).zfill(16)
                    elif tipo_operacion == "2":
                        num_cbte = apunte.move_id.l10n_latam_document_number.replace(
                            "-", ""
                        ).zfill(16)
                    else:
                        num_cbte = "NNNNNNNNNNNNNNNN"
                    string += num_cbte
                    # 7  campo: largo 10 fecha
                    d = str(apunte.date.day).zfill(2)
                    m = str(apunte.date.month).zfill(2)
                    string += str(d) + "/" + str(m) + "/" + str(apunte.date.year)
                    # DIVIDO EL CODIGO tipo de operación
                    # Calculo importes -- Modifico monto total para que coincida con el monto imponible
                    imp_iva = 0.0
                    imp_otros = 0.0
                    imp_caba = 0.0
                    if tipo_operacion == "2":
                        for impuesto in apunte.move_id.line_ids:
                            if impuesto.name.find("IVA 21%") != -1:
                                imp_iva += impuesto.credit
                            elif impuesto.filtered(
                                lambda r: r.tax_ids.filtered(
                                    lambda i: i.tipo_txt == "piibbcabaa"
                                )
                            ):  # name.find('Percepción IIBB CABA Aplicada') != -1: # apunte.move_id.line_ids.filtered(lambda r: r.tax_ids.filtered(lambda i: i.tipo_txt == 'piibbcabaa')):
                                imp_caba += impuesto.credit
                            else:
                                imp_otros += impuesto.credit
                    # 8  campo: largo 16 monto del comprobante 132241.31
                    if tipo_operacion == "2":
                        parte_decimal, parte_entera = math.modf(
                            apunte.move_id.amount_untaxed + imp_iva
                        )
                        parte_entera = str(parte_entera).split(".")[0]
                        parte_decimal = (
                            str(round(parte_decimal, 2)).split(".")[1][:2]
                        ).ljust(2, "0")
                        monto = (parte_entera + "," + parte_decimal).zfill(16)
                    else:
                        monto = "NNNNNNNNNNNNNNNN"
                    string += monto
                    # 9  campo: largo 16 nro de cert propio (blancos o nro cert)
                    string += " " * 16
                    # 10 campo: largo 01 tipo de doc del retenido (1: CDI 2: CUIL 3:CUIT)
                    if (
                        apunte.move_id.partner_id.l10n_latam_identification_type_id.name
                        == "CDI"
                    ):  # aqui estoy
                        tipo_doc = "1"
                    elif (
                        apunte.move_id.partner_id.l10n_latam_identification_type_id.name
                        == "CUIL"
                    ):
                        tipo_doc = "2"
                    elif (
                        apunte.move_id.partner_id.l10n_latam_identification_type_id.name
                        == "CUIT"
                    ):
                        tipo_doc = "3"
                    else:
                        tipo_doc = "N"
                    string += tipo_doc
                    # 11 campo: largo 11 nro de documento del retenido
                    string += apunte.move_id.partner_id.vat.zfill(11)
                    # 12 campo: largo 01 situacion IB del retenido (1:local 2:convenio multilateral 4:no inscripto 5: reg. simplificado) si tipo doc 1 o 2 =4
                    if tipo_doc in ["1", "4"]:
                        string += "4"
                    else:
                        string += "4"
                    # 13 campo: largo 11 nro inscripcion ID del Retenido  Si Situación IB del Retenido=4 : 00000000000
                    string += "0" * 11
                    # 14 campo: largo 01 situacion frente al iva del retenido (1 -RI 2- Exento 3-monotributo)
                    if (
                        apunte.move_id.partner_id.l10n_ar_afip_responsibility_type_id.id
                        in [1, 2, 12]
                    ):
                        string += "1"
                    elif (
                        apunte.move_id.partner_id.l10n_ar_afip_responsibility_type_id.id
                        in [5]
                    ):
                        string += "2"
                    elif (
                        apunte.move_id.partner_id.l10n_ar_afip_responsibility_type_id.id
                        in [7, 14]
                    ):
                        string += "3"
                    else:
                        string += "N"
                    # 15 campo: largo 30 Razon social del retenido
                    string += (
                        apunte.move_id.partner_id.name[0:29]
                        .replace("ñ", "n")
                        .replace("Ñ", "N")
                        .replace("\n", " ")
                        .replace("\r", " ")
                        .ljust(30)
                    )
                    # 16 campo: largo 16 importe otros conceptos
                    parte_decimal, parte_entera = math.modf(imp_otros)
                    parte_entera = str(parte_entera).split(".")[0]
                    parte_decimal = (
                        str(round(parte_decimal, 2)).split(".")[1][:2]
                    ).ljust(2, "0")
                    monto = (parte_entera + "," + parte_decimal).zfill(16)
                    string += "0000000000000,00"
                    # 17 campo: largo 16 importe iva (completar si letra comp =A,M)
                    parte_decimal, parte_entera = math.modf(imp_iva)
                    parte_entera = str(parte_entera).split(".")[0]
                    parte_decimal = (
                        str(round(parte_decimal, 2)).split(".")[1][:2]
                    ).ljust(2, "0")
                    monto = (parte_entera + "," + parte_decimal).zfill(16)
                    string += monto
                    # 18 campo: largo 16 Monto sujeto a retencion/ recepcion (Monto Sujeto a Retención/ Percepción=(Monto del comprobante - Importe Iva -Importe otros conceptos))
                    if tipo_operacion == "2":
                        parte_decimal, parte_entera = math.modf(
                            apunte.move_id.amount_untaxed
                        )
                        parte_entera = str(parte_entera).split(".")[0]
                        parte_decimal = (
                            str(round(parte_decimal, 2)).split(".")[1][:2]
                        ).ljust(2, "0")
                        monto = (parte_entera + "," + parte_decimal).zfill(16)
                    else:
                        monto = "NNNNNNNNNNNNNNNN"
                    string += monto
                    # 19 campo: largo 05 Alícuota
                    if apunte.move_id.amount_untaxed:
                        alicuota = round(
                            (imp_caba / apunte.move_id.amount_untaxed * 100), 2
                        )
                    parte_decimal, parte_entera = math.modf(alicuota)
                    parte_entera = str(parte_entera).split(".")[0]
                    parte_decimal = (
                        str(round(parte_decimal, 2)).split(".")[1][:2]
                    ).ljust(2, "0")
                    monto = (parte_entera + "," + parte_decimal).zfill(5)
                    string += monto
                    # 20 campo: largo 16 Retención/Percepción Practicada
                    parte_decimal, parte_entera = math.modf(imp_caba)
                    parte_entera = str(parte_entera).split(".")[0]
                    parte_decimal = (
                        str(round(parte_decimal, 2)).split(".")[1][:2]
                    ).ljust(2, "0")
                    monto = (parte_entera + "," + parte_decimal).zfill(16)
                    string += monto
                    # 21 campo: largo 16 Monto Total Retenido/Percibido (igual a campo 20)
                    parte_decimal, parte_entera = math.modf(imp_caba)
                    parte_entera = str(parte_entera).split(".")[0]
                    parte_decimal = (
                        str(round(parte_decimal, 2)).split(".")[1][:2]
                    ).ljust(2, "0")
                    monto = (parte_entera + "," + parte_decimal).zfill(16)
                    string += monto
                string += windows_line_ending
        string = string.encode("utf-8")
        return string

    @api.model
    def _tax_sicore(self, payments):
        """Genera TXT, para presentar Sicore"""
        nombre = "SICORE.txt"
        global NAME_FILE
        NAME_FILE = nombre
        windows_line_ending = "\r" + "\n"
        string = ""
        pay = ""
        for payment in payments.filtered(
            lambda v: v.tax_withholding_id.tax_group_id.tax_type
            in ["withholdings", "perception"]
            and v.state == "posted"
        ):
            # 1er campo codigo de comprobante: pago 06
            string = string + "06"
            # 2do campo fecha de emision de comprobante
            d = str(payment.date.day).zfill(2)
            m = str(payment.date.month).zfill(2)
            string = string + str(d) + "/" + str(m) + "/" + str(payment.date.year)
            # 3er campo nro de comprobante - imprimimos nro de retenciontring = string + payment_data['withholding_number'].zfill(16)
            payment_name = str(payment.payment_group_id.name)
            string = string + payment_name.zfill(16)
            # 4to campo amoun
            if payment.payment_group_id.payments_amount > 0:
                cadena = "%.2f" % payment.payment_group_id.payments_amount
                cadena = cadena.replace(".", ",")
            else:
                cadena = "0,00"
            string = string + cadena.zfill(16)
            # 5to Campo - Codigo de Impuesto - 217 tomado de tabla de codigos de sicore
            string = (
                string + "0217"
            )  # str(payment.tax_withholding_id.tax_group_id.afip_code).zfill(3)[:3]
            # 6to campo - Codigo de regimen 078 tomado de tabla de codigos de sicore - Enajenacion de bienes de cambio
            if payment.payment_group_id.regimen_ganancias_id.codigo_de_regimen:
                parse_string = str(
                    payment.payment_group_id.regimen_ganancias_id.codigo_de_regimen.zfill(
                        3
                    )
                )
                concepto = int(parse_string[0:3])
            else:
                concepto = int("078")
            string = string + str(concepto).zfill(3)
            # 7mo campo - codigo de operacion 1 tomado de ejemplo XLS
            string = string + "1"
            # 8vo campo - base de calculo (52 a 65)
            cadena = "%.2f" % payment.withholding_base_amount
            cadena = cadena.replace(".", ",")
            string = string + cadena.zfill(14)
            # 9vo campo - fecha de emision de la retencion (66 a 75)
            d = str(payment.date.day).zfill(2)
            m = str(payment.date.month).zfill(2)
            string = string + str(d) + "/" + str(m) + "/" + str(payment.date.year)
            # 10 (76 a 77) codigo de condicioon
            if payment.tax_withholding_id.tax_group_id.tax_type == "withholdings":
                string = string + "01"
            elif payment.tax_withholding_id.tax_group_id.tax_type == "perception":
                string = string + "02"
            # (78 a 78) 11- retencion a sujetos suspendidos REVISAR
            string = string + "0"
            # 12 mo campo - importe retencion - 79 a 92
            #    cadena = str(round(payment_data['amount'],2))
            cadena = "%.2f" % payment.amount
            cadena = cadena.replace(".", ",")
            # string = string + str(payment_data['amount']).zfill(14)
            string = string + cadena.zfill(14)
            # 13vo campo - porcentaje de la exclusion
            cadena = "000,00"
            string = string + cadena
            # 14vo campo - fecha de emision del boletin REVISAR
            d = str(payment.date.day).zfill(2)
            m = str(payment.date.month).zfill(2)
            string = string + str(d) + "/" + str(m) + "/" + str(payment.date.year)
            # string = string + ' '.rjust(10)
            # 15 tipo de documento del retenido
            string = string + "80"
            # 16vo campo - ro de CUIT revisar
            string = string + "000000000" + payment.partner_id.vat
            # 17vo campo nro certificado original
            string = string + payment.withholding_number.zfill(14)
            # Denominacion del ordenante
            string = string + windows_line_ending
        return string

    @api.model
    def _tax_sicoreiva(self, payments):
        """Genera TXT, para presentar Sicore de Iva"""
        nombre = "SICOREIVA.txt"
        global NAME_FILE
        NAME_FILE = nombre
        windows_line_ending = "\r" + "\n"
        string = ""
        pay = ""
        for payment in payments.filtered(
            lambda v: v.tax_withholding_id.tax_group_id.tax_type
            in ["withholdings", "perception"]
        ):
            # 1er campo codigo de comprobante: pago 06
            string = string + "06"
            # 2do campo fecha de emision de comprobante
            d = str(payment.date.day).zfill(2)
            m = str(payment.date.month).zfill(2)
            string = string + str(d) + "/" + str(m) + "/" + str(payment.date.year)
            # 3er campo nro de comprobante - imprimimos nro de retenciontring = string + payment_data['withholding_number'].zfill(16)
            if payment.payment_group_id.matched_move_line_ids.move_id:
                payment_n = payment.payment_group_id.matched_move_line_ids.move_id.l10n_latam_document_number.split(
                    "-"
                )
                payment_pto = payment_n[0]
                payment_name = str(payment_pto) + str(payment_n[1])
            else:
                payment_name = str(payment.payment_group_id.name)
            string = string + payment_name.zfill(16)
            # 4to campo amoun
            if payment.payment_group_id.payments_amount > 0:
                cadena = "%.2f" % payment.payment_group_id.payments_amount
                cadena = cadena.replace(".", ",")
            else:
                cadena = "0,00"
            string = string + cadena.zfill(16)
            # 5to Campo - Codigo de Impuesto - 767 tomado de tabla de codigos de sicore
            string = (
                string + "0767"
            )  # str(payment.tax_withholding_id.tax_group_id.afip_code).zfill(3)[:3]
            # 6to campo - Codigo de regimen 078 tomado de tabla de codigos de sicore - Enajenacion de bienes de cambio
            if payment.payment_group_id.tabla_iva:
                concepto = int(str(payment.payment_group_id.tabla_iva.zfill(3)))
            else:
                raise UserError(
                    _(
                        "Error: Para continuar debe selecionar el régimen correspondiente."
                    )
                )
            string = string + str(concepto).zfill(3)
            # 7mo campo - codigo de operacion 1 tomado de ejemplo XLS
            string = string + "1"
            # 8vo campo - base de calculo (52 a 65)
            cadena = "%.2f" % payment.withholding_base_amount
            cadena = cadena.replace(".", ",")
            string = string + cadena.zfill(14)
            # 9vo campo - fecha de emision de la retencion (66 a 75)
            d = str(payment.date.day).zfill(2)
            m = str(payment.date.month).zfill(2)
            string = string + str(d) + "/" + str(m) + "/" + str(payment.date.year)
            # 10 (76 a 77) codigo de condicioon
            if payment.tax_withholding_id.tax_group_id.tax_type == "withholdings":
                string = string + "01"
            elif payment.tax_withholding_id.tax_group_id.tax_type == "perception":
                string = string + "02"
            # (78 a 78) 11- retencion a sujetos suspendidos REVISAR
            string = string + "0"
            # 12 mo campo - importe retencion - 79 a 92
            #    cadena = str(round(payment_data['amount'],2))
            cadena = "%.2f" % payment.amount
            cadena = cadena.replace(".", ",")
            # string = string + str(payment_data['amount']).zfill(14)
            string = string + cadena.zfill(14)
            # 13vo campo - porcentaje de la exclusion
            cadena = "000,00"
            string = string + cadena
            # 14vo campo - fecha de emision del boletin REVISAR
            d = str(payment.date.day).zfill(2)
            m = str(payment.date.month).zfill(2)
            string = string + str(d) + "/" + str(m) + "/" + str(payment.date.year)
            # string = string + ' '.rjust(10)
            # 15 tipo de documento del retenido
            string = string + "80"
            # 16vo campo - ro de CUIT revisar
            string = string + "000000000" + payment.partner_id.vat
            # 17vo campo nro certificado original
            string = string + payment.withholding_number.zfill(14)
            # Denominacion del ordenante
            string = string + windows_line_ending
        return string

    @api.model
    def _tax_sifereperc(self, apuntes):
        """Genera TXT, para presentar impuesto Sifere Percepciones Sufridas"""
        nombre = "SIFEREPERC.txt"
        global NAME_FILE
        NAME_FILE = nombre
        windows_line_ending = "\r" + "\n"
        string = ""
        for apunte in apuntes.move_id.line_ids.filtered(
            lambda v: str(v.name).find("Sufrida") != -1
        ):
            # Solo comprobantes Fiscales
            # 1er campo: jURISDICCION
            tax = self.env["account.tax"].search([("name", "=", apunte.name)])
            if tax.tipo_txt == "sifereperc":
                if tax.state_id.country_code_afip:
                    string += str(tax.state_id.country_code_afip).zfill(3)
                else:
                    string += "   "
                # 2do campo: cuit separado por guiones
                string += (
                    apunte.partner_id.vat[:2]
                    + "-"
                    + apunte.partner_id.vat[2:-1].zfill(8)
                    + "-"
                    + apunte.partner_id.vat[-1:]
                )
                # 3do campo: fecha formato dd/mm/yyyy
                d = str(apunte.date.day).zfill(2)
                m = str(apunte.date.month).zfill(2)
                string += str(d) + "/" + str(m) + "/" + str(apunte.date.year)
                # 4.1to campo: numero de sucursal, 4 digitos (nro pto vta)
                # 4.2to campo: numero de emision 8 digitos (num factura)
                numcompro = str(apunte.move_id.l10n_latam_document_number).split("-")
                ptoventa = str(apunte.move_id.l10n_latam_document_number)[1:5]
                numcom = str(numcompro[1].zfill(8))
                numcompro = str(ptoventa + numcom)
                string += numcompro
                # 5.1er campo: tipo de comprobante (F=Factura, R=Recibo, C=Nota Crédito, D =Nota Debito, V=Nota de Venta)
                # 5.2to campo: letra de comprobante (ABC o "blanco")
                if (
                    apunte.move_id.l10n_latam_document_type_id.doc_code_prefix[0:2]
                    == "FA"
                ):
                    comprob = (
                        "F" + apunte.move_id.l10n_latam_document_type_id.l10n_ar_letter
                    )  # _id.name
                elif (
                    apunte.move_id.l10n_latam_document_type_id.doc_code_prefix[0:2]
                    == "ND"
                ):
                    comprob = (
                        "D" + apunte.move_id.l10n_latam_document_type_id.l10n_ar_letter
                    )
                elif (
                    apunte.move_id.l10n_latam_document_type_id.doc_code_prefix[0:2]
                    == "NC"
                ):
                    comprob = (
                        "C" + apunte.move_id.l10n_latam_document_type_id.l10n_ar_letter
                    )
                else:
                    comprob = "O "
                string += comprob
                # 5.3vo campo: importe percibido 11 digitos con la coma
                if comprob[:1] != "C":
                    parte_decimal, parte_entera = math.modf(apunte.debit)
                else:
                    parte_decimal, parte_entera = math.modf(apunte.credit)
                parte_entera = str(parte_entera).split(".")[0]
                parte_decimal = (str(round(parte_decimal, 2)).split(".")[1][:2]).ljust(
                    2, "0"
                )
                if comprob[:1] != "C":
                    string += (parte_entera + "," + parte_decimal).zfill(11)
                else:
                    string += "-" + (parte_entera + "," + parte_decimal).zfill(10)
                string += windows_line_ending
        return string

    # _tax_tucumanpercsufrida
    @api.model
    def _tax_tucumanpercsufrida(self, apuntes):
        """Genera TXT, para presentar impuesto Tucuman Percepciones Sufridas aplicativo SIAPRE el nombre del impuesto debe ser Percepción IIBB TUCUMAN Sufrida"""
        nombre = "TUCUMANPERCSUFRIDA.txt"
        global NAME_FILE
        NAME_FILE = nombre
        windows_line_ending = "\r" + "\n"
        string = ""
        for apunte in apuntes.move_id.line_ids.filtered(
            lambda v: str(v.name).find("Percepción IIBB TUCUMAN Sufrida") != -1
        ):
            # Solo comprobantes Fiscales
            # 1er campo: jURISDICCION
            tax = self.env["account.tax"].search([("name", "=", apunte.name)])
            if tax.siapre_txt == "tucumanpercsufrida":
                # 1 Vat
                string += (
                    apunte.partner_id.vat[:2]
                    + apunte.partner_id.vat[2:-1].zfill(8)
                    + apunte.partner_id.vat[-1:]
                )
                # 2 Nombre
                name = apunte.partner_id.name.zfill(50)
                if len(name) <= 50:
                    string += apunte.partner_id.name.zfill(50)
                else:
                    string += name[0:50]
                # 3do campo: fecha formato dd/mm/yyyy percepcion
                d = str(apunte.date.day).zfill(2)
                m = str(apunte.date.month).zfill(2)
                string += str(d) + "/" + str(m) + "/" + str(apunte.date.year)
                # 4.1to campo: numero de sucursal, 4 digitos (nro pto vta)
                # 4.2to campo: numero de emision 8 digitos (num factura)
                numcompro = str(apunte.move_id.l10n_latam_document_number).split("-")
                ptoventa = str(apunte.move_id.l10n_latam_document_number)[1:5]
                numcom = str(numcompro[1].zfill(8))
                numcompro = str(ptoventa + numcom)
                # 5.1er campo: tipo de comprobante (F=Factura, R=Recibo, C=Nota Crédito, D =Nota Debito, V=Nota de Venta)
                # 5.2to campo: letra de comprobante (ABC o "blanco")
                if (
                    apunte.move_id.l10n_latam_document_type_id.doc_code_prefix[0:2]
                    == "FA"
                ):
                    comprob = (
                        apunte.move_id.l10n_latam_document_type_id.l10n_ar_letter
                    )  # _id.name
                elif (
                    apunte.move_id.l10n_latam_document_type_id.doc_code_prefix[0:2]
                    == "ND"
                ):
                    comprob = apunte.move_id.l10n_latam_document_type_id.l10n_ar_letter
                elif (
                    apunte.move_id.l10n_latam_document_type_id.doc_code_prefix[0:2]
                    == "NC"
                ):
                    comprob = apunte.move_id.l10n_latam_document_type_id.l10n_ar_letter
                else:
                    comprob = "O"
                # Codigo comprobante
                string += apunte.move_id.l10n_latam_document_type_id.code.zfill(3)
                # Letra del comprobante
                string += comprob
                # punto de venta y número del comprobante
                string += numcompro
                # 5.3vo campo: importe percibido 18 digitos con la coma
                if (
                    apunte.move_id.l10n_latam_document_type_id.doc_code_prefix[0:2]
                    != "NC"
                ):
                    parte_decimal, parte_entera = math.modf(apunte.debit)
                else:
                    parte_decimal, parte_entera = math.modf(apunte.credit)
                parte_entera = str(parte_entera).split(".")[0]
                parte_decimal = (str(round(parte_decimal, 2)).split(".")[1][:2]).ljust(
                    2, "0"
                )
                if (
                    apunte.move_id.l10n_latam_document_type_id.doc_code_prefix[0:2]
                    != "NC"
                ):
                    string += (parte_entera + "," + parte_decimal).zfill(18)
                else:
                    string += "-" + (parte_entera + "," + parte_decimal).zfill(17)
                # Tipo de operacion
                string += "P"
                # Dato Random
                string += "11"
                string += windows_line_ending
        return string

    # _tax_perceivasufrida
    @api.model
    def _tax_perceivasufrida(self, apuntes):
        """Genera TXT, para presentar impuesto Tucuman Percepciones Sufridas aplicativo SIAPRE"""
        nombre = "PERCEPCIONIVASUFRIDA.txt"
        global NAME_FILE
        NAME_FILE = nombre
        windows_line_ending = "\r" + "\n"
        string = ""
        for apunte in apuntes.move_id.line_ids:
            # Solo comprobantes Fiscales str(v.name).find('Percepción IIBB TUCUMAN Sufrida') != -1 and
            # 1er campo: jURISDICCIO
            taxs = self.env["account.tax"].search([("name", "=", apunte.name)])
            for tax in taxs:
                if tax.tipo_txt == "perceivasufrida":
                    if (
                        apunte.move_id.l10n_latam_document_type_id.doc_code_prefix[0:2]
                        != "NC"
                    ):
                        # 1 REGIMEN
                        string += apunte.partner_id.tabla_percepcion_iva.codigo_de_regimen.zfill(
                            3
                        )
                        # 2 CUIT AGENTE
                        string += (
                            apunte.partner_id.vat[:2]
                            + apunte.partner_id.vat[2:-1].zfill(8)
                            + apunte.partner_id.vat[-1:]
                        )
                        # 3do campo: fecha formato dd/mm/yyyy percepcion
                        d = str(apunte.date.day).zfill(2)
                        m = str(apunte.date.month).zfill(2)
                        string += str(d) + "/" + str(m) + "/" + str(apunte.date.year)
                        # 4.1to campo: numero de sucursal, 4 digitos (nro pto vta)
                        # 4.2to campo: numero de emision 8 digitos (num factura)
                        numcompro = str(
                            apunte.move_id.l10n_latam_document_number
                        ).split("-")
                        ptoventa = str(apunte.move_id.l10n_latam_document_number)[1:5]
                        numcom = str(numcompro[1].zfill(8))
                        numcompro = str(ptoventa + numcom).zfill(16)
                        # punto de venta y número del comprobante
                        string += numcompro
                        # 5.3vo campo: importe percibido 18 digitos con la coma
                        if (
                            apunte.move_id.l10n_latam_document_type_id.doc_code_prefix[
                                0:2
                            ]
                            != "NC"
                        ):
                            parte_decimal, parte_entera = math.modf(apunte.debit)
                        else:
                            parte_decimal, parte_entera = math.modf(apunte.credit)
                        parte_entera = str(parte_entera).split(".")[0]
                        parte_decimal = (
                            str(round(parte_decimal, 2)).split(".")[1][:2]
                        ).ljust(2, "0")
                        string += (parte_entera + "," + parte_decimal).zfill(16)
                        string += windows_line_ending
                    else:
                        # 1 REGIMEN
                        string += apunte.partner_id.tabla_percepcion_iva.codigo_de_regimen.zfill(
                            3
                        )
                        # 2 CUIT AGENTE
                        string += (
                            apunte.partner_id.vat[:2]
                            + apunte.partner_id.vat[2:-1].zfill(8)
                            + apunte.partner_id.vat[-1:]
                        )
                        # 3do campo: fecha formato dd/mm/yyyy percepcion
                        d = str(apunte.date.day).zfill(2)
                        m = str(apunte.date.month).zfill(2)
                        string += str(d) + "/" + str(m) + "/" + str(apunte.date.year)
                        # 4.1to campo: numero de sucursal, 4 digitos (nro pto vta)
                        # 4.2to campo: numero de emision 8 digitos (num factura)
                        numcompro = str(
                            apunte.move_id.l10n_latam_document_number
                        ).split("-")
                        ptoventa = str(apunte.move_id.l10n_latam_document_number)[1:5]
                        numcom = str(numcompro[1].zfill(8))
                        numcompro = str(ptoventa).zfill(5) + "-" + str(numcom).zfill(8)
                        # punto de venta y número del comprobante
                        string += numcompro
                        string += "    "
                        # 5.3vo campo: importe percibido 18 digitos con la coma
                        if (
                            apunte.move_id.l10n_latam_document_type_id.doc_code_prefix[
                                0:2
                            ]
                            != "NC"
                        ):
                            parte_decimal, parte_entera = math.modf(apunte.debit)
                        else:
                            parte_decimal, parte_entera = math.modf(apunte.credit)
                        parte_entera = str(parte_entera).split(".")[0]
                        parte_decimal = (
                            str(round(parte_decimal, 2)).split(".")[1][:2]
                        ).ljust(2, "0")
                        string += "-" + (parte_entera + "," + parte_decimal).zfill(13)
                        string += windows_line_ending
        return string

    @api.model
    def _tax_tucumanretsufrida(self, payments):
        """Genera TXT, para presentar impuesto Tucuman Retenciones Sufridas aplicativo SIAPRE"""
        nombre = "TUCUMANRETSUFRIDA.txt"
        global NAME_FILE
        NAME_FILE = nombre
        windows_line_ending = "\r" + "\n"
        string = ""
        pay = ""
        for payment in payments.filtered(
            lambda r: r.tax_withholding_id.siapre_txt == "tucumanretsufrida"
        ):
            # 1 Vat
            string += (
                payment.partner_id.vat[:2]
                + payment.partner_id.vat[2:-1].zfill(8)
                + payment.partner_id.vat[-1:]
            )
            # 2 Nombre
            name = payment.partner_id.name.zfill(50)
            if len(name) <= 50:
                string += payment.partner_id.name.zfill(50)
            else:
                string += name[0:50]
            # 3do campo: fecha formato dd/mm/yyyy
            # string+=apunte.date[-2:]+'/'+apunte.date[5:-3]+'/'+apunte.date[:4]
            d = str(payment.date.day).zfill(2)
            m = str(payment.date.month).zfill(2)
            string += str(d) + "/" + str(m) + "/" + str(payment.date.year)
            # 4.1to campo: numero de sucursal, 4 digitos (nro pto vta)
            # 4.2to campo: numero de emision 8 digitos (num factura)
            if payment.payment_group_id.matched_move_line_ids.move_id:
                apunte = payment.payment_group_id.matched_move_line_ids.move_id
                numcompro = str(apunte.l10n_latam_document_number).split("-")
                ptoventa = str(apunte.l10n_latam_document_number)[1:5]
                numcom = str(numcompro[1].zfill(8))
                numcompro = str(ptoventa + numcom)
                # 5.1er campo: tipo de comprobante (F=Factura, R=Recibo, C=Nota Crédito, D =Nota Debito, V=Nota de Venta)
                # 5.2to campo: letra de comprobante (ABC o "blanco")
                if apunte.l10n_latam_document_type_id.doc_code_prefix[0:2] == "FA":
                    comprob = (
                        apunte.l10n_latam_document_type_id.l10n_ar_letter
                    )  # _id.name
                elif apunte.l10n_latam_document_type_id.doc_code_prefix[0:2] == "ND":
                    comprob = apunte.l10n_latam_document_type_id.l10n_ar_letter
                elif apunte.l10n_latam_document_type_id.doc_code_prefix[0:2] == "NC":
                    comprob = apunte.l10n_latam_document_type_id.l10n_ar_letter
                else:
                    comprob = "O"
                # Codigo comprobante
                string += apunte.l10n_latam_document_type_id.code.zfill(3)
                # Letra del comprobante
                string += comprob
                # punto de venta y número del comprobante
                string += numcompro
            else:
                # Codigo comprobante
                string += (
                    payment.payment_group_id.receiptbook_id.document_type_id.code.zfill(
                        3
                    )
                )
                # Letra del comprobante
                string += (
                    payment.payment_group_id.receiptbook_id.document_type_id.l10n_ar_letter
                )
                # punto de venta y número del comprobante
                numreplace = payment.payment_group_id.name
                numcompro = str(numreplace)
                if len(numcompro) < 12:
                    string += numcompro.zfill(12)
                else:
                    string += numcompro[0:12]
            # 5.3vo campo: importe percibido 11 digitos con la coma
            if comprob[:1] != "C":
                parte_decimal, parte_entera = math.modf(payment.amount)
            else:
                parte_decimal, parte_entera = math.modf(payment.amount)
            parte_entera = str(parte_entera).split(".")[0]
            parte_decimal = (str(round(parte_decimal, 2)).split(".")[1][:2]).ljust(
                2, "0"
            )
            string += (parte_entera + "," + parte_decimal).zfill(18)
            # Tipo de operacion
            string += "R"
            # Dato Random
            string += "11"
            string += windows_line_ending
        return string

    @api.model
    def _tax_sifereret(self, payments):
        """Genera TXT, para presentar impuesto Sifere Retenciones Sufridas"""
        nombre = "SIFERERET.txt"
        global NAME_FILE
        NAME_FILE = nombre
        windows_line_ending = "\r" + "\n"
        string = ""
        pay = ""
        for payment in payments.filtered(
            lambda r: r.tax_withholding_id.tipo_txt == "sifereret"
        ):
            if payment.tax_withholding_id.state_id.country_code_afip:
                string += str(
                    payment.tax_withholding_id.state_id.country_code_afip
                ).zfill(3)
            else:
                string += "000"
            string += (
                payment.partner_id.vat[:2]
                + "-"
                + payment.partner_id.vat[2:-1].zfill(8)
                + "-"
                + payment.partner_id.vat[-1:]
            )
            # 3do campo: fecha formato dd/mm/yyyy
            # string+=apunte.date[-2:]+'/'+apunte.date[5:-3]+'/'+apunte.date[:4]
            d = str(payment.date.day).zfill(2)
            m = str(payment.date.month).zfill(2)
            string += str(d) + "/" + str(m) + "/" + str(payment.date.year)
            # 4.1to campo: numero de sucursal, 4 digitos (nro pto vta)
            # 4.2to campo: numero de emision 8 digitos (num factura)
            if payment.payment_group_id.matched_move_line_ids.move_id:
                apunte = payment.payment_group_id.matched_move_line_ids
            else:
                pay = payment.payment_group_id
            numreplace = payment.payment_group_id.name
            numcompro = str(numreplace).zfill(20)
            string += numcompro
            # 5.1er campo: tipo de comprobante (F=Factura, R=Recibo, C=Nota Crédito, D =Nota Debito, V=Nota de Venta)
            # 5.2to campo: letra de comprobante (ABC o "blanco")
            comprob = "O "
            # 5.1er campo: tipo de comprobante (F=Factura, R=Recibo, C=Nota Crédito, D =Nota Debito, V=Nota de Venta)
            # 5.2to campo: letra de comprobante (ABC o "blanco")
            comprob = "O "
            string += comprob
            # 5.3vo campo: importe percibido 11 digitos con la coma
            if comprob[:1] != "C":
                parte_decimal, parte_entera = math.modf(payment.amount)
            else:
                parte_decimal, parte_entera = math.modf(payment.amount)
            parte_entera = str(parte_entera).split(".")[0]
            parte_decimal = (str(round(parte_decimal, 2)).split(".")[1][:2]).ljust(
                2, "0"
            )
            if comprob[:1] != "C":
                string += (parte_entera + "," + parte_decimal).zfill(31)
            else:
                string += "-" + (parte_entera + "," + parte_decimal).zfill(30)
            string += windows_line_ending
        apuntes = self.env["account.move.line"].search(
            [
                ("date", ">=", self.fecha_desde),
                ("date", "<=", self.fecha_hasta),
                ("move_id.state", "not in", ["cancelled", "draft"]),
                ("tax_ids.tipo_txt", "=", self.tipo_txt),
            ],
            order="date",
        )
        for apunte in apuntes.move_id.line_ids.filtered(
            lambda v: str(v.name).find("Retención") != -1
            and v.move_id.journal_id.provisional == False
        ):
            tax = self.env["account.tax"].search([("name", "=", apunte.name)])
            if tax.tipo_txt == "sifereret":
                if tax.state_id.country_code_afip:
                    string += str(tax.state_id.country_code_afip).zfill(3)
                else:
                    string += "   "
                # 2do campo: cuit separado por guiones
                string += (
                    apunte.partner_id.vat[:2]
                    + "-"
                    + apunte.partner_id.vat[2:-1].zfill(8)
                    + "-"
                    + apunte.partner_id.vat[-1:]
                )
                # 3do campo: fecha formato dd/mm/yyyy
                d = str(apunte.date.day).zfill(2)
                m = str(apunte.date.month).zfill(2)
                string += str(d) + "/" + str(m) + "/" + str(apunte.date.year)
                # 4.1to campo: numero de sucursal, 4 digitos (nro pto vta)
                # 4.2to campo: numero de emision 8 digitos (num factura)
                numcompro = str(apunte.move_id.l10n_latam_document_number).split("-")
                ptoventa = str(apunte.move_id.l10n_latam_document_number)[1:5]
                numcom = str(numcompro[1].zfill(16))
                numcompro = str(ptoventa + numcom)
                string += numcompro
                # 5.1er campo: tipo de comprobante (F=Factura, R=Recibo, C=Nota Crédito, D =Nota Debito, V=Nota de Venta)
                # 5.2to campo: letra de comprobante (ABC o "blanco")
                if (
                    apunte.move_id.l10n_latam_document_type_id.doc_code_prefix[0:2]
                    == "FA"
                ):
                    comprob = (
                        "F" + apunte.move_id.l10n_latam_document_type_id.l10n_ar_letter
                    )  # _id.name
                elif (
                    apunte.move_id.l10n_latam_document_type_id.doc_code_prefix[0:2]
                    == "ND"
                ):
                    comprob = (
                        "D" + apunte.move_id.l10n_latam_document_type_id.l10n_ar_letter
                    )
                elif (
                    apunte.move_id.l10n_latam_document_type_id.doc_code_prefix[0:2]
                    == "NC"
                ):
                    comprob = (
                        "C" + apunte.move_id.l10n_latam_document_type_id.l10n_ar_letter
                    )
                else:
                    comprob = "O "
                string += comprob
                # 5.3vo campo: importe percibido 11 digitos con la coma
                if comprob[:1] != "C":
                    parte_decimal, parte_entera = math.modf(apunte.debit)
                else:
                    parte_decimal, parte_entera = math.modf(apunte.credit)
                parte_entera = str(parte_entera).split(".")[0]
                parte_decimal = (str(round(parte_decimal, 2)).split(".")[1][:2]).ljust(
                    2, "0"
                )
                if comprob[:1] != "C":
                    string += (parte_entera + "," + parte_decimal).zfill(31)
                else:
                    string += "-" + (parte_entera + "," + parte_decimal).zfill(30)
                string += windows_line_ending
        return string

    # Retenciones Iva sufridas
    @api.model
    def _tax_reteivasufrida(self, payments):
        """Genera TXT, para presentar impuesto _tax_reteivasufrida"""
        nombre = "RETIVASUFRIDA.txt"
        global NAME_FILE
        NAME_FILE = nombre
        windows_line_ending = "\r" + "\n"
        string = ""
        pay = ""
        for payment in payments.filtered(
            lambda r: r.tax_withholding_id.tipo_txt == "reteivasufrida"
        ):
            # 1 Regimen
            string += payment.partner_id.tabla_iva.zfill(3)
            # 2 CUIT
            string += (
                payment.partner_id.vat[:2]
                + payment.partner_id.vat[2:-1].zfill(8)
                + payment.partner_id.vat[-1:]
            )
            # 3do campo: fecha formato dd/mm/yyyy
            # string+=apunte.date[-2:]+'/'+apunte.date[5:-3]+'/'+apunte.date[:4]
            d = str(payment.date.day).zfill(2)
            m = str(payment.date.month).zfill(2)
            string += str(d) + "/" + str(m) + "/" + str(payment.date.year)
            # 4.1to campo: numero de sucursal, 4 digitos (nro pto vta)
            # 4.2to campo: numero de emision 8 digitos (num factura)
            if payment.payment_group_id.matched_move_line_ids.move_id:
                apunte = payment.payment_group_id.matched_move_line_ids
            else:
                pay = payment.payment_group_id
            if pay:
                numreplace = payment.payment_group_id.name
                numcompro = str(numreplace)
                string += numcompro.zfill(16)
            else:
                numreplace = payment.payment_group_id.matched_move_line_ids.name[4:5]
                numcompro = str(
                    payment.payment_group_id.matched_move_line_ids.move_id.l10n_latam_document_number
                ).split("-")
                ptoventa = str(
                    payment.payment_group_id.matched_move_line_ids.move_id.l10n_latam_document_number
                )[1:5]
                numcom = str(numcompro[1])
                numcompro = str(ptoventa + numcom)
                string += numcompro.zfill(16)

            # 5.3vo campo: importe percibido 11 digitos con la coma
            parte_decimal, parte_entera = math.modf(payment.amount)
            parte_entera = str(parte_entera).split(".")[0]
            parte_decimal = (str(round(parte_decimal, 2)).split(".")[1][:2]).ljust(
                2, "0"
            )
            string += (parte_entera + "," + parte_decimal).zfill(16)
            string += windows_line_ending
        apuntes = self.env["account.move.line"].search(
            [
                ("date", ">=", self.fecha_desde),
                ("date", "<=", self.fecha_hasta),
                ("move_id.state", "not in", ["cancelled", "draft"]),
                ("tax_ids.tipo_txt", "=", self.tipo_txt),
            ],
            order="date",
        )
        for apunte in apuntes.move_id.line_ids.filtered(
            lambda v: v.move_id.journal_id.provisional == False
        ):
            taxs = self.env["account.tax"].search([("name", "=", apunte.name)])
            for tax in taxs:
                if tax.tipo_txt == "reteivasufrida":
                    # 1 REGIMEN
                    string += apunte.partner_id.tabla_iva.zfill(3)
                    # 2do campo: cuit separado por guiones
                    string += (
                        apunte.partner_id.vat[:2]
                        + apunte.partner_id.vat[2:-1].zfill(8)
                        + apunte.partner_id.vat[-1:]
                    )
                    # 3do campo: fecha formato dd/mm/yyyy
                    d = str(apunte.date.day).zfill(2)
                    m = str(apunte.date.month).zfill(2)
                    string += str(d) + "/" + str(m) + "/" + str(apunte.date.year)
                    # 4.1to campo: numero de sucursal, 4 digitos (nro pto vta)
                    # 4.2to campo: numero de emision 8 digitos (num factura)
                    numcompro = str(apunte.move_id.l10n_latam_document_number).split(
                        "-"
                    )
                    ptoventa = str(apunte.move_id.l10n_latam_document_number)[1:5]
                    numcom = str(numcompro[1])
                    numcompro = str(ptoventa + numcom)
                    string += numcompro.zfill(16)
                    # 5.3vo campo: importe percibido 11 digitos con la coma
                    if (
                        apunte.move_id.l10n_latam_document_type_id.doc_code_prefix[0:2]
                        == "FA"
                    ):
                        comprob = (
                            "F"
                            + apunte.move_id.l10n_latam_document_type_id.l10n_ar_letter
                        )  # _id.name
                    elif (
                        apunte.move_id.l10n_latam_document_type_id.doc_code_prefix[0:2]
                        == "ND"
                    ):
                        comprob = (
                            "D"
                            + apunte.move_id.l10n_latam_document_type_id.l10n_ar_letter
                        )
                    elif (
                        apunte.move_id.l10n_latam_document_type_id.doc_code_prefix[0:2]
                        == "NC"
                    ):
                        comprob = (
                            "C"
                            + apunte.move_id.l10n_latam_document_type_id.l10n_ar_letter
                        )
                    else:
                        comprob = "O "
                    if comprob[:1] != "C":
                        parte_decimal, parte_entera = math.modf(apunte.debit)
                    else:
                        parte_decimal, parte_entera = math.modf(apunte.credit)
                    parte_entera = str(parte_entera).split(".")[0]
                    parte_decimal = (
                        str(round(parte_decimal, 2)).split(".")[1][:2]
                    ).ljust(2, "0")
                    if comprob[:1] != "C":
                        string += (parte_entera + "," + parte_decimal).zfill(16)
                    else:
                        string += "-" + (parte_entera + "," + parte_decimal).zfill(15)
                    string += windows_line_ending
        return string

    @api.model
    def _tax_tucumandatos(self, apuntes):
        """Genera TXT, para presentar impuesto Tucuman DATOS.TXT"""
        nombre = "DATOS.txt"
        global NAME_FILE
        NAME_FILE = nombre
        windows_line_ending = "\r" + "\n"
        string = ""
        fecha_desde = self.fecha_desde
        fecha_hasta = self.fecha_hasta
        num_fact = ""
        for apunte in apuntes:
            if apunte.move_id.l10n_latam_document_number != num_fact:
                num_fact = apunte.move_id.l10n_latam_document_number
                for line_id in apunte.move_id.line_ids.filtered(
                    lambda r: r.tax_ids.filtered(lambda i: i.tucuman_datos_txt)
                ):
                    alicuota = 0.0
                    for alic in apunte.move_id.partner_id.arba_alicuot_ids.filtered(
                        lambda v: v.tag_id.jurisdiction_code == "924"
                        and v.from_date != False
                        and v.from_date <= fecha_desde
                        and v.to_date >= fecha_hasta
                    ):
                        if alic.alicuota_percepcion != 0.0:
                            alicuota = float(alic.alicuota_percepcion)
                    if alicuota != 0.0:
                        if (
                            apunte.move_id.l10n_latam_document_type_id.doc_code_prefix[
                                0:2
                            ]
                            != "NC"
                        ):
                            # 1ro campo: fecha formato dd/mm/yyyy
                            d = str(apunte.date.day).zfill(2)
                            m = str(apunte.date.month).zfill(2)
                            string += str(apunte.date.year) + str(m) + str(d)
                            # 2do campo: tipo doc
                            tdoc = ""
                            if (
                                apunte.partner_id.l10n_latam_identification_type_id.name
                                == "CUIT"
                            ):
                                tdoc = "80"
                            elif (
                                apunte.partner_id.l10n_latam_identification_type_id.name
                                == "DNI"
                            ):
                                tdoc = "79"
                            elif (
                                apunte.partner_id.l10n_latam_identification_type_id.name
                                == "CUIL"
                            ):
                                tdoc = "80"
                            else:
                                tdoc = "11"
                            string += tdoc
                            string += (
                                apunte.partner_id.vat[:2]
                                + apunte.partner_id.vat[2:-1].zfill(8)
                                + apunte.partner_id.vat[-1:]
                            )
                            # 4to campo: TPO Comprobante aqui
                            string += (
                                apunte.move_id.l10n_latam_document_type_id.code.zfill(2)
                            )
                            # 5to campo: letra de comprobante (ABC o "blanco")
                            comprob2 = "NN"
                            if (
                                apunte.move_id.l10n_latam_document_type_id.doc_code_prefix[
                                    0:2
                                ]
                                == "FA"
                            ):
                                comprob = (
                                    apunte.move_id.l10n_latam_document_type_id.l10n_ar_letter
                                )  # _id.name
                            elif (
                                apunte.move_id.l10n_latam_document_type_id.doc_code_prefix[
                                    0:2
                                ]
                                == "ND"
                            ):
                                comprob = (
                                    apunte.move_id.l10n_latam_document_type_id.l10n_ar_letter
                                )
                            elif (
                                apunte.move_id.l10n_latam_document_type_id.doc_code_prefix[
                                    0:2
                                ]
                                == "NC"
                            ):
                                comprob2 = (
                                    "C"
                                    + apunte.move_id.l10n_latam_document_type_id.l10n_ar_letter
                                )
                                comprob = (
                                    apunte.move_id.l10n_latam_document_type_id.l10n_ar_letter
                                )
                            else:
                                comprob = "O "
                            string += comprob
                            # 4.2to campo: numero de emision 8 digitos (num factura)
                            numcompro = str(
                                apunte.move_id.l10n_latam_document_number
                            ).split("-")
                            ptoventa = str(apunte.move_id.l10n_latam_document_number)[
                                1:5
                            ]
                            numcom = str(numcompro[1].zfill(8))
                            numcompro = str(ptoventa + numcom)
                            string += numcompro
                            # 8 Base del calculo
                            parte_decimal, parte_entera = math.modf(
                                apunte.move_id.amount_untaxed
                            )
                            parte_entera = str(parte_entera).split(".")[0]
                            parte_decimal = (
                                str(round(parte_decimal, 2)).split(".")[1][:2]
                            ).ljust(2, "0")
                            string += (parte_entera + "." + parte_decimal).zfill(15)
                            # 9 Porcentaje / Alucuota
                            val2 = ""
                            alicuota = 0.0
                            for (
                                alic
                            ) in apunte.move_id.partner_id.arba_alicuot_ids.filtered(
                                lambda v: v.tag_id.jurisdiction_code == "924"
                            ):
                                if (
                                    alic.from_date <= fecha_desde
                                    and alic.to_date >= fecha_hasta
                                ):
                                    if alic.alicuota_percepcion != 0.0:
                                        alicuota = float(alic.alicuota_percepcion)
                                    else:
                                        alicuota = 0.0
                            alicuota = str(alicuota).replace(",", ".")
                            parte_decimal, parte_entera = math.modf(float(alicuota))
                            parte_entera = str(parte_entera).split(".")[0]
                            parte_decimal = (
                                str(round(parte_decimal, 2)).split(".")[1][:2]
                            ).ljust(3, "0")
                            monto = (parte_entera + "." + parte_decimal).zfill(6)
                            # import pdb; pdb.set_trace()
                            string += monto
                            # 10 Monto RET/PERC
                            val2 = ""
                            if comprob2[:1] != "C":
                                parte_decimal, parte_entera = math.modf(line_id.credit)
                            else:
                                parte_decimal, parte_entera = math.modf(line_id.debit)
                            parte_entera = str(parte_entera).split(".")[0]
                            parte_decimal = (
                                str(round(parte_decimal, 2)).split(".")[1][:2]
                            ).ljust(2, "0")
                            if comprob2[:1] != "C":
                                # string += (parte_entera + '.' + parte_decimal).zfill(6)
                                val2 = (parte_entera + "." + parte_decimal).zfill(15)
                            else:
                                # string += '-' + (parte_entera + '.' + parte_decimal).zfill(5)
                                val2 = (parte_entera + "." + parte_decimal).zfill(15)
                            string += val2
                        elif (
                            apunte.move_id.l10n_latam_document_type_id.doc_code_prefix[
                                0:2
                            ]
                            == "NC"
                        ):
                            # 1ro campo: fecha formato dd/mm/yyyy
                            d = str(apunte.date.day).zfill(2)
                            m = str(apunte.date.month).zfill(2)
                            string += str(apunte.date.year) + str(m) + str(d)
                            # 2do campo: tipo doc
                            tdoc = ""
                            if (
                                apunte.partner_id.l10n_latam_identification_type_id.name
                                == "CUIT"
                            ):
                                tdoc = "80"
                            elif (
                                apunte.partner_id.l10n_latam_identification_type_id.name
                                == "DNI"
                            ):
                                tdoc = "79"
                            elif (
                                apunte.partner_id.l10n_latam_identification_type_id.name
                                == "CUIL"
                            ):
                                tdoc = "80"
                            else:
                                tdoc = "11"
                            string += tdoc
                            string += (
                                apunte.partner_id.vat[:2]
                                + apunte.partner_id.vat[2:-1].zfill(8)
                                + apunte.partner_id.vat[-1:]
                            )
                            # 4to campo: TPO Comprobante aqui
                            string += (
                                apunte.move_id.l10n_latam_document_type_id.code.zfill(2)
                            )
                            # 5to campo: letra de comprobante (ABC o "blanco")
                            comprob2 = "NN"
                            if (
                                apunte.move_id.l10n_latam_document_type_id.doc_code_prefix[
                                    0:2
                                ]
                                == "FA"
                            ):
                                comprob = (
                                    apunte.move_id.l10n_latam_document_type_id.l10n_ar_letter
                                )  # _id.name
                            elif (
                                apunte.move_id.l10n_latam_document_type_id.doc_code_prefix[
                                    0:2
                                ]
                                == "ND"
                            ):
                                comprob = (
                                    apunte.move_id.l10n_latam_document_type_id.l10n_ar_letter
                                )
                            elif (
                                apunte.move_id.l10n_latam_document_type_id.doc_code_prefix[
                                    0:2
                                ]
                                == "NC"
                            ):
                                comprob2 = (
                                    "C"
                                    + apunte.move_id.l10n_latam_document_type_id.l10n_ar_letter
                                )
                                comprob = (
                                    apunte.move_id.l10n_latam_document_type_id.l10n_ar_letter
                                )
                            else:
                                comprob = "O "
                            string += comprob
                            # 4.2to campo: numero de emision 8 digitos (num factura)
                            numcompro = str(
                                apunte.move_id.l10n_latam_document_number
                            ).split("-")
                            ptoventa = str(apunte.move_id.l10n_latam_document_number)[
                                1:5
                            ]
                            numcom = str(numcompro[1].zfill(8))
                            numcompro = str(ptoventa + numcom)
                            string += numcompro
                            # 8 Base del calculo
                            parte_decimal, parte_entera = math.modf(
                                apunte.move_id.amount_untaxed
                            )
                            parte_entera = str(parte_entera).split(".")[0]
                            parte_decimal = (
                                str(round(parte_decimal, 2)).split(".")[1][:2]
                            ).ljust(2, "0")
                            string += (parte_entera + "." + parte_decimal).zfill(15)
                            # 9 Porcentaje / Alucuota
                            val2 = ""
                            alicuota = 0.0
                            for (
                                alic
                            ) in apunte.move_id.partner_id.arba_alicuot_ids.filtered(
                                lambda v: v.tag_id.jurisdiction_code == "924"
                                and v.from_date <= fecha_desde
                                and v.to_date >= fecha_hasta
                            ):
                                if alic.alicuota_percepcion != 0.0:
                                    alicuota = float(alic.alicuota_percepcion)
                                else:
                                    alicuota = 0.0
                            alicuota = str(alicuota).replace(",", ".")
                            parte_decimal, parte_entera = math.modf(float(alicuota))
                            parte_entera = str(parte_entera).split(".")[0]
                            parte_decimal = (
                                str(round(parte_decimal, 2)).split(".")[1][:2]
                            ).ljust(3, "0")
                            monto = (parte_entera + "." + parte_decimal).zfill(6)
                            string += monto
                            # 10 Monto RET/PERC
                            val2 = ""
                            if comprob2[:1] != "C":
                                parte_decimal, parte_entera = math.modf(line_id.credit)
                            else:
                                parte_decimal, parte_entera = math.modf(line_id.debit)
                            parte_entera = str(parte_entera).split(".")[0]
                            parte_decimal = (
                                str(round(parte_decimal, 2)).split(".")[1][:2]
                            ).ljust(2, "0")
                            if comprob2[:1] != "C":
                                val2 = (parte_entera + "." + parte_decimal).zfill(15)
                            else:
                                val2 = (parte_entera + "." + parte_decimal).zfill(15)
                            string += val2
                        string += windows_line_ending
        return string

    @api.model
    def _tax_tucumanretper(self, apuntes):
        """Genera TXT, para presentar impuesto Tucuman RETPER.TXT"""
        nombre = "RETPER.txt"
        global NAME_FILE
        NAME_FILE = nombre
        windows_line_ending = "\r" + "\n"
        string = ""
        num_fact = ""
        for apunte in apuntes:
            if apunte.move_id.l10n_latam_document_number != num_fact:
                num_fact = apunte.move_id.l10n_latam_document_number
                for line_id in apunte.move_id.line_ids.filtered(
                    lambda r: r.tax_ids.filtered(lambda i: i.tucuman_retper_txt)
                ):
                    comprob2 = "NN"
                    if (
                        apunte.move_id.l10n_latam_document_type_id.doc_code_prefix[0:2]
                        == "FA"
                    ):
                        comprob = (
                            apunte.move_id.l10n_latam_document_type_id.l10n_ar_letter
                        )
                    elif (
                        apunte.move_id.l10n_latam_document_type_id.doc_code_prefix[0:2]
                        == "ND"
                    ):
                        comprob = (
                            apunte.move_id.l10n_latam_document_type_id.l10n_ar_letter
                        )
                    elif (
                        apunte.move_id.l10n_latam_document_type_id.doc_code_prefix[0:2]
                        == "NC"
                    ):
                        comprob2 = (
                            "C"
                            + apunte.move_id.l10n_latam_document_type_id.l10n_ar_letter
                        )
                        comprob = (
                            apunte.move_id.l10n_latam_document_type_id.l10n_ar_letter
                        )
                    else:
                        comprob = "O "
                    val2 = ""
                    if comprob2[:1] != "C":
                        parte_decimal, parte_entera = math.modf(line_id.credit)
                    else:
                        parte_decimal, parte_entera = math.modf(line_id.debit)
                    parte_entera = str(parte_entera).split(".")[0]
                    parte_decimal = (
                        str(round(parte_decimal, 2)).split(".")[1][:2]
                    ).ljust(2, "0")
                    if comprob2[:1] != "C":
                        # string += (parte_entera + '.' + parte_decimal).zfill(6)
                        val2 = (parte_entera + "." + parte_decimal).zfill(15)
                    else:
                        # string += '-' + (parte_entera + '.' + parte_decimal).zfill(5)
                        val2 = (parte_entera + "." + parte_decimal).zfill(15)
                    if val2 != "000000000000.00":
                        # 1ro campo: tipo doc
                        string += (
                            apunte.partner_id.l10n_latam_identification_type_id.l10n_ar_afip_code
                        )
                        string += (
                            apunte.partner_id.vat[:2]
                            + apunte.partner_id.vat[2:-1].zfill(8)
                            + apunte.partner_id.vat[-1:]
                        )
                        # 3to campo: Nombre  apunte.move_id.partner_id.name.rjust(40, 'N')
                        if len(apunte.move_id.partner_id.name) <= 40:
                            string += apunte.move_id.partner_id.name.rjust(40, "N")
                        else:
                            string += apunte.move_id.partner_id.name[0:40]
                        # 4to campo: Domicilio
                        street_number = ""
                        if apunte.move_id.partner_id.street != False:
                            # Obtenemos number street
                            size = len(apunte.move_id.partner_id.street.split())
                            tot = size - 1
                            street_number = apunte.move_id.partner_id.street.split()[
                                tot
                            ]
                            # Obtenemos solo cadena de street
                            street_s = ""
                            if size == 4:
                                street_s = (
                                    apunte.move_id.partner_id.street.split()[0]
                                    + " "
                                    + apunte.move_id.partner_id.street.split()[1]
                                    + " "
                                    + apunte.move_id.partner_id.street.split()[tot - 1]
                                )
                            if size == 3:
                                street_s = (
                                    apunte.move_id.partner_id.street.split()[0]
                                    + " "
                                    + apunte.move_id.partner_id.street.split()[tot - 1]
                                )
                            elif size == 2:
                                street_s = apunte.move_id.partner_id.street.split()[0]
                            if len(street_s) <= 40:
                                string += street_s.rjust(40, "N")
                            else:
                                string += street_s[0:40]
                        else:
                            string += "N".rjust(40, "N")
                        # 5to campo: Numero
                        num = "0"
                        if street_number and street_number.isdigit():
                            if len(street_number) <= 5:
                                string += street_number.zfill(5)
                            else:
                                string += street_number[0:5]
                        else:
                            string += num.zfill(5)
                        # 6to campo: Localidad
                        if apunte.move_id.partner_id.city != False:
                            if len(apunte.move_id.partner_id.city) <= 15:
                                string += apunte.move_id.partner_id.city.rjust(15, "N")
                            else:
                                string += apunte.move_id.partner_id.city[0:15]
                        else:
                            string += "N".rjust(15, "N")
                        # 7to campo: Provincia
                        if len(apunte.move_id.partner_id.state_id.name) <= 15:
                            string += apunte.move_id.partner_id.state_id.name.rjust(
                                15, "N"
                            )
                        else:
                            string += "{0:<15}".format(
                                apunte.move_id.partner_id.state_id.name
                            )[0:15]
                        # 8to campo: Numero
                        noused = "----------"
                        string += noused
                        # 9to campo: Codigo Postal  apunte.move_id.partner_id.zip[0:1].isdigit()
                        if apunte.move_id.partner_id.zip:
                            if apunte.move_id.partner_id.zip[0:1].isdigit():
                                string += apunte.move_id.partner_id.zip.zfill(8)
                            else:
                                zip = apunte.move_id.partner_id.zip[1:]
                                string += zip.zfill(8)
                        string += windows_line_ending
        return string

    @api.model
    def _tax_tucumanncfact(self, apuntes):
        """Genera TXT, para presentar impuesto Tucuman NCFACT.TXT"""
        nombre = "NCFACT.txt"
        global NAME_FILE
        NAME_FILE = nombre
        windows_line_ending = "\r" + "\n"
        string = ""
        num_fact = ""
        for apunte in apuntes.move_id.line_ids.filtered(
            lambda r: r.tax_ids.filtered(lambda i: i.tucuman_ncfact_txt)
        ):
            if apunte.move_id.l10n_latam_document_number != num_fact:
                num_fact = apunte.move_id.l10n_latam_document_number
                if (
                    apunte.move_id.l10n_latam_document_type_id.doc_code_prefix[0:2]
                    == "NC"
                ):
                    # import pdb; pdb.set_trace()
                    # 1ro y 2do: punto venta(lugar emision) y numero de NC apunte.move_id.amount_by_group
                    numcompronc = str(apunte.move_id.l10n_latam_document_number).split(
                        "-"
                    )
                    ptoventanc = str(apunte.move_id.l10n_latam_document_number)[1:5]
                    numcomnc = str(numcompronc[1].zfill(8))
                    numcompronc = str(ptoventanc + numcomnc)
                    string += numcompronc
                    # 3ro y 4to: punto venta(lugar emision) y numero de FC
                    numcompro = str(
                        apunte.move_id.reversed_entry_id.l10n_latam_document_number
                    ).split("-")
                    ptoventa = str(
                        apunte.move_id.reversed_entry_id.l10n_latam_document_number
                    )[1:5]
                    numcom = str(numcompro[1].zfill(8))
                    numcompro = str(ptoventa + numcom)
                    string += numcompro
                    # Tipo Factura
                    string += apunte.move_id.reversed_entry_id.l10n_latam_document_type_id.code.zfill(
                        2
                    )
                    string += windows_line_ending
        return string

    @api.model
    def _tax_misionesperc(self, apuntes):
        """Genera TXT, para presentar impuesto Percepción Misiones"""
        nombre = "MISIONESPERC.txt"
        global NAME_FILE
        NAME_FILE = nombre
        windows_line_ending = "\r" + "\n"
        string = ""
        value = "0.00"
        fecha_desde = self.fecha_desde
        fecha_hasta = self.fecha_hasta
        for apunte in apuntes.move_id.line_ids:
            taxs = self.env["account.tax"].search([("name", "=", apunte.name)])
            for tax in taxs:
                if tax.tipo_txt == "misionesperc":
                    if (
                        apunte.move_id.l10n_latam_document_type_id.doc_code_prefix[0:1]
                        != "C"
                    ):
                        value = apunte.credit
                    else:
                        value = apunte.debit
                    if value != "0.00":
                        # 1rdo campo: fecha formato dd/mm/yyyy
                        d = ""
                        if len(str(apunte.date.day)) == 1:
                            d = "0" + str(apunte.date.day)
                        else:
                            d = str(apunte.date.day)
                        m = ""
                        if len(str(apunte.date.month)) == 1:
                            m = "0" + str(apunte.date.month)
                        else:
                            m = str(apunte.date.month)
                        string += str(d) + "-" + str(m) + "-" + str(apunte.date.year)
                        # 2er campo: tipo de comprobante (F=Factura, R=Recibo, C=Nota Crédito, D =Nota Debito, V=Nota de Venta)
                        # 2to campo: letra de comprobante (ABC o "blanco")
                        if (
                            apunte.move_id.l10n_latam_document_type_id.doc_code_prefix[
                                0:2
                            ]
                            == "FA"
                        ):
                            comprob = (
                                "FA_"
                                + apunte.move_id.l10n_latam_document_type_id.l10n_ar_letter
                            )  # _id.name
                        elif (
                            apunte.move_id.l10n_latam_document_type_id.doc_code_prefix[
                                0:2
                            ]
                            == "ND"
                        ):
                            comprob = (
                                "ND_"
                                + apunte.move_id.l10n_latam_document_type_id.l10n_ar_letter
                            )
                        elif (
                            apunte.move_id.l10n_latam_document_type_id.doc_code_prefix[
                                0:2
                            ]
                            == "NC"
                        ):
                            comprob = (
                                "NC_"
                                + apunte.move_id.l10n_latam_document_type_id.l10n_ar_letter
                            )
                        else:
                            comprob = "NN_N"
                        string += "," + comprob
                        # 3to campo: numero de sucursal, 4 digitos (nro pto vta)
                        # 3to campo: numero de emision 8 digitos (num factura)
                        numcompro = str(
                            apunte.move_id.l10n_latam_document_number
                        ).split("-")
                        ptoventa = str(apunte.move_id.l10n_latam_document_number)[1:5]
                        numcom = str(numcompro[1].zfill(8))
                        numcompro = str(ptoventa + numcom)
                        if len(numcompro) < 20:
                            string += "," + numcompro
                        else:
                            string += "," + numcompro.zfill(20)
                        # 4to: Partner
                        if len(apunte.move_id.partner_id.name) < 100:
                            string += "," + apunte.move_id.partner_id.name
                        else:
                            string += "," + apunte.move_id.partner_id.name.zfill(100)
                        # 5to: cuit separado por guiones
                        string += (
                            ","
                            + apunte.partner_id.vat[:2]
                            + "-"
                            + apunte.partner_id.vat[2:-1].zfill(8)
                            + "-"
                            + apunte.partner_id.vat[-1:]
                        )
                        # 6to campo: importe percibido 5 digitos con la coma
                        parte_decimal, parte_entera = math.modf(
                            apunte.move_id.amount_untaxed
                        )
                        parte_entera = str(parte_entera).split(".")[0]
                        parte_decimal = (
                            str(round(parte_decimal, 2)).split(".")[1][:2]
                        ).ljust(2, "0")
                        imp_per = parte_entera + "." + parte_decimal
                        if len(imp_per) < 5:
                            if comprob[:1] != "C":
                                string += "," + imp_per
                            else:
                                string += "," + "-" + imp_per
                        else:
                            if comprob[:1] != "C":
                                string += "," + imp_per.zfill(5)
                            else:
                                string += "," + "-" + imp_per.zfill(4)
                        # 7mo_ Alicuota
                        alicuota = 0.0
                        for alic in apunte.move_id.partner_id.arba_alicuot_ids.filtered(
                            lambda v: v.tag_id.jurisdiction_code == "914"
                        ):
                            if alic.alicuota_percepcion != 0.0:
                                alicuota = float(alic.alicuota_percepcion)
                            else:
                                alicuota = 0.0
                        alicuota = str(alicuota).replace(",", ".")
                        parte_decimal, parte_entera = math.modf(float(alicuota))
                        parte_entera = str(parte_entera).split(".")[0]
                        parte_decimal = (
                            str(round(parte_decimal, 2)).split(".")[1][:2]
                        ).ljust(2, "0")
                        monto = (parte_entera + "." + parte_decimal).zfill(4)
                        string += "," + monto
                        string += windows_line_ending
        return string

    @api.model
    def _tax_santafe(self, apuntes):
        """Genera TXT, para presentar impuesto Perc/Ret Santa Fe"""
        nombre = "SANTAFEPERCRET.txt"
        global NAME_FILE
        NAME_FILE = nombre
        windows_line_ending = "\r" + "\n"
        string = ""
        monto = "0.0"
        alic = 0
        tipo_sale = ""
        mont_com = ""
        monto_sujeto = ""
        imp_caba = ""
        for apunte in apuntes.filtered(
            lambda r: r.move_id.l10n_latam_document_type_id.doc_code_prefix[0:2] != "NC"
        ):
            # Solo comprobantes Fiscales
            # if apunte.move_id.journal_id.provisional == False:
            for taxid in apunte.move_id.line_ids.filtered(
                lambda r: r.tax_ids.filtered(lambda i: i.tipo_txt == "percsantafe")
            ):
                if taxid.debit != "0.0" or taxid.credit != "0.0":
                    # 1  campo: largo 01 tipo operación (1:retencion 2:percepcion)
                    if apunte.tax_ids[0].type_tax_use == "supplier":
                        tipo_operacion = "1"
                    elif apunte.tax_ids[0].type_tax_use == "sale":
                        tipo_operacion = "2"
                        tipo_sale = "1"
                        monto_sujeto = str(taxid.credit)
                        if (
                            apunte.move_id.l10n_latam_document_type_id.doc_code_prefix[
                                0:2
                            ]
                            == "NC"
                        ):
                            tipo_sale = "2"
                            monto_sujeto = str(taxid.debit)
                        #    continue
                    else:
                        tipo_operacion = "0"
                    string += tipo_operacion
                    # 2  campo: largo 10 fecha de retencion/percepcion
                    d = str(apunte.date.day).zfill(2)
                    m = str(apunte.date.month).zfill(2)
                    string += str(d) + "/" + str(m) + "/" + str(apunte.date.year)
                    # 3  campo: largo 03 codigo articulo inciso por el que retiene
                    api_codigo = ""
                    for arba_alicuot_id in apunte.partner_id.arba_alicuot_ids:
                        if arba_alicuot_id.api_codigo_articulo_percepcion:
                            api_codigo = arba_alicuot_id.api_codigo_articulo_percepcion
                        if arba_alicuot_id.alicuota_percepcion != "0.00":
                            alic = arba_alicuot_id.alicuota_percepcion
                    string += api_codigo.zfill(3)
                    # 4  campo: largo 02 tipo cbte origen de la ret (si tipo op =1 :01 fac 02 ndeb 03 ordenpago 04 boleta de dep 05 liq de pago 06 cert de obra 07 recibo 08 con de loc de serv 09 otro comb si tipo op =2 01 factura 09 otro comp)
                    # 5  campo: largo 01 letra del comprobante (A,M,B,C o espacio blanco)
                    if tipo_operacion == "1":
                        comprob = "03 "
                    elif tipo_operacion == "2":
                        comprob = (
                            "01"
                            + apunte.move_id.l10n_latam_document_type_id.l10n_ar_letter
                        )
                    else:
                        comprob = "NNN"
                    string += comprob
                    # 6  campo: largo 12 nro de comprobante
                    if tipo_operacion == "1":
                        num_cbte = ("0001" + apunte.name).zfill(11)
                    elif tipo_operacion == "2":
                        if (
                            len(
                                apunte.move_id.l10n_latam_document_number.replace(
                                    "-", ""
                                )
                            )
                            == 12
                        ):
                            num_cbte = (
                                apunte.move_id.l10n_latam_document_number.replace(
                                    "-", ""
                                ).zfill(11)
                            )
                        elif (
                            len(
                                apunte.move_id.l10n_latam_document_number.replace(
                                    "-", ""
                                )
                            )
                            > 12
                        ):
                            num_cbte = (
                                apunte.move_id.l10n_latam_document_number.replace(
                                    "-", ""
                                )[1:13]
                            )
                    else:
                        num_cbte = "NNNNNNNNNNNN"
                    if len(num_cbte) == 12:
                        string += "    " + num_cbte
                    else:
                        string += num_cbte.zfill(16)
                    # 7  campo: largo 10 fecha
                    d = str(apunte.date.day).zfill(2)
                    m = str(apunte.date.month).zfill(2)
                    string += str(d) + "/" + str(m) + "/" + str(apunte.date.year)
                    # DIVIDO EL CODIGO
                    if tipo_operacion == "1":
                        # CALCULO TOTALES DE TODOO (Monto,Importe otros conceptos,Importe IVA,Monto Sujeto a Retención/ Percepción,Alícuota,Retención/Percepción Practicada)
                        imp_caba = apunte.credit
                        # Calculo imp iva
                        imp_iva = "000000000,00"
                        # Calculo imp otros
                        imp_otros = "000000000,00"
                        alicuota = round(
                            (imp_caba / apunte.move_id.amount_untaxed * 100), 2
                        )
                        # Calculo monto sujeto
                        alicuota = str(alicuota).replace(",", ".")
                        monto_sujeto = round(float(imp_caba) * 100 / float(alicuota), 2)
                        parte_decimal, parte_entera = math.modf(monto_sujeto)
                        parte_entera = str(parte_entera).split(".")[0]
                        parte_decimal = (
                            str(round(parte_decimal, 2)).split(".")[1][:2]
                        ).ljust(2, "0")
                        parte_entera = parte_entera.zfill(11)
                        monto_sujeto = (parte_entera + "," + parte_decimal).zfill(14)
                        parte_decimal, parte_entera = math.modf(imp_caba)
                        parte_entera = str(parte_entera).split(".")[0]
                        parte_decimal = (
                            str(round(parte_decimal, 2)).split(".")[1][:2]
                        ).ljust(2, "0")
                        imp_caba = (parte_entera + "," + parte_decimal).zfill(14)
                        parte_decimal, parte_entera = math.modf(float(alicuota))
                        parte_entera = str(parte_entera).split(".")[0]
                        parte_decimal = (
                            str(round(parte_decimal, 2)).split(".")[1][:2]
                        ).ljust(2, "0")
                        # alicuota = (parte_entera + ',' + parte_decimal).zfill(5)
                        # Calculo monto sujeto
                        monto_total = monto_sujeto.zfill(14)
                        ######################################################################
                        # 8  campo: largo 14 monto del comprobante
                        string += monto_total
                        # 9 campo: largo 01 tipo de doc del retenido (1: CDI 2: CUIL 3:CUIT)
                        if (
                            apunte.partner_id.l10n_latam_identification_type_id.name
                            == "CDI"
                        ):
                            tipo_doc = "1"
                        elif (
                            apunte.partner_id.l10n_latam_identification_type_id.name
                            == "CUIL"
                        ):
                            tipo_doc = "2"
                        elif (
                            apunte.partner_id.l10n_latam_identification_type_id.name
                            == "CUIT"
                        ):
                            tipo_doc = "3"
                        else:
                            tipo_doc = "N"
                        string += tipo_doc
                        # 10 campo: largo 11 nro de documento del retenido
                        string += apunte.partner_id.vat.zfill(11)
                        # 11 campo: largo 01 situacion IB del retenido (1:local 2:convenio multilateral 4:no inscripto 5: reg. simplificado) si tipo doc 1 o 2 =4
                        if tipo_doc in ["1", "4"]:
                            string += "4"
                        else:
                            string += "4"
                        # 12 campo: largo 10 nro inscripcion ID del Retenido  Si Situación IB del Retenido=4 : 00000000000
                        string += "0" * 10
                        # 13 campo: largo 01 situacion frente al iva del retenido (1 -RI 2- Exento 3-monotributo)
                        if (
                            apunte.partner_id.l10n_ar_afip_responsibility_type_id.id
                            in [1, 2, 12]
                        ):
                            string += "1"
                        elif (
                            apunte.partner_id.l10n_ar_afip_responsibility_type_id.id
                            in [5]
                        ):
                            string += "2"
                        elif (
                            apunte.partner_id.l10n_ar_afip_responsibility_type_id.id
                            in [7, 14]
                        ):
                            string += "3"
                        else:
                            string += "N"
                        # 13 Marca inscripcion otros gravamenes
                        string += "0"
                        # 14 Marca inscripcion DRel
                        if apunte.partner_id.drei == "Activo":
                            string += "1"
                        else:
                            string += "0"
                        # 15  campo: largo 9,2 importe otros gravamenes
                        string += imp_otros
                        # 16 campo: largo 9,2 importe iva (completar si letra comp =A,M)
                        string += imp_iva
                        # 17 campo: largo 11,2 Monto sujeto a retencion/ percepcion (Monto Sujeto a Retención/ Percepción=(Monto del comprobante - Importe Iva -Importe otros conceptos))
                        string += monto_sujeto
                        # 19 campo: largo 05 Alícuota
                        alic = 0
                        for taxid in apunte.move_id.line_ids.filtered(
                            (
                                lambda r: r.name == "Percepción IIBB Santa Fe Aplicada"
                                or r.name == "Percepción IIBB SANTA FE Aplicada"
                            )
                        ):
                            alic = taxid.debit
                        alic = str(alic).replace(".", ",")
                        string += str(alic).zfill(5)
                        # 20 campo: largo 14 Retención/Percepción Practicada
                        string += imp_caba
                        # 21 campo: Derecho registro e inspeccion ( Completar con ceros )
                        string += "0".zfill(12)
                        # 22 campo: largo 14 Monto Total Retenido/Percibido (igual a campo 20)
                        string += imp_caba.zfill(14)
                        # 23 Articulo/Inciso para calculo 3 caracteres numerico
                        api_articulo = ""
                        if (
                            apunte.partner_id.arba_alicuot_ids.api_articulo_inciso_calculo_percepcion
                        ):
                            api_articulo = (
                                apunte.partner_id.arba_alicuot_ids.api_articulo_inciso_calculo_percepcion
                            )
                        string += api_articulo.zfill(3)
                        # 24 Tipo de exencion
                        if (
                            apunte.partner_id.arba_alicuot_ids.type_exemption
                            == "unexencion"
                        ):
                            string += "0"
                        elif (
                            apunte.partner_id.arba_alicuot_ids.type_exemption
                            == "cert_unretencion"
                        ):
                            string += "1"
                        elif (
                            apunte.partner_id.arba_alicuot_ids.type_exemption
                            == "cert_exencion"
                        ):
                            string += "2"
                        elif (
                            apunte.partner_id.arba_alicuot_ids.type_exemption
                            == "cert_unpercepcion"
                        ):
                            string += "3"
                        elif (
                            apunte.partner_id.arba_alicuot_ids.type_exemption
                            == "articule3"
                        ):
                            string += "4"
                        # 25) Año de exencion
                        if (
                            apunte.partner_id.arba_alicuot_ids.type_exemption
                            != "unexencion"
                        ):
                            string += str(
                                apunte.partner_id.arba_alicuot_ids.year_exemption
                            )
                        else:
                            string += "    "
                        # 26 Numero de certificado de exencion
                        if (
                            apunte.partner_id.arba_alicuot_ids.type_exemption
                            != "unexencion"
                        ):
                            string += str(
                                apunte.partner_id.arba_alicuot_ids.number_certificate
                            ).zfill(6)
                        else:
                            string += "      "
                        # 27 Numero de Certificado Propio
                        string += "0".zfill(12)
                    elif tipo_operacion == "2":
                        # CALCULO TOTALES DE TODOO (Monto,Importe otros conceptos,Importe IVA,Monto Sujeto a Retención/ Percepción,Alícuota,Retención/Percepción Practicada)
                        if tipo_sale == "1":
                            parte_decimal, parte_entera = math.modf(apunte.credit)
                            parte_entera = str(parte_entera).split(".")[0]
                            parte_decimal = (
                                str(round(parte_decimal, 2)).split(".")[1][:2]
                            ).ljust(2, "0")
                            imp_caba = str()
                        elif tipo_sale == "2":
                            imp_caba = str(apunte.debit)
                        # Calculo imp iva
                        imp_iva = "000000000,00"
                        # Calculo imp otros
                        imp_otros = "000000000,00"
                        parte_decimal, parte_entera = math.modf(
                            apunte.move_id.amount_total
                        )
                        parte_entera = str(parte_entera).split(".")[0]
                        parte_decimal = (
                            str(round(parte_decimal, 2)).split(".")[1][:2]
                        ).ljust(2, "0")
                        amount_total = parte_entera + "," + parte_decimal
                        mont_com = str(amount_total)
                        # Calculo monto sujeto
                        monto_total2 = 0
                        if tipo_sale == "1":
                            monto_total2 = mont_com.zfill(14)
                        elif tipo_sale == "2":
                            monto_total2 = "-" + mont_com.zfill(13)
                        # 8  campo: largo 14 monto del comprobante
                        string += monto_total2
                        # 9 campo: largo 01 tipo de doc del retenido (1: CDI 2: CUIL 3:CUIT)
                        if (
                            apunte.partner_id.l10n_latam_identification_type_id.name
                            == "CDI"
                        ):
                            tipo_doc = "1"
                        elif (
                            apunte.partner_id.l10n_latam_identification_type_id.name
                            == "CUIL"
                        ):
                            tipo_doc = "2"
                        elif (
                            apunte.partner_id.l10n_latam_identification_type_id.name
                            == "CUIT"
                        ):
                            tipo_doc = "3"
                        else:
                            tipo_doc = "N"
                        string += tipo_doc
                        # 10 campo: largo 11 nro de documento del retenido
                        string += apunte.partner_id.vat.zfill(11)
                        # 11 campo: largo 01 situacion IB del retenido (1:local 2:convenio multilateral 4:no inscripto 5: reg. simplificado) si tipo doc 1 o 2 =4
                        if tipo_doc in ["1", "3"]:
                            string += tipo_doc
                        else:
                            string += "4"
                        # 12 campo: largo 10 nro inscripcion ID del Retenido  Si Situación IB del Retenido=4 : 00000000000
                        string += "0" * 10
                        # 13 campo: largo 01 situacion frente al iva del retenido (1 -RI 2- Exento 3-monotributo)
                        if (
                            apunte.partner_id.l10n_ar_afip_responsibility_type_id.id
                            in [1, 2, 12]
                        ):
                            string += "1"
                        elif (
                            apunte.partner_id.l10n_ar_afip_responsibility_type_id.id
                            in [5]
                        ):
                            string += "2"
                        elif (
                            apunte.partner_id.l10n_ar_afip_responsibility_type_id.id
                            in [7, 14]
                        ):
                            string += "3"
                        else:
                            string += "N"
                        # 13 Marca inscripcion otros gravamenes
                        string += "0"
                        # 14 Marca inscripcion DRel
                        if apunte.partner_id.drei == "Activo":
                            string += "1"
                        else:
                            string += "0"
                        # 15  campo: largo 9,2 importe otros gravamenes
                        string += imp_otros
                        # 16 campo: largo 9,2 importe iva (completar si letra comp =A,M)
                        string += imp_iva
                        # 17 campo: largo 11,2 Monto sujeto a retencion/ percepcion (Monto Sujeto a Retención/ Percepción=(Monto del comprobante - Importe Iva -Importe otros conceptos))
                        if tipo_sale == "1":
                            parte_decimal, parte_entera = math.modf(taxid.credit)
                        elif tipo_sale == "2":
                            parte_decimal, parte_entera = math.modf(taxid.debit)
                        parte_entera = str(parte_entera).split(".")[0]
                        parte_decimal = (
                            str(round(parte_decimal, 2)).split(".")[1][:2]
                        ).ljust(2, "0")
                        imp_per = parte_entera + "," + parte_decimal
                        if tipo_sale == "1":
                            string += imp_per.zfill(14)
                        elif tipo_sale == "2":
                            string += "-" + imp_per.zfill(13)
                        # 19 campo: largo 05 Alícuota apunte.move_id.line_ids[0].name
                        alic = str(alic).replace(".", ",")
                        string += str(alic).zfill(5)
                        # 20 campo: largo 14 Retención/Percepción Practicada
                        string += str(imp_caba).zfill(14)
                        # 21 campo: Derecho registro e inspeccion ( Completar con ceros )
                        string += "000000000,00"
                        # 22 campo: largo 14 Monto Total Retenido/Percibido (igual a campo 20) aca
                        if (
                            apunte.move_id.l10n_latam_document_type_id.doc_code_prefix[
                                0:2
                            ]
                            == "FA"
                        ):
                            comprob = (
                                "FA_"
                                + apunte.move_id.l10n_latam_document_type_id.l10n_ar_letter
                            )  # _id.name
                        elif (
                            apunte.move_id.l10n_latam_document_type_id.doc_code_prefix[
                                0:2
                            ]
                            == "ND"
                        ):
                            comprob = (
                                "ND_"
                                + apunte.move_id.l10n_latam_document_type_id.l10n_ar_letter
                            )
                        elif (
                            apunte.move_id.l10n_latam_document_type_id.doc_code_prefix[
                                0:2
                            ]
                            == "NC"
                        ):
                            comprob = (
                                "NC_"
                                + apunte.move_id.l10n_latam_document_type_id.l10n_ar_letter
                            )
                        else:
                            comprob = "NN_N"
                        # for taxid in apunte.move_id.line_ids.filtered(lambda r: r.name == 'Percepción IIBB Santa Fe Aplicada' or r.name == 'Percepción IIBB SANTA FE Aplicada'):
                        if tipo_sale == "1":
                            parte_decimal, parte_entera = math.modf(taxid.credit)
                        elif tipo_sale == "2":
                            parte_decimal, parte_entera = math.modf(taxid.debit)
                        parte_entera = str(parte_entera).split(".")[0]
                        parte_decimal = (
                            str(round(parte_decimal, 2)).split(".")[1][:2]
                        ).ljust(2, "0")
                        imp_per = parte_entera + "," + parte_decimal
                        if tipo_sale == "1":
                            string += imp_per.zfill(14)
                        elif tipo_sale == "2":
                            string += "-" + imp_per.zfill(13)
                        # 23 Articulo/Inciso para calculo 3 caracteres numerico
                        api_articulo = ""
                        for (
                            arba_alicuot_id
                        ) in apunte.partner_id.arba_alicuot_ids.filtered(
                            lambda v: v.tag_id.name == "Jur: 921 - Santa Fe"
                        ):
                            if arba_alicuot_id.api_articulo_inciso_calculo_percepcion:
                                api_articulo = (
                                    arba_alicuot_id.api_articulo_inciso_calculo_percepcion
                                )
                            string += api_articulo.zfill(3)
                            # 24 Tipo de exencion
                            if arba_alicuot_id.type_exemption == "unexencion":
                                string += "0"
                            elif arba_alicuot_id.type_exemption == "cert_unretencion":
                                string += "1"
                            elif arba_alicuot_id.type_exemption == "cert_exencion":
                                string += "2"
                            elif arba_alicuot_id.type_exemption == "cert_unpercepcion":
                                string += "3"
                            elif arba_alicuot_id.type_exemption == "articule3":
                                string += "4"
                            # 25) Año de exencion
                            if arba_alicuot_id.type_exemption != "unexencion":
                                string += str(arba_alicuot_id.year_exemption)
                            else:
                                string += "0000"
                            # 26 Numero de certificado de exencion
                            if (
                                arba_alicuot_id.type_exemption != "unexencion"
                            ):  # apunte.partner_id.arba_alicuot_ids.type_exemption
                                string += str(arba_alicuot_id.number_certificate).zfill(
                                    6
                                )
                            else:
                                string += "      "
                            # 27 Numero de Certificado Propio r.name == 'Percepción IIBB Santa Fe Aplicada' or r.name == 'Percepción IIBB SANTA FE Aplicada'
                            string += "0".zfill(12)
                    string += windows_line_ending
        payments = self.env["account.payment"].search(
            [
                ("payment_type", "=", "outbound"),
                ("tax_withholding_id.name", "=", "Retención IIBB SANTA FE Aplicada"),
                ("state", "not in", ["cancelled", "draft"]),
                ("payment_group_id.payment_date", "<=", self.fecha_hasta),
                ("payment_group_id.payment_date", ">=", self.fecha_desde),
            ],
            order="date",
        )
        imp_caba = 0.00
        tipo_operacion = "1"
        pay = ""
        for payment in payments:
            if tipo_operacion == "1":
                pay = payment.payment_group_id
                # 1  campo: largo 01 tipo    operación (1:retencion 2:percepcion)
                monto_sujeto = str(payment.computed_withholding_amount)
                string += tipo_operacion
                # 2  campo: largo 10 fecha de retencion/percepcion
                d = str(pay.payment_date.day).zfill(2)
                m = str(pay.payment_date.month).zfill(2)
                string += str(d) + "/" + str(m) + "/" + str(pay.payment_date.year)
                # 3  campo: largo 03 codigo articulo inciso por el que retiene
                api_codigo = ""
                for arba_alicuot_id in pay.partner_id.arba_alicuot_ids.filtered(
                    lambda v: v.tag_id.name == "Jur: 921 - Santa Fe"
                ):
                    if arba_alicuot_id.api_codigo_articulo_retencion:
                        api_codigo = arba_alicuot_id.api_codigo_articulo_retencion
                    if arba_alicuot_id.alicuota_retencion != "0.00":
                        alic = arba_alicuot_id.alicuota_retencion
                string += api_codigo.zfill(3)
                # 4  campo: largo 02 tipo cbte origen de la ret (si tipo op =1 :01 fac 02 ndeb 03 ordenpago 04 boleta de dep 05 liq de pago 06 cert de obra 07 recibo 08 con de loc de serv 09 otro comb si tipo op =2 01 factura 09 otro comp)
                # 5  campo: largo 01 letra del comprobante (A,M,B,C o espacio blanco)
                if tipo_operacion == "1":
                    comprob = "03 "
                string += comprob
                # 6  campo: largo 12 nro de comprobante
                num_cbte = ""
                num_cbte = (pay.name).zfill(11)
                if len(num_cbte) == 12:
                    string += "    " + num_cbte
                else:
                    string += num_cbte.zfill(16)
                # 7  campo: largo 10 fecha
                d = str(pay.payment_date.day).zfill(2)
                m = str(pay.payment_date.month).zfill(2)
                string += str(d) + "/" + str(m) + "/" + str(pay.payment_date.year)
                # CALCULO TOTALES DE TODOO (Monto,Importe otros conceptos,Importe IVA,Monto Sujeto a Retención/ Percepción,Alícuota,Retención/Percepción Practicada)
                parte_decimal, parte_entera = math.modf(
                    payment.computed_withholding_amount
                )
                parte_entera = str(parte_entera).split(".")[0]
                parte_decimal = (str(round(parte_decimal, 2)).split(".")[1][:2]).ljust(
                    2, "0"
                )
                imp_caba = (parte_entera + "," + parte_decimal).zfill(14)
                # Calculo imp iva
                imp_iva = "000000000,00"
                # Calculo imp otros
                imp_otros = "000000000,00"
                parte_decimal, parte_entera = math.modf(payment.amount)
                parte_entera = str(parte_entera).split(".")[0]
                parte_decimal = (str(round(parte_decimal, 2)).split(".")[1][:2]).ljust(
                    2, "0"
                )
                amount_total = (parte_entera + "," + parte_decimal).zfill(14)
                mont_com = str(amount_total)
                # Calculo monto sujeto
                monto_total2 = 0
                monto_total2 = mont_com.zfill(14)
                # 8  campo: largo 14 monto del comprobante
                string += monto_total2
                # 9 campo: largo 01 tipo de doc del retenido (1: CDI 2: CUIL 3:CUIT)
                if pay.partner_id.l10n_latam_identification_type_id.name == "CDI":
                    tipo_doc = "1"
                elif pay.partner_id.l10n_latam_identification_type_id.name == "CUIL":
                    tipo_doc = "2"
                elif pay.partner_id.l10n_latam_identification_type_id.name == "CUIT":
                    tipo_doc = "3"
                else:
                    tipo_doc = "N"
                string += tipo_doc
                # 10 campo: largo 11 nro de documento del retenido
                string += pay.partner_id.vat.zfill(11)
                # 11 campo: largo 01 situacion IB del retenido (1:local 2:convenio multilateral 4:no inscripto 5: reg. simplificado) si tipo doc 1 o 2 =4
                if tipo_doc in ["1", "3"]:
                    string += tipo_doc
                else:
                    string += "4"
                # 12 campo: largo 10 nro inscripcion ID del Retenido  Si Situación IB del Retenido=4 : 00000000000
                string += "0" * 10
                # 13 campo: largo 01 situacion frente al iva del retenido (1 -RI 2- Exento 3-monotributo)
                if pay.partner_id.l10n_ar_afip_responsibility_type_id.id in [1, 2, 12]:
                    string += "1"
                elif pay.partner_id.l10n_ar_afip_responsibility_type_id.id in [5]:
                    string += "2"
                elif pay.partner_id.l10n_ar_afip_responsibility_type_id.id in [7, 14]:
                    string += "3"
                else:
                    string += "N"
                # 13 Marca inscripcion otros gravamenes
                string += "0"
                # 14 Marca inscripcion DRel
                if pay.partner_id.drei == "Activo":
                    string += "1"
                else:
                    string += "0"
                # 15  campo: largo 9,2 importe otros gravamenes
                string += imp_otros
                # 16 campo: largo 9,2 importe iva (completar si letra comp =A,M)
                string += imp_iva
                # 17 campo: largo 11,2 Monto sujeto a retencion/ percepcion (Monto Sujeto a Retención/ Percepción=(Monto del comprobante - Importe Iva -Importe otros conceptos))
                parte_decimal, parte_entera = math.modf(
                    payment.computed_withholding_amount
                )
                parte_entera = str(parte_entera).split(".")[0]
                parte_decimal = (str(round(parte_decimal, 2)).split(".")[1][:2]).ljust(
                    2, "0"
                )
                imp_per = parte_entera + "," + parte_decimal
                string += imp_per.zfill(14)
                # 19 campo: largo 05 Alícuota apunte.move_id.line_ids[0].name
                # for taxid in apunte.move_id.line_ids.filtered((lambda r: r.name == 'Percepción IIBB Santa Fe Aplicada' or r.name == 'Percepción IIBB SANTA FE Aplicada')):
                #    alic = taxid.credit
                alic = str(alic).replace(".", ",")
                string += str(alic).zfill(5)
                # 20 campo: largo 14 Retención/Percepción Practicada
                string += imp_caba
                # 21 campo: Derecho registro e inspeccion ( Completar con ceros )
                string += "000000000,00"
                # for taxid in apunte.move_id.line_ids.filtered(lambda r: r.name == 'Percepción IIBB Santa Fe Aplicada' or r.name == 'Percepción IIBB SANTA FE Aplicada'):
                parte_decimal, parte_entera = math.modf(
                    payment.computed_withholding_amount
                )
                parte_entera = str(parte_entera).split(".")[0]
                parte_decimal = (str(round(parte_decimal, 2)).split(".")[1][:2]).ljust(
                    2, "0"
                )
                imp_per = parte_entera + "," + parte_decimal
                string += imp_per.zfill(14)
                # 23 Articulo/Inciso para calculo 3 caracteres numerico
                api_articulo = ""
                for arba_alicuot_id in pay.partner_id.arba_alicuot_ids.filtered(
                    lambda v: v.tag_id.name == "Jur: 921 - Santa Fe"
                ):
                    if arba_alicuot_id.api_articulo_inciso_calculo_retencion:
                        api_articulo = (
                            arba_alicuot_id.api_articulo_inciso_calculo_retencion
                        )
                    elif arba_alicuot_id.api_articulo_inciso_calculo_percepcion:
                        api_articulo = (
                            arba_alicuot_id.api_articulo_inciso_calculo_percepcion
                        )
                    string += api_articulo.zfill(3)
                    # 24 Tipo de exencion
                    if arba_alicuot_id.type_exemption == "unexencion":
                        string += "0"
                    elif arba_alicuot_id.type_exemption == "cert_unretencion":
                        string += "1"
                    elif arba_alicuot_id.type_exemption == "cert_exencion":
                        string += "2"
                    elif arba_alicuot_id.type_exemption == "cert_unpercepcion":
                        string += "3"
                    elif arba_alicuot_id.type_exemption == "articule3":
                        string += "4"
                    # 25) Año de exencion
                    if arba_alicuot_id.type_exemption != "unexencion":
                        string += str(arba_alicuot_id.year_exemption)
                    else:
                        string += "0000"
                    # 26 Numero de certificado de exencion
                    if (
                        arba_alicuot_id.type_exemption != "unexencion"
                    ):  # apunte.partner_id.arba_alicuot_ids.type_exemption
                        string += str(arba_alicuot_id.number_certificate).zfill(6)
                    else:
                        string += "      "
                    # 27 Numero de Certificado Propio r.name == 'Percepción IIBB Santa Fe Aplicada' or r.name == 'Percepción IIBB SANTA FE Aplicada'
                    string += "0".zfill(12)
                string += windows_line_ending
        string = string.encode("utf-8")
        return string

    def generar_archivo(self):
        """Segun Impuesto seleccionado, genera TXT para correspondiente presentación"""
        string = ""
        company = self.env.company
        if self.tipo_txt == "piibbcabaanc":
            # Llama al metodo _tax_piibbcabaanc que genera TXT, para presentar impuesto Percepción IIBB CABA Aplicada (Notas de Crédito)
            apuntes = self.env["account.move.line"].search(
                [
                    ("date", ">=", self.fecha_desde),
                    ("date", "<=", self.fecha_hasta),
                    ("move_id.state", "not in", ["cancelled", "draft"]),
                    ("tax_ids.tipo_txt", "=", "piibbcabaa"),
                    ("company_id", "=", company.id),
                ],
                order="date",
            )
            string = self._tax_piibbcabaanc(apuntes=apuntes)
        elif self.tipo_txt == "priibbcabaa":
            # Llama al metodo _tax_priibbcabaa que genera TXT, para presentar impuesto Percepción/Retención IIBB CABA Aplicada
            apuntes2 = self.env["account.move.line"].search(
                [
                    ("date", ">=", self.fecha_desde),
                    ("date", "<=", self.fecha_hasta),
                    ("tax_line_id.tipo_txt", "in", ["piibbcabaa", "riibbcabaa"]),
                    ("move_id.state", "not in", ["cancelled", "draft"]),
                    ("company_id", "=", company.id),
                ],
                order="date",
            )
            # ('tax_ids.name','in',['Percepción IIBB CABA Aplicada', 'Retención IIBB CABA Aplicada'])  'piibbcabaa',
            string = self._tax_priibbcabaa(apuntes2=apuntes2)
        elif self.tipo_txt == "sicore":
            # Llama al metodo _tax_sicore que genera TXT, para presentar Sicore
            self.ensure_one()
            payments = self.env["account.payment"].search(
                [
                    ("payment_type", "=", "outbound"),
                    ("journal_id.name", "ilike", "Retenciones Aplicadas"),
                    ("tax_withholding_id.name", "ilike", "Retenciones de Ganancias"),
                    ("state", "not in", ["cancelled", "draft"]),
                    ("date", "<=", self.fecha_hasta),
                    ("date", ">=", self.fecha_desde),
                    ("company_id", "=", company.id),
                ],
                order="date",
            )
            string = self._tax_sicore(payments=payments)
        elif self.tipo_txt == "sicoreiva":
            # Llama al metodo _tax_sicoreiva que genera TXT, para presentar Sicore de IVA
            self.ensure_one()
            payments = self.env["account.payment"].search(
                [
                    ("payment_type", "=", "outbound"),
                    ("journal_id.name", "ilike", "Retenciones Aplicadas"),
                    ("tax_withholding_id.tipo_txt", "=", "sicoreiva"),
                    ("state", "not in", ["cancelled", "draft"]),
                    ("payment_group_id.payment_date", "<=", self.fecha_hasta),
                    ("payment_group_id.payment_date", ">=", self.fecha_desde),
                    ("company_id", "=", company.id),
                ],
                order="date",
            )
            string = self._tax_sicoreiva(payments=payments)
        elif self.tipo_txt == "sifereperc":
            # Llama al metodo _tax_sifereperc que genera TXT, para presentar impuesto Sifere Percepciones Sufridas
            apuntes = self.env["account.move.line"].search(
                [
                    ("date", ">=", self.fecha_desde),
                    ("date", "<=", self.fecha_hasta),
                    ("move_id.state", "not in", ["cancelled", "draft"]),
                    ("tax_ids.tipo_txt", "in", ["sifereperc"]),
                    ("company_id", "=", company.id),
                ],
                order="date",
            )
            string = self._tax_sifereperc(apuntes=apuntes)
        elif self.tipo_txt == "sifereret":
            # Llama al metodo _tax_sifereret que genera TXT, para presentar impuesto Sifere Retenciones Sufridas
            payments = self.env["account.payment"].search(
                [
                    ("payment_type", "=", "inbound"),
                    ("journal_id.name", "ilike", "Retenciones Sufridas"),
                    ("state", "not in", ["cancelled", "draft"]),
                    ("date", "<=", self.fecha_hasta),
                    ("date", ">=", self.fecha_desde),
                    ("company_id", "=", company.id),
                ],
                order="date",
            )
            string = self._tax_sifereret(payments=payments)
        elif self.tipo_txt == "tucumandatos":
            # Llama al metodo _tax_tucumandatos que genera TXT, para presentar impuesto Tucuman DATOS.TXT
            apuntes = self.env["account.move.line"].search(
                [
                    ("date", ">=", self.fecha_desde),
                    ("date", "<=", self.fecha_hasta),
                    ("move_id.state", "not in", ["cancelled", "draft"]),
                    ("tax_ids.tucuman_datos_txt", "=", True),
                    ("company_id", "=", company.id),
                ],
                order="date",
            )
            string = self._tax_tucumandatos(apuntes=apuntes)
        elif self.tipo_txt == "tucumanretper":
            # Llama al metodo _tax_tucumanretper que genera TXT, para presentar impuesto Tucuman RETPER.TXT
            apuntes = self.env["account.move.line"].search(
                [
                    ("date", ">=", self.fecha_desde),
                    ("date", "<=", self.fecha_hasta),
                    ("move_id.state", "not in", ["cancelled", "draft"]),
                    ("tax_ids.tucuman_retper_txt", "=", True),
                    ("company_id", "=", company.id),
                ],
                order="date",
            )
            string = self._tax_tucumanretper(apuntes=apuntes)
        elif self.tipo_txt == "tucumanncfact":
            # Llama al metodo _tax_tucumanncfact que genera TXT, para presentar impuesto Tucuman NCFACT.TXT
            apuntes = self.env["account.move.line"].search(
                [
                    ("date", ">=", self.fecha_desde),
                    ("date", "<=", self.fecha_hasta),
                    ("move_id.state", "not in", ["cancelled", "draft"]),
                    ("tax_ids.tucuman_ncfact_txt", "=", True),
                    ("company_id", "=", company.id),
                ],
                order="date",
            )
            string = self._tax_tucumanncfact(apuntes=apuntes)
        elif self.tipo_txt == "misionesperc":
            # Llama al metodo _tax_misionesperc que genera TXT, para presentar impuesto Percepción Misiones
            apuntes = self.env["account.move.line"].search(
                [
                    ("date", ">=", self.fecha_desde),
                    ("date", "<=", self.fecha_hasta),
                    ("move_id.state", "not in", ["cancelled", "draft"]),
                    ("tax_ids.tipo_txt", "=", "misionesperc"),
                    ("company_id", "=", company.id),
                ],
                order="date",
            )
            string = self._tax_misionesperc(apuntes=apuntes)
        elif self.tipo_txt == "santafepercret":
            # Llama al metodo _tax_santafe que genera TXT, para presentar impuesto Perc/Ret Santa Fe
            apuntes = self.env["account.move.line"].search(
                [
                    ("date", ">=", self.fecha_desde),
                    ("date", "<=", self.fecha_hasta),
                    ("move_id.state", "not in", ["cancelled", "draft"]),
                    ("tax_ids.tipo_txt", "in", ["percsantafe", "retsantafe"]),
                    ("company_id", "=", company.id),
                ],
                order="date",
            )
            string = self._tax_santafe(apuntes=apuntes)
        elif self.tipo_txt == "misionesret":
            # Llama al metodo _tax_misionesret que genera TXT, para presentar impuesto Retención Misiones
            payments = self.env["account.payment"].search(
                [
                    ("payment_type", "=", "outbound"),
                    ("journal_id.name", "ilike", "Retenciones Aplicadas"),
                    ("tax_withholding_id.tipo_txt", "=", "misionesret"),
                    ("state", "not in", ["cancelled", "draft"]),
                    ("date", "<=", self.fecha_hasta),
                    ("date", ">=", self.fecha_desde),
                    ("company_id", "=", company.id),
                ],
                order="date",
            )
            string = self._tax_misionesret(apuntes=payments)
        elif self.tipo_txt == "riibbarbaa":
            # Llama al metodo _tax_riibbarbaa que genera TXT, para presentar impuesto Retención IIBB ARBA Aplicada
            payments = self.env["account.payment"].search(
                [
                    ("payment_type", "=", "outbound"),
                    ("journal_id.name", "ilike", "Retenciones Aplicadas"),
                    (
                        "tax_withholding_id.name",
                        "ilike",
                        "Retención IIBB ARBA Aplicada",
                    ),
                    ("state", "not in", ["cancelled", "draft"]),
                    ("date", "<=", self.fecha_hasta),
                    ("date", ">=", self.fecha_desde),
                    ("company_id", "=", company.id),
                ],
                order="date",
            )
            string = self._tax_riibbarbaa(apuntes=payments)
        elif self.tipo_txt == "piibbarbaa":
            # Llama al metodo _tax_piibbarbaa que genera TXT, para presentar impuesto Percepción IIBB ARBA Aplicada
            apuntes = self.env["account.move.line"].search(
                [
                    ("date", ">=", self.fecha_desde),
                    ("date", "<=", self.fecha_hasta),
                    ("move_id.state", "not in", ["cancelled", "draft"]),
                    ("tax_ids.tipo_txt", "=", self.tipo_txt),
                    ("company_id", "=", company.id),
                ],
                order="date",
            )
            string = self._tax_piibbarbaa(apuntes=apuntes)
        # tucumanpercsufrida
        elif self.tipo_txt == "tucumanpercsufrida":
            # Llama al metodo _tax_piibbarbaa que genera TXT, para presentar impuesto Percepción IIBB ARBA Aplicada
            apuntes = self.env["account.move.line"].search(
                [
                    ("date", ">=", self.fecha_desde),
                    ("date", "<=", self.fecha_hasta),
                    ("move_id.state", "not in", ["cancelled", "draft"]),
                    ("tax_ids.siapre_txt", "in", ["tucumanpercsufrida"]),
                    ("company_id", "=", company.id),
                ],
                order="date",
            )
            string = self._tax_tucumanpercsufrida(apuntes=apuntes)
        # tucumanretsufrida
        elif self.tipo_txt == "tucumanretsufrida":
            # Llama al metodo _tax_tucumanretsufrida que genera TXT, para presentar impuesto Retencion tucuman Sufrida
            payments = self.env["account.payment"].search(
                [
                    ("payment_type", "=", "inbound"),
                    ("journal_id.name", "ilike", "Retenciones Sufridas"),
                    ("state", "not in", ["cancelled", "draft"]),
                    ("date", "<=", self.fecha_hasta),
                    ("date", ">=", self.fecha_desde),
                    ("company_id", "=", company.id),
                ],
                order="date",
            )
            string = self._tax_tucumanretsufrida(payments=payments)
        # percsufridaIVA
        elif self.tipo_txt == "perceivasufrida":
            # Llama al metodo _tax_piibbarbaa que genera TXT, para presentar impuesto Percepción IIBB ARBA Aplicada
            apuntes = self.env["account.move.line"].search(
                [
                    ("date", ">=", self.fecha_desde),
                    ("date", "<=", self.fecha_hasta),
                    ("move_id.state", "not in", ["cancelled", "draft"]),
                    ("tax_ids.tipo_txt", "in", ["perceivasufrida"]),
                    ("company_id", "=", company.id),
                ],
                order="date",
            )
            string = self._tax_perceivasufrida(apuntes=apuntes)
        # retensufridaIVA
        elif self.tipo_txt == "reteivasufrida":
            payments = self.env["account.payment"].search(
                [
                    ("payment_type", "=", "inbound"),
                    ("journal_id.name", "ilike", "Retenciones Sufridas"),
                    ("state", "not in", ["cancelled", "draft"]),
                    ("date", "<=", self.fecha_hasta),
                    ("date", ">=", self.fecha_desde),
                    ("company_id", "=", company.id),
                ],
                order="date",
            )
            string = self._tax_reteivasufrida(payments=payments)
        else:
            raise UserError(
                _("El formato de generación TXT seleccionado no está definido")
            )
        self.export_filename = NAME_FILE
        self.export_data = string
        self.export_file = base64.b64encode(self.export_data.encode("UTF-8"))
        return {
            "type": "ir.actions.client",
            "tag": "reload",
        }
