"""Wizard to generate lists of withholdings and perceptions"""
import base64
import io
import logging

from openpyxl import Workbook

from odoo import api, fields, models

_logger = logging.getLogger(__name__)


class WzdListRetPerv(models.TransientModel):
    """Wizard to generate lists of withholdings and perceptions"""

    _name = "nybble.wzd_list_ret_perc"
    _description = " Wizard to generate lists of withholdings and perceptions "

    name = fields.Char("Descripción", required=True)
    fecha_desde = fields.Date("Fecha Desde", required=True)
    fecha_hasta = fields.Date("Fecha Hasta", required=True)
    tipo_list = fields.Selection(
        [
            ("Retenciones SUSS", "Retenciones SUSS"),
            ("Retenciones DREI", "Retenciones DREI"),
            ("Retención Ganancias Sufrida", "Retención Ganancias Sufrida"),
            ("Retención Ganancias Aplicada", "Retención Ganancias Aplicada"),
            ("Retenciones IVA", "Retenciones IVA"),
            ("Retenciones Ingresos brutos", "Retenciones Ingresos brutos"),
            ("Percepción Ganancias Sufrida", "Percepción Ganancias Sufrida"),
            ("Percepción IVA Sufrida", "Percepción IVA Sufrida"),
            (
                "Percepción Ingresos brutos Sufrida",
                "Percepción Ingresos brutos Sufrida",
            ),
        ],
        "Tipo",
    )
    export_data = fields.Text(string="Contenido archivo")
    export_file = fields.Binary("Archivo")

    export_filename = fields.Char("Archivo nombre")
    NAME_FILE = None
    company_id = fields.Many2one(
        "res.company", string="Company", default=lambda self: self.env.company
    )

    @api.model
    def _tax_list_retencion(self, payments):
        """Generates data to present a list of withholdings in Excel."""
        data = []
        for payment in payments:
            row = [
                payment.date.strftime("%d/%m/%Y"),  # Date
                (payment.payment_group_id.name or ""),  # Associated document number
                (payment.partner_id.name or ""),  # partner
                (payment.partner_id.vat or ""),  # CUIT
                (payment.journal_id.name or ""),  # Journal
                (payment.tax_withholding_id.name or ""),  # Tax
                str(payment.withholding_number or ""),  # Withholding/perception number
                str(
                    payment.withholding_base_amount or ""
                ),  # Withholding/perception basis
                str(payment.amount or ""),  # Withholding/perception amount
            ]
            data.append(row)
        return data

    @api.model
    def _tax_list_percepcion(self, move_lines):
        """Generates data to present a list of perceptions in Excel."""
        data = []
        for move in move_lines:
            row = [
                move.date.strftime("%d/%m/%Y"),  # Date
                (move.move_id.name or ""),  # Associated document number
                (move.partner_id.name or ""),  # partner
                (move.partner_id.vat or ""),  # CUIT
                (move.move_id.journal_id.name or ""),  # Journal
                (move.tax_line_id.name or ""),  # Tax
                str(""),  # Withholding/perception number
                str(move.move_id.amount_untaxed or ""),  # Withholding/perception basis
                str(move.debit or move.credit),  # Withholding/perception amount
            ]
            data.append(row)
        return data

    def generar_archivo(self):
        """According to the selected tax, Excel generates a list of withholdings/perceptions"""
        workbook = Workbook()
        sheet = workbook.active

        data = []
        if self.tipo_list == "Retenciones SUSS":
            payments = self.env["account.payment"].search(
                [
                    ("payment_type", "=", "inbound"),
                    ("tax_withholding_id.tipo_list", "=", "Retenciones SUSS"),
                    ("state", "not in", ["cancelled", "draft"]),
                    ("date", "<=", self.fecha_hasta),
                    ("date", ">=", self.fecha_desde),
                ],
                order="date",
            )
            data = self._tax_list_retencion(payments=payments)
        elif self.tipo_list == "Retenciones DREI":
            payments = self.env["account.payment"].search(
                [
                    ("payment_type", "=", "inbound"),
                    ("tax_withholding_id.tipo_list", "=", "Retenciones DREI"),
                    ("state", "not in", ["cancelled", "draft"]),
                    ("date", "<=", self.fecha_hasta),
                    ("date", ">=", self.fecha_desde),
                ],
                order="date",
            )
            data = self._tax_list_retencion(payments=payments)
        elif self.tipo_list == "Retención Ganancias Sufrida":
            payments = self.env["account.payment"].search(
                [
                    ("payment_type", "=", "inbound"),
                    (
                        "tax_withholding_id.tipo_list",
                        "=",
                        "Retención Ganancias Sufrida",
                    ),
                    ("state", "not in", ["cancelled", "draft"]),
                    ("date", "<=", self.fecha_hasta),
                    ("date", ">=", self.fecha_desde),
                ],
                order="date",
            )
            data = self._tax_list_retencion(payments=payments)
        elif self.tipo_list == "Retención Ganancias Aplicada":
            payments = self.env["account.payment"].search(
                [
                    ("payment_type", "=", "outbound"),
                    (
                        "tax_withholding_id.tipo_list",
                        "=",
                        "Retención Ganancias Aplicada",
                    ),
                    ("state", "not in", ["cancelled", "draft"]),
                    ("date", "<=", self.fecha_hasta),
                    ("date", ">=", self.fecha_desde),
                ],
                order="date",
            )
            data = self._tax_list_retencion(payments=payments)
        elif self.tipo_list == "Retenciones IVA":
            payments = self.env["account.payment"].search(
                [
                    ("payment_type", "=", "inbound"),
                    ("tax_withholding_id.tipo_list", "=", "Retenciones IVA"),
                    ("state", "not in", ["cancelled", "draft"]),
                    ("date", "<=", self.fecha_hasta),
                    ("date", ">=", self.fecha_desde),
                ],
                order="date",
            )
            data = self._tax_list_retencion(payments=payments)
        elif self.tipo_list == "Retenciones Ingresos brutos":
            payments = self.env["account.payment"].search(
                [
                    ("payment_type", "=", "inbound"),
                    (
                        "tax_withholding_id.tipo_list",
                        "=",
                        "Retenciones Ingresos brutos",
                    ),
                    ("state", "not in", ["cancelled", "draft"]),
                    ("date", "<=", self.fecha_hasta),
                    ("date", ">=", self.fecha_desde),
                ],
                order="date",
            )
            data = self._tax_list_retencion(payments=payments)
        elif self.tipo_list == "Percepción Ganancias Sufrida":
            move_lines = self.env["account.move.line"].search(
                [
                    ("date", ">=", self.fecha_desde),
                    ("date", "<=", self.fecha_hasta),
                    ("move_id.state", "not in", ["cancelled", "draft"]),
                    ("tax_line_id.tipo_list", "=", "Percepción Ganancias Sufrida"),
                ],
                order="date",
            )
            data = self._tax_list_percepcion(move_lines=move_lines)
        elif self.tipo_list == "Percepción IVA Sufrida":
            move_lines = self.env["account.move.line"].search(
                [
                    ("date", ">=", self.fecha_desde),
                    ("date", "<=", self.fecha_hasta),
                    ("move_id.state", "not in", ["cancelled", "draft"]),
                    ("tax_line_id.tipo_list", "=", "Percepción IVA Sufrida"),
                ],
                order="date",
            )
            data = self._tax_list_percepcion(move_lines=move_lines)
        elif self.tipo_list == "Percepción Ingresos brutos Sufrida":
            move_lines = self.env["account.move.line"].search(
                [
                    ("date", ">=", self.fecha_desde),
                    ("date", "<=", self.fecha_hasta),
                    ("move_id.state", "not in", ["cancelled", "draft"]),
                    (
                        "tax_line_id.tipo_list",
                        "=",
                        "Percepción Ingresos brutos Sufrida",
                    ),
                ],
                order="date",
            )
            data = self._tax_list_percepcion(move_lines=move_lines)

        # Add header names
        header_row = [
            "Fecha",
            "Numero de documento asocidado",
            "Partner",
            "CUIT",
            "Diario",
            "Impuesto",
            "Numero de retencion / percepción",
            "Base de retención/ percepción",
            "Importe de retención/ percepción",
        ]
        sheet.append(header_row)

        # Add the data
        for row in data:
            sheet.append(row)

        buffer = io.BytesIO()
        workbook.save(buffer)
        buffer.seek(0)
        excel_content = buffer.getvalue()

        print("Contenido de excel_content:", excel_content)

        self.export_filename = "listado_retenciones_percepciones.xlsx"
        self.export_file = excel_content
        self.export_file = base64.b64encode(self.export_file)
        return {
            "type": "ir.actions.client",
            "tag": "reload",
        }
