from odoo import fields, models


class ResCompany(models.Model):
    """
    This model represents the company configuration settings for the internal vouchers module.
    It inherits from the base 'res.company' model.
    """

    _inherit = "res.company"

    internal_voucher_sale_tax = fields.Many2one(
        "account.tax",
        string="Tax for internal receipts - sales",
        domain=[("type_tax_use", "=", "sale")],
        company_dependent=True,
    )
    internal_voucher_purchase_tax = fields.Many2one(
        "account.tax",
        string="Tax for internal receipts - purchase",
        domain=[("type_tax_use", "=", "purchase")],
        company_dependent=True,
    )


class ResConfigSettings(models.TransientModel):
    """
    This model represents the configuration settings for the internal vouchers module.
    It inherits from the base 'res.config.settings' model.
    """

    _inherit = "res.config.settings"

    internal_voucher_sale_tax = fields.Many2one(
        "account.tax",
        string="Tax for internal receipts - sales",
        domain=[("type_tax_use", "=", "sale")],
        default=lambda self: self.env["account.tax"].search(
            [("type_tax_use", "=", "sale"), ("name", "=", "IVA No Gravado")], limit=1
        ),
        company_dependent=True,
    )
    internal_voucher_purchase_tax = fields.Many2one(
        "account.tax",
        string="Tax for internal receipts - purchase",
        domain=[("type_tax_use", "=", "purchase")],
        default=lambda self: self.env["account.tax"].search(
            [("type_tax_use", "=", "purchase"), ("name", "=", "IVA No Gravado")],
            limit=1,
        ),
        company_dependent=True,
    )

    def set_values(self):
        """
        Overrides the base 'set_values' method to set the values of the internal voucher taxes for the active company.
        """
        super(ResConfigSettings, self).set_values()

        active_company = self.env["res.company"].browse(
            self.env.context.get("active_id")
        )

        if active_company:
            # Get the record of the active company
            company = self.env["res.company"].browse(active_company.id)

            # Assign the values of the internal sales taxes
            if self.internal_voucher_sale_tax:
                company.write(
                    {
                        "internal_voucher_sale_tax": self.internal_voucher_sale_tax.id,
                    }
                )

            if self.internal_voucher_purchase_tax:
                company.write(
                    {
                        "internal_voucher_purchase_tax": self.internal_voucher_purchase_tax.id,
                    }
                )
