import xml.etree.cElementTree as ET

from odoo import api, fields, models


class StockPicking(models.Model):
    """Add provisional field in Delivery / Reception Order."""

    _inherit = ["stock.picking"]

    provisional = fields.Boolean(
        "On behalf of",
    )

    @api.model
    def get_view(self, view_id=None, view_type="form", **options):
        """Oculta el campo provisional para el grupo group_user."""
        res = super().get_view(view_id, view_type, **options)
        group_user = self.env.user.has_group(
            "nybble_internal_voucher.group_voucher_user"
        )
        group_manager = self.env.user.has_group(
            "nybble_internal_voucher.group_voucher_admin"
        )
        if not group_manager and (group_user):
            if view_type in ["form"]:
                root = ET.fromstring(res["arch"])
                for el in root.iter("field"):
                    if el.attrib.get("name") in ["provisional"]:
                        el.attrib.update(
                            {
                                "modifiers": '{"invisible": [["id", "!=", 0]]}',
                            }
                        )
                res.update({"arch": ET.tostring(root, encoding="utf8", method="xml")})
        return res
