"""Inherit product.pricelist model to add new fields"""
import math

from odoo import fields, models, api, _
from odoo.exceptions import ValidationError, UserError


class ProductPricelistItemInherit(models.Model):
    _inherit = "product.pricelist.item"

    '''
    @api.onchange("price_discount", "date_start", "dis_2", "dis_3", "dis_inc_SIA", "dis_franchise_SIA")
    def _onchange_discounts(self):
        """Recalculate the price when the vendor price or discounts change."""
        self._compute_price()

    @api.depends("price_discount", "date_start", "dis_2", "dis_3", "dis_inc_SIA", "dis_franchise_SIA")
    def _compute_price(self):
        """Compute the price based on the vendor price and discounts."""
    
    @api.onchange("price_discount", "date_start")
    def _onchange_franchise_discounts(self):
        """Recalculate the cost from the franchise when the date start or discounts change."""
        if self.pricelist_id.is_franchise_list:
            self._update_vendor_price(self, self.price_discount, self.company_id)
    '''

    @api.model_create_multi
    def create(self, vals_list):
        res = super(ProductPricelistItemInherit, self).create(vals_list)
        for record in res:
            if record.pricelist_id.is_main_list:
                new_record = self.update_child_pricelists(record)
                res = res | new_record
            if record.pricelist_id.is_updated_by_main:
                raise UserError(_('You cannot create a product from a franchise pricelist with main update.'))
        return res

    def write(self, values):
        res = super(ProductPricelistItemInherit, self).write(values)
        if self.pricelist_id.is_franchise_list:
            company = self.with_company(self.pricelist_id.valid_for)
            company = company.with_context(visible_product_tmpl_id=False)
            company._update_vendor_price(self.price_discount, self.company_id, self.pricelist_id.valid_for)
        '''
        for record in self:
            if record.pricelist_id.is_main_list:
                self.update_child_pricelists(record)
        '''
        return res
    
    def unlink(self):
        for record in self:
            if record.pricelist_id.is_updated_by_main:
                raise UserError(_('You cannot delete a product from a franchise pricelist with main update.'))
            if record.pricelist_id.is_main_list:
                record.delete_child_pricelists()
            
            res = super(ProductPricelistItemInherit, self).unlink()
        
        return res
    
    def delete_child_pricelists(self):
        #import wdb; wdb.set_trace()
        companies = self.env["res.company"].search([])
        for company_id in companies:
            company = self.with_company(company_id)
            child_pricelists = company.env["product.pricelist"].search(
                [
                    ("company_id", "=", company_id.id),
                    ("is_updated_by_main", "=", True),
                ]
            )
            for pl in child_pricelists:
                matching_id = next((item.id for item in pl.item_ids if item.product_id.id == self.product_id.id), False)
                if matching_id:
                    product_pricelist_item = company.env["product.pricelist.item"].search([('id', '=', matching_id)])
                    super(ProductPricelistItemInherit, product_pricelist_item).unlink()
    
    def update_child_pricelists(self, record):
        #child_companies = self.env["res.company"].search([('id', '!=', record.company_id.id)])
        companies = self.env["res.company"].search([])
        res = self.env["product.pricelist.item"]
        for company_id in companies:
            company = self.with_company(company_id)
            child_pricelists = company.env["product.pricelist"].search(
                [
                    ("company_id", "=", company_id.id),
                    ("is_updated_by_main", "=", True),
                ]
            )
            '''
            if child_pricelists and len(child_pricelists) > 1:
                raise ValidationError(_('There is more than one pricelist for this franchise.'))
            if not child_pricelists:
                raise ValidationError(_('There is no pricelist for this franchise.'))
            '''
            for pl in child_pricelists:
                matching_id = next((item.id for item in pl.item_ids if item.product_id.id == record.product_id.id), False)
                if not matching_id:
                    pricelist_id = pl.id
                    new_res = company.env["product.pricelist.item"]._create_child_pricelists(record, pricelist_id)
                    res = res | new_res
        return res

    def _update_child_pricelists(self, parent):
        """Update the child pricelists when the parent pricelist is updated."""
        for record in self:
            record.update({
                "date_start": parent.date_start,
                "date_end": parent.date_end,
                "min_quantity": parent.min_quantity,
                "currency_id": parent.currency_id.id,
                "compute_price": parent.compute_price,
                "applied_on": parent.applied_on,
                "base": parent.base,
            })
            record._update_vendor_price(record.price_discount, parent.company_id, record.company_id)
            '''
            record.write({
                "date_start": parent.date_start,
                "date_end": parent.date_end,
                "min_quantity": parent.min_quantity,
                "currency_id": parent.currency_id.id,
                "compute_price": parent.compute_price,
                "applied_on": parent.applied_on,
                "base": parent.base,
            })
            '''
    
    def _create_child_pricelists(self, parent, pricelist_id):
        """Create the child pricelists when the parent pricelist is updated."""
        vals = {
            "pricelist_id": pricelist_id,
            "product_tmpl_id": parent.product_tmpl_id.id,
            "product_id": parent.product_id.id,
            "date_start": parent.date_start,
            "date_end": parent.date_end,
            "min_quantity": parent.min_quantity,
            "currency_id": parent.currency_id.id,
            "compute_price": parent.compute_price,
            "applied_on": parent.applied_on,
            "base": parent.base,
            "name": parent.name,
            "price_discount": parent.price_discount,
        }
        res = super(ProductPricelistItemInherit, self).create(vals)
        res._update_vendor_price(res.price_discount, parent.company_id, res.company_id)

        return res
    
    def _update_vendor_price(self, price_discount, main_company, franchise_company):
        child_pricelists = self.env["product.supplierinfo"].search(
            [
                ("product_id", "=", self.product_id.id),
                ("company_id", "=", franchise_company.id),
                ("partner_id", "=", main_company.id),
            ]
        )
        if child_pricelists:
            child_pricelists.update({
                "dis_franchise_SIA": price_discount,
            })
