"""Tests for the product.pricelist model in the nybble_price_list module."""


class TestProductPricelist(TransactionCase):
    """Test cases for the ProductPricelistInherit model."""

    def setUp(self):
        """Set up the test environment."""
        super().setUp()
        self.pricelist = self.env["product.pricelist"].create(
            {
                "name": "Test Pricelist",
                "is_franchise_list": True,
                "valid_for": self.env["res.company"]
                .create(
                    {
                        "name": "Test Franchise",
                        "is_main_company": False,
                    }
                )
                .id,
            }
        )
        self.product = self.env["product.product"].create(
            {
                "name": "Test Product",
                "type": "product",
                "list_price": 100.0,
            }
        )
        self.product_template = self.env["product.template"].create(
            {
                "name": "Test Product Template",
                "categ_id": self.env.ref("product.product_category_all").id,
            }
        )
        self.pricelist_item = self.env["product.pricelist.item"].create(
            {
                "pricelist_id": self.pricelist.id,
                "applied_on": "1_product",
                "product_tmpl_id": self.product_template.id,
                "fixed_price": 90.0,
            }
        )

    def test_update_pricelist(self):
        """Test the update_pricelist method."""
        self.pricelist.update_pricelist()
        # Assert that the supplierinfo record is created correctly
        supplierinfo = self.env["product.supplierinfo"].search(
            [
                ("product_id", "=", self.product.id),
                ("company_id", "=", self.pricelist.valid_for.id),
            ]
        )
        self.assertTrue(supplierinfo)
        self.assertEqual(supplierinfo.vendor_price, 90.0)
        # Add more assertions if needed
        self.assertEqual(supplierinfo.min_qty, self.pricelist_item.min_quantity)
        self.assertEqual(supplierinfo.date_start, self.pricelist_item.date_start)
        self.assertEqual(supplierinfo.date_end, self.pricelist_item.date_end)
