"""This module contains the definition of the ProductSupplierinfo model."""
import logging

from odoo import api, fields, models

_logger = logging.getLogger(__name__)


class ResConfigSettings(models.TransientModel):
    """Inherit res.config.settings to add a field for users to notify when a cost is determined."""

    _inherit = "res.config.settings"

    cost_determination_users = fields.Many2many(
        "res.users", string="Users for Cost Determination Notifications"
    )

    def set_values(self):
        """Override set_values to save the cost_determination_users in the ir.config_parameter."""
        super(ResConfigSettings, self).set_values()
        self.env["ir.config_parameter"].set_param(
            "nybble_purchase.cost_determination_users",
            ",".join(map(str, self.cost_determination_users.ids)),
        )

    @api.model
    def get_values(self):
        """Override get_values to get the cost_determination_users from the ir.config_parameter."""
        res = super(ResConfigSettings, self).get_values()
        cost_determination_users = (
            self.env["ir.config_parameter"]
            .sudo()
            .get_param("nybble_purchase.cost_determination_users")
        )
        if cost_determination_users:
            user_ids = [int(id) for id in cost_determination_users.split(",") if id]
            res.update(cost_determination_users=[(6, 0, user_ids)])
        else:
            res.update(cost_determination_users=[(6, 0, [])])
        return res
