"""Tests for the ProductSupplierinfoHistorical model."""
from datetime import timedelta

from odoo.tests.common import TransactionCase


class TestProductSupplierinfoHistorical(TransactionCase):
    """Tests for the ProductSupplierinfoHistorical model."""

    def setUp(self):
        """Set up the test environment."""
        super(TestProductSupplierinfoHistorical, self).setUp()
        self.product_template = self.env["product.template"].create(
            {"name": "Test Product Template"}
        )
        self.product = self.env["product.product"].create(
            {"name": "Test Product", "product_tmpl_id": self.product_template.id}
        )
        self.partner = self.env["res.partner"].create({"name": "Test Partner"})
        self.company = self.env["res.company"].create({"name": "Test Company"})
        self.currency = self.env["res.currency"].create(
            {"name": "Test Currency", "symbol": "$"}
        )
        self.initial_record = self.env["product.supplierinfo.historical"].create(
            {
                "product_tmpl_id": self.product_template.id,
                "product_id": self.product.id,
                "partner_id": self.partner.id,
                "company_id": self.company.id,
                "currency_id": self.currency.id,
                "date_start": "2023-01-01",
                "date_end": "2023-01-31",
                "vendor_price": 100.0,
                "price": 90.0,
                "inverse_company_rate": 1.0,
                "ars_price": 90.0,
            }
        )


def test_check_date_overlap_no_overlap(self):
    """Test the _check_date_overlap method with no overlapping dates."""
    new_record = self.env["product.supplierinfo.historical"].create(
        {
            "product_tmpl_id": self.product_template.id,
            "product_id": self.product.id,
            "partner_id": self.partner.id,
            "company_id": self.company.id,
            "currency_id": self.currency.id,
            "date_start": "2023-02-01",
            "date_end": "2023-02-28",
            "vendor_price": 100.0,
            "price": 85.0,
            "inverse_company_rate": 1.0,
            "ars_price": 85.0,
        }
    )
    self.assertEqual(new_record.date_start, "2023-02-01")
    self.assertEqual(new_record.date_end, "2023-02-28")


def test_check_date_overlap_with_overlap(self):
    """Test the _check_date_overlap method with overlapping dates."""
    with self.assertRaises(ValidationError):
        self.env["product.supplierinfo.historical"].create(
            {
                "product_tmpl_id": self.product_template.id,
                "product_id": self.product.id,
                "partner_id": self.partner.id,
                "company_id": self.company.id,
                "currency_id": self.currency.id,
                "date_start": "2023-01-15",
                "date_end": "2023-02-15",
                "vendor_price": 100.0,
                "price": 80.0,
                "inverse_company_rate": 1.0,
                "ars_price": 80.0,
            }
        )
        self.initial_record.refresh()
        self.assertEqual(
            self.initial_record.date_end,
            overlapping_record.date_start - timedelta(days=1),
        )
        self.assertEqual(overlapping_record.date_start, "2023-01-15")
        self.assertEqual(overlapping_record.date_end, "2023-02-15")
