##############################################################################
# For copyright and license notices, see __manifest__.py file in module root
# directory
##############################################################################
"""Defines Stock Picking"""
from odoo import api, fields, models

import odoo.addons.decimal_precision as dp


class StockPicking(models.Model):
    """Extiende y agrega compos para remito."""

    _inherit = "stock.picking"

    total_stock_id_product = fields.Integer(
        String="Cantidad de productos",
        store=True,
        compute="_compute_total_stock_product",
        help="Cantidad de productos",
    )
    total_stock_id_product_qty = fields.Integer(
        String="Cantidad total de artículos",
        store=True,
        compute="_compute_total_stock_product_qty",
        help="Cantidad total",
    )
    declared_value = fields.Float(
        digits=dp.get_precision("Account"),
        compute="_compute_declared_value",
    )
    position_carga = fields.Integer(
        String="Posicion de la Entrega", compute="_compute_position"
    )
    position_total = fields.Integer(
        String="Posicion Total", compute="_compute_position_total"
    )
    position_ruta = fields.Char(String="Ruta", compute="_compute_position_ruta")
    number_invoice = fields.Char(String="N invoice", compute="_compute_number_invoice")
    total_volume = fields.Float(
        string="Volumen total", compute="_compute_volumen_total"
    )

    def get_vouchers_list(self):
        """Devuelve el numero de voucher"""
        for rec in self:
            cant = len(rec.voucher_ids)
            if cant == 0:
                return ""
            elif cant == 1:
                return rec.voucher_ids.number
            else:
                return rec.voucher_ids[cant - 1].number

    def _compute_number_invoice(self):
        """Obtiene numero de factura asociado al remito."""
        for rec in self:
            invoic_id = (
                rec.env["account.move"]
                .sudo()
                .search([("invoice_origin", "=", self.sale_id.name)])
            )
            if invoic_id:
                for invoice_id in invoic_id:
                    if invoice_id:
                        rec.number_invoice = invoice_id.name
                    else:
                        rec.number_invoice = ""
            else:
                rec.number_invoice = ""

    def _compute_position_ruta(self):
        """Ruta de la Carga en Optimizacion Logistica."""
        for rec in self:
            picking_incs = rec.env["lcd.carga_mercaderia_group"].search(
                [("picking_id", "=", self.id), ("estado", "=", "procesado")]
            )
            for picking_inc in picking_incs:
                rec.position_ruta = picking_inc.carga_id.ruta_id.name

    def _compute_position_total(self):
        """Total de entregas en Carga de Mercaderia Optimizacion Logistica."""
        for rec in self:
            picking_incs = rec.env["lcd.carga_mercaderia_group"].search(
                [("picking_id", "=", self.id), ("estado", "=", "procesado")]
            )
            if picking_incs:
                for picking_inc in picking_incs:
                    picking_total = len(
                        rec.env["lcd.carga_mercaderia_group"]
                        .search(
                            [
                                ("carga_id", "=", picking_inc.carga_id.id),
                                ("estado", "=", "procesado"),
                            ]
                        )
                        .mapped("picking_id")
                        .ids
                    )
                    rec.position_total = picking_total
            else:
                rec.position_total = 0

    def _compute_position(self):
        """Posicion de la Entrega en Carga de Mercaderia en Optimizacion Logistica."""
        for rec in self:
            picking_incs = rec.env["lcd.carga_mercaderia_group"].search(
                [("picking_id", "=", self.id), ("estado", "=", "procesado")]
            )
            if picking_incs:
                for picking_inc in picking_incs:
                    picking_all = (
                        rec.env["lcd.carga_mercaderia_group"]
                        .search(
                            [
                                ("carga_id", "=", picking_inc.carga_id.id),
                                ("estado", "=", "procesado"),
                            ]
                        )
                        .mapped("picking_id")
                        .ids.index(self.id)
                    )
                    rec.position_carga = picking_all + 1
            else:
                rec.position_carga = 0

    @api.depends("move_ids_without_package")
    def _compute_declared_value(self):
        """Valor declarado."""
        for rec in self:
            rec.declared_value = rec.sale_id.amount_total

    @api.depends("move_ids_without_package")
    def _compute_total_stock_product(self):
        """Total productos."""
        for record in self:
            total_product = 0
            lists = []
            for line in record.move_ids_without_package:
                if line.product_id in lists:
                    total_product = total_product
                else:
                    lists.append(line.product_id)
                    total_product = total_product + 1
            record.total_stock_id_product = total_product

    @api.depends("move_ids_without_package")
    def _compute_total_stock_product_qty(self):
        """Cantidad total productos."""
        for record in self:
            total_product_qty = 0
            if record.state == "done":
                for line in record.move_ids_without_package:
                    total_product_qty += line.quantity_done
            else:
                for line in record.move_ids_without_package:
                    total_product_qty += line.product_uom_qty
            record.total_stock_id_product_qty = total_product_qty

    def do_print_voucher(self):
        """Imprime Remito"""
        self._compute_total_stock_product()
        self._compute_total_stock_product_qty()
        report = self.env["ir.actions.report"]._get_report_from_name(
            "nybble_remitos_report.report_voucher_nybble"
        )
        return report.report_action(docids=self)

    def _compute_volumen_total(self):
        for rec in self:
            totals = []
            for line in rec.move_ids_without_package.product_id:
                totals.append(line.volume)
            total = sum(totals)
            if total:
                rec.total_volume = total
            else:
                rec.total_volume = 0

    def force_recompute_declared_value(self, ids=None):
        if ids:
            self = self.browse(ids)
        else:
            self = self.search([])
        for record in self:
            record._compute_declared_value()
