odoo.define("nybble_reports_dinamyc.journal_report", function (require) {
    "use strict";

    var AbstractAction = require("web.AbstractAction");
    var core = require("web.core");
    var QWeb = core.qweb;
    var _t = core._t;
    var Dialog = require("web.Dialog");

    var JournalReport = AbstractAction.extend({
        template: "JournalReportTemplate",
        events: {
            "click .toggle-move": "_onToggleMove",
            "click .o_journal_print_pdf": "_onPrintPDF",
            "click .o_journal_export_excel": "_onExportExcel",
        },

        init: function (parent, action) {
            this._super.apply(this, arguments);
            this.report_data = action.params.report_data || [];
            this.initial_move_id = action.params.initial_move_id;
            console.log(
                "Received report data for",
                this.report_data.length,
                "journal entries"
            );
        },

        start: function () {
            var self = this;
            return this._super().then(function () {
                self._renderReport();
            });
        },

        _renderReport: function () {
            if (!Array.isArray(this.report_data)) {
                console.error("report_data is not an array:", this.report_data);
                this.report_data = [];
            }
            var $content = $(
                QWeb.render("JournalReportContent", {
                    report_data: this.report_data,
                })
            );
            this.$(".o_journal_report_content").html($content);
            console.log("Rendered", this.report_data.length, "journal entries");
        },

        _onToggleMove: function (ev) {
            var $target = $(ev.currentTarget);
            $target.toggleClass("fa-caret-right fa-caret-down");
            $target
                .closest(".move-section")
                .find(".move-content")
                .toggleClass("d-none");
        },

        _onPrintPDF: function () {
            var self = this;
            if (!this.initial_move_id) {
                this._showErrorDialog(
                    _t("Please select an initial journal entry before printing.")
                );
                return;
            }
            this._rpc({
                model: "journal.report",
                method: "generate_report",
                args: [this.initial_move_id],
            })
                .then(function (action) {
                    if (action && action.type) {
                        self.do_action(action);
                    } else if (action && action.error) {
                        self._showErrorDialog(action.error);
                    } else {
                        self._showErrorDialog(
                            _t("The server did not return a valid action.")
                        );
                    }
                })
                .catch(function (error) {
                    console.error("Error printing PDF:", error);
                    self._showErrorDialog(
                        _t(
                            "An error occurred while printing the PDF. Please try again."
                        )
                    );
                });
        },

        _onExportExcel: function () {
            var self = this;
            if (!this.initial_move_id) {
                this._showErrorDialog(
                    _t("Please select an initial journal entry before exporting.")
                );
                return;
            }
            this._rpc({
                model: "journal.report",
                method: "export_excel_report",
                args: [this.initial_move_id],
            })
                .then(function (action) {
                    if (action && action.type === "ir.actions.act_url") {
                        self.do_action(action);
                    } else if (action && action.error) {
                        self._showErrorDialog(action.error);
                    } else {
                        self._showErrorDialog(
                            _t("The server did not return a valid action.")
                        );
                    }
                })
                .catch(function (error) {
                    console.error("Error exporting Excel:", error);
                    self._showErrorDialog(
                        _t(
                            "An error occurred while exporting to Excel. Please try again."
                        )
                    );
                });
        },

        _showWarningDialog: function (message) {
            Dialog.alert(this, message, {title: _t("Warning")});
        },

        _showErrorDialog: function (message) {
            Dialog.alert(this, message, {title: _t("Error")});
        },
    });

    core.action_registry.add("journal_report", JournalReport);

    return JournalReport;
});
