"""Test cases for Journal Report Wizard."""
from odoo.tests.common import TransactionCase


class TestJournalReportWizard(TransactionCase):
    """Test cases for Journal Report Wizard."""

    def setUp(self):
        """Set up the test environment."""
        super().setUp()
        self.journal_report_wizard = self.env["journal.report.wizard"].create(
            {
                "date_from": "2023-01-01",
                "date_to": "2023-12-31",
                "initial_move_id": self.env["account.move"]
                .create(
                    {
                        "date": "2023-01-01",
                        "journal_id": self.env["account.journal"]
                        .create(
                            {
                                "name": "Test Journal",
                                "code": "TJ",
                                "type": "general",
                            }
                        )
                        .id,
                        "line_ids": [
                            (
                                0,
                                0,
                                {
                                    "account_id": self.env["account.account"]
                                    .create(
                                        {
                                            "name": "Test Account",
                                            "code": "TA",
                                            "user_type_id": self.env.ref(
                                                "account.data_account_type_expenses"
                                            ).id,
                                        }
                                    )
                                    .id,
                                    "debit": 100.0,
                                    "credit": 0.0,
                                },
                            )
                        ],
                    }
                )
                .id,
            }
        )

    def test_onchange_date(self):
        """Test the _onchange_date method."""
        self.journal_report_wizard.date_from = "2023-01-01"
        self.journal_report_wizard.date_to = "2023-12-31"
        self.journal_report_wizard._onchange_date()
        domain = self.journal_report_wizard._onchange_date()["domain"][
            "initial_move_id"
        ]
        self.assertIn(("date", ">=", "2023-01-01"), domain)
        self.assertIn(("date", "<=", "2023-12-31"), domain)
        self.assertIn(("state", "=", "posted"), domain)

    def test_enumerate_entries(self):
        """Test the enumerate_entries method."""
        self.journal_report_wizard.enumerate_entries()
        moves = self.env["account.move"].search(
            [
                ("date", ">=", self.journal_report_wizard.initial_move_id.date),
                ("date", "<=", self.journal_report_wizard.date_to),
                ("state", "=", "posted"),
            ],
            order="date, id",
        )
        for i, move in enumerate(moves, start=1):
            self.assertEqual(move.x_entry_number, i)

    def test_prepare_report_data(self):
        """Test the _prepare_report_data method."""
        moves = self.env["account.move"].search(
            [
                ("date", ">=", self.journal_report_wizard.initial_move_id.date),
                ("date", "<=", self.journal_report_wizard.date_to),
                ("state", "=", "posted"),
            ],
            order="date, id",
        )
        report_data = self.journal_report_wizard._prepare_report_data(moves)
        self.assertTrue(report_data)
        self.assertEqual(report_data[0]["entry_number"], 1)
        self.assertEqual(report_data[0]["name"], moves[0].name)

    def test_print_pdf_report(self):
        """Test the print_pdf_report method."""
        result = self.journal_report_wizard.print_pdf_report("2023-01-01", "2023-12-31")
        self.assertIn("report_data", result["data"])

    def test_export_excel_report(self):
        """Test the export_excel_report method."""
        result = self.journal_report_wizard.export_excel_report(
            "2023-01-01", "2023-12-31"
        )
        self.assertIn("url", result)
        self.assertTrue(result["url"].startswith("/web/content/"))
