"""This module contains the custom implementation of the `account.move.line` model."""
from odoo import models


class AccountMoveLine(models.Model):
    """Added methods for creating and deleting records from the AccountSalesAnalysis"""

    _inherit = ["account.move.line"]

    def get_sale_analysis_lines(self):
        """
        Creates and returns a list with the values necessary for the creation of the lines of the AccountSalesAnalysis
        for the recordset.
        """
        sale_analysis_lines = []
        for line in self:
            if line.product_id:
                if not self.env["account.sales.analysis"].search([("move_line_id", "=", line.id)]):
                    document_type = ""
                    if not line.move_id.provisional:
                        document_type = line.move_id.l10n_latam_document_type_id.name
                    product_supplierinfo = self.env["product.supplierinfo"].search(
                        [
                            ("product_id", "=", line.product_id.id), 
                            ("determine_cost", "=", True),
                            ("company_id", "=", line.company_id.id)
                        ]
                    )
                    cost_currency = False
                    if product_supplierinfo:
                        cost_currency = product_supplierinfo.ensure_one().currency_id
                    inverse_company_rate = 0.0
                    if cost_currency:
                        if cost_currency.is_current_company_currency:
                            inverse_company_rate = 1.0
                        else:
                            cost_currency_rate = self.env["res.currency.rate"].search(
                                [
                                    ("name", "=", line.move_id.invoice_date),
                                    ("currency_id", "=", cost_currency.id),
                                ]
                            )
                            if cost_currency_rate:
                                inverse_company_rate = cost_currency_rate.inverse_company_rate
                        cost_currency = cost_currency.id
                    sale_analysis_lines.append(
                        {
                            "brand_id": line.product_id.brand_id.id,
                            "company_id": line.move_id.company_id.id,
                            "cost_currency": cost_currency,
                            "date": line.move_id.invoice_date if line.move_id.invoice_date else line.move_id.date,
                            "default_code": line.product_id.default_code,
                            "document_type": document_type,
                            "family_id": line.product_id.family_id.id,
                            "historical_cost": line.product_id.standard_price,
                            "inverse_company_rate": inverse_company_rate,
                            "journal_id": line.move_id.journal_id.id,
                            "name": line.move_id.name,
                            "partner_id": line.move_id.partner_id.id,
                            "partner_city": line.move_id.partner_id.city,
                            "price_unit": line.price_unit,
                            "product_id": line.product_id.id,
                            "product_qty": line.quantity,
                            "team_id": line.move_id.team_id.id,
                            "user_id": line.move_id.invoice_user_id.id,
                            "subfamily_id": line.product_id.subfamily_id.id,
                            "move_line_id": line.id,
                        }
                    )
        return sale_analysis_lines

    def clean_sale_analysis_lines(self):
        """Finds the lines of the AccountSalesAnalysis related to the recordset and deletes them."""
        self.env["account.sales.analysis"].search(
            [("move_line_id", "in", self.ids)]
        ).unlink()
