"""This module contains the custom implementation of the `sale.order` model."""
from odoo import models


class SaleOrder(models.Model):
    """Added to the invoice creation flow the removal of AccountSalesAnalysis lines."""

    _inherit = "sale.order"

    def _create_invoices(self, grouped=False, final=False, date=None):
        """
        When the invoice is created (sale order state change to 'invoiced') the associated
        AccountSalesAnalysis lines are deleted.
        """
        invoices_created = super(SaleOrder, self)._create_invoices(
            grouped=grouped, final=final, date=date
        )
        for order in self:
            order.order_line.clean_sale_analysis_lines()
        return invoices_created

    def _action_cancel(self):
        res = super(SaleOrder, self)._action_cancel()
        for order in self:
            order.order_line.clean_sale_analysis_lines()
        return res
