"""This module contains the custom implementation of the `stock.move` model."""
from odoo import models


class StockMove(models.Model):
    """Added method for creating AccountSalesAnalysis lines"""

    _inherit = "stock.move"

    def get_sale_analysis_lines(self):
        """
        Creates and returns a list with the values necessary for the creation of the
        AccountSalesAnalysis lines for the recordset.
        """
        sale_analysis_lines = []
        for line in self:
            if line.quantity_done > 0:
                order_line = line.sale_line_id
                order = order_line.order_id
                if order.invoice_status == "to invoice":
                    document_type = "Pedido"
                    if order.provisional2:
                        document_type += " Provisorio"
                    product_supplierinfo = self.env["product.supplierinfo"].search(
                        [
                            ("product_id", "=", line.product_id.id),
                            ("determine_cost", "=", True),
                        ]
                    )
                    cost_currency = False
                    if product_supplierinfo:
                        cost_currency = product_supplierinfo.currency_id
                    inverse_company_rate = 0.0
                    if cost_currency:
                        if cost_currency.is_current_company_currency:
                            inverse_company_rate = 1.0
                        else:
                            cost_currency_rate = self.env["res.currency.rate"].search(
                                [
                                    ("name", "=", order.date_order),
                                    ("currency_id", "=", cost_currency.id),
                                ]
                            )
                            if cost_currency_rate:
                                inverse_company_rate = (
                                    cost_currency_rate.inverse_company_rate
                                )
                        cost_currency = cost_currency.id
                    sale_analysis_lines.append(
                        {
                            "brand_id": line.product_id.brand_id.id,
                            "company_id": order.company_id.id,
                            "cost_currency": cost_currency,
                            "date": order.date_order,
                            "default_code": line.product_id.default_code,
                            "document_type": document_type,
                            "family_id": line.product_id.family_id.id,
                            "historical_cost": line.product_id.standard_price,
                            "inverse_company_rate": inverse_company_rate,
                            "name": order.name,
                            "partner_id": order.partner_id.id,
                            "partner_city": order.partner_id.city,
                            "price_unit": order_line.price_unit,
                            "product_id": line.product_id.id,
                            "product_qty": line.quantity_done,
                            "team_id": order.team_id.id,
                            "user_id": order.user_id.id,
                            "subfamily_id": line.product_id.subfamily_id.id,
                            "sale_order_line_id": order_line.id,
                        }
                    )
        return sale_analysis_lines
