"""This module contains the custom implementation of the `stock.picking` model."""
from odoo import models


class StockPicking(models.Model):
    """The creation of AccountSalesAnalysis lines is added to the validation flow."""

    _inherit = ["stock.picking"]

    def _create_sale_analysis_lines(self):
        """The AccountSalesAnalysis lines are created when the products of a given Sales Order are delivered."""
        sale_analysis_lines_vals = []
        for picking in self:
            if picking.picking_type_id.code == "outgoing":
                sale_analysis_lines_vals.extend(
                    picking.move_ids.get_sale_analysis_lines()
                )
        if sale_analysis_lines_vals:
            self.env["account.sales.analysis"].create(sale_analysis_lines_vals)

    def _action_done(self):
        """The AccountSalesAnalysis line creation added to validation flow."""
        res = super(StockPicking, self)._action_done()
        self._create_sale_analysis_lines()
        return res
