"""Add reason for lost sale"""
from odoo import _, api, fields, models
from odoo.exceptions import ValidationError


class SaleLostReason(models.Model):
    """Model representing the reasons for sales loss."""

    _name = "sale.lost.reason"
    _description = "Reason for lost sale"
    _order = "sequence, id"

    name = fields.Char(string="Description", required=True)
    sequence = fields.Integer(string="Secuence", default=10)
    company_id = fields.Many2one(
        "res.company",
        string="Company",
        required=True,
        default=lambda self: self.env.company,
    )

    @api.constrains("name")
    def _check_name_unique(self):
        """Check that the name field is unique."""
        for record in self:
            if not record.name or record.name.strip() == "":
                raise ValidationError(_("The field Description cannot be empty."))

            existing = self.search(
                [("id", "!=", record.id), ("name", "=ilike", record.name.strip())]
            )
            if existing:
                raise ValidationError(
                    _("The lost reason '{}' already exists!").format(record.name)
                )

    @api.constrains("name")
    def _check_name_not_empty(self):
        """Check that the name field is not empty."""
        for record in self:
            if not record.name or record.name.strip() == "":
                raise ValidationError(_("The field Description cannot be empty."))
