"""Add new fields to Sale Order model"""
from odoo import fields, models


class SaleOrder(models.Model):
    """Add new fields to Sale Order model"""

    _inherit = "sale.order"

    is_lost = fields.Boolean(string="Sale lost", default=False)
    lost_reason = fields.Many2one("sale.lost.reason", string="Motivo de Pérdida")

    def action_mark_as_lost(self):
        """Action to mark a sale as lost."""
        return {
            "name": "Marcar como perdida",
            "type": "ir.actions.act_window",
            "res_model": "sale.order.lost.reason",
            "view_mode": "form",
            "target": "new",
            "context": {"default_order_id": self.id},
        }

    def action_draft(self):
        """Override the action_draft method to delete lost sale lost record and reset lost sale fields."""
        result = super(SaleOrder, self).action_draft()
        for order in self:
            # delete lost sale lost record
            lost_sales = self.env["sale.order.lost"].search([("name", "=", order.name)])
            if lost_sales:
                lost_sales.unlink()
            # reset lost sale fields
            order.write(
                {
                    "is_lost": False,
                    "lost_reason": False,
                }
            )
        return result


class SaleOrderLine(models.Model):
    """Add new fields to Sale Order Line model"""

    _inherit = "sale.order.line"

    def name_get(self):
        """Return the name of the product with the default code if it exists."""
        result = []
        for line in self:
            name = line.product_id.name
            if line.product_id.default_code:
                name = "[%s] %s" % (line.product_id.default_code, name)
            result.append((line.id, name))
        return result
