"""Add new fields to Sale Order Lost model"""
from odoo import api, fields, models


class SaleOrderLost(models.Model):
    """Model representing the lost sales."""

    _name = "sale.order.lost"
    _description = "Sale Order Lost"
    _order = "create_order desc"

    create_order = fields.Date(string="Date", readonly=True)
    team_id = fields.Many2one(
        "crm.team", string="Team sale", readonly=True, ondelete="cascade"
    )
    name = fields.Char(string="Número", readonly=True)
    user_id = fields.Many2one(
        "res.users", string="Salesperson", readonly=True, ondelete="cascade"
    )
    partner_id = fields.Many2one(
        "res.partner", string="Client", readonly=True, ondelete="cascade"
    )
    order_line_product_name = fields.Char(string="Sale order line name", readonly=True)
    product_code = fields.Char(string="Cod Purchase", readonly=True)
    product_id = fields.Many2one(
        "product.product", string="Product variant", readonly=True, ondelete="cascade"
    )
    product_tmpl_id = fields.Many2one(
        "product.template", string="Product", required=True, nullable=False
    )
    brand_name = fields.Char(
        related="product_tmpl_id.tmpl_brand_id.name",
        string="Brand",
        store=True,
        readonly=True,
    )
    family_name = fields.Char(
        related="product_tmpl_id.family_id.name",
        string="Family",
        store=True,
        readonly=True,
    )
    subfamily_name = fields.Char(
        related="product_tmpl_id.subfamily_id.name",
        string="Subfamily",
        store=True,
        readonly=True,
    )
    product_uom_qty = fields.Float(string="Cant", readonly=True)
    standard_price_total = fields.Float(
        string="Coste",
        readonly=True,
        compute="_compute_standard_price_total",
        store=True,
    )
    price_total = fields.Monetary(
        string="Mount", readonly=True, compute="_compute_price_total", store=True
    )
    sale_lost_reason_id = fields.Many2one(
        "sale.lost.reason", string="reason lost", readonly=True, ondelete="cascade"
    )
    sale_lost_reason_obs = fields.Char(string="Obs", readonly=True)
    currency_id = fields.Many2one(
        "res.currency", string="Money", readonly=True, ondelete="cascade"
    )
    total_price = fields.Monetary(
        string="Mount Total",
        compute="_compute_total_price",
        currency_field="currency_id",
        store=True,
    )
    company_id = fields.Many2one(
        "res.company",
        string="Company",
        required=True,
        default=lambda self: self.env.company,
    )

    @api.depends("price_total")
    def _compute_total_price(self):
        """Compute the Sum total price"""
        total = sum(self.mapped("price_total"))
        for record in self:
            record.total_price = total

    @api.depends("product_id", "product_uom_qty")
    def _compute_standard_price_total(self):
        """Compute the standard price of the sale order line"""
        for record in self:
            if record.product_id and record.product_uom_qty:
                record.standard_price_total = (
                    record.product_id.standard_price * record.product_uom_qty
                )
            else:
                record.standard_price_total = 0

    @api.depends("product_id", "product_uom_qty")
    def _compute_price_total(self):
        """Compute the total price of the sale order line"""
        for record in self:
            pricelist = self.env["product.pricelist"].search(
                [("is_cash_list", "=", True)], limit=1
            )
            if pricelist and record.product_id and record.product_uom_qty:
                price = pricelist._get_product_price(
                    record.product_id, record.product_uom_qty, uom=None, date=False
                )
                record.price_total = price * record.product_uom_qty
            else:
                record.price_total = 0
