"""test_sale_order_lost_reason module for the nybble_sale_lost module."""
from odoo.tests.common import TransactionCase


class TestSaleOrderLostReason(TransactionCase):
    """Test cases for sale order lost reason wizard."""

    def setUp(self):
        """Create a sale order and a sale order lost reason."""
        super().setUp()
        self.sale_order = self.env["sale.order"].create(
            {
                # Add necessary fields for sale.order creation
                name: "Test Sale Order",
                partner_id: self.env["res.partner"]
                .create(
                    {
                        "name": "Test Partner",
                        "email": "",
                    }
                )
                .id,
                order_line: [
                    (
                        0,
                        0,
                        {
                            "product_id": self.env["product.product"]
                            .create(
                                {
                                    "name": "Test Product",
                                    "type": "consu",
                                    "list_price": 100.0,
                                }
                            )
                            .id,
                            "product_uom_qty": 1,
                            "price_unit": 100.0,
                        },
                    )
                ],
            }
        )
        self.wizard = self.env["sale.order.lost.reason"].create(
            {
                "order_id": self.sale_order.id,
                "lost_reason": self.env.ref("nybble_sale_lost.sale_lost_reason_1").id,
                "sale_lost_reason_obs": "Observations",
            }
        )

    def test_create_lost_sale_record(self):
        """Test the _create_lost_sale_record method."""
        self.wizard._create_lost_sale_record()
        # Assert that the record is created correctly
        lost_sale = self.env["sale.order.lost"].search(
            [
                ("name", "=", self.sale_order.name),
                ("product_id", "=", self.sale_order.order_line[0].product_id.id),
            ]
        )
        self.assertEqual(lost_sale.create_order, self.sale_order.date_order)
        self.assertEqual(lost_sale.team_id, self.sale_order.team_id)
        # Add more assertions for other fields
        self.assertEqual(lost_sale.product_id, self.sale_order.order_line[0].product_id)
        self.assertEqual(lost_sale.sale_lost_reason_id, self.wizard.lost_reason)
        self.assertEqual(
            lost_sale.sale_lost_reason_obs, self.wizard.sale_lost_reason_obs
        )
        self.assertEqual(lost_sale.currency_id, self.sale_order.currency_id)
        # Assert that the sale order is marked as lost
        self.assertTrue(self.sale_order.is_lost)
        self.assertEqual(self.sale_order.lost_reason, self.wizard.lost_reason)

    def test_action_mark_as_lost(self):
        """Test the action_mark_as_lost method."""
        self.wizard.action_mark_as_lost()
        # Assert that the sale order is marked as lost
        self.assertTrue(self.wizard.order_id.is_lost)
        self.assertEqual(self.wizard.order_id.lost_reason, self.wizard.lost_reason.id)
        # Add more assertions if needed
        self.assertTrue(
            self.env["sale.order.lost"].search(
                [("name", "=", self.wizard.order_id.name)]
            )
        )
        self.assertEqual(
            self.env["sale.order.lost"]
            .search([("name", "=", self.wizard.order_id.name)])
            .sale_lost_reason_obs,
            self.wizard.sale_lost_reason_obs,
        )
