"""Wizard for marking sale orders as lost."""
from odoo import fields, models


class SaleOrderLostReason(models.TransientModel):
    """Wizard for marking sale orders as lost."""

    _name = "sale.order.lost.reason"
    _description = "Lost Sales Reason"

    order_id = fields.Many2one("sale.order", string="Budget", required=True)
    lost_reason = fields.Many2one(
        "sale.lost.reason", string="Loss Reason", required=True
    )
    sale_lost_reason_obs = fields.Char(string="Observations")

    def action_mark_as_lost(self):
        """Action to mark a sale as lost."""
        self.ensure_one()
        self.order_id.write(
            {
                "is_lost": True,
                "lost_reason": self.lost_reason.id,
            }
        )
        self._create_lost_sale_record()
        return {"type": "ir.actions.act_window_close"}

    def _create_lost_sale_record(self):
        """Create a record in the lost sales model."""
        order = self.order_id
        for line in order.order_line:
            attribute_values = ", ".join(
                line.product_id.product_template_attribute_value_ids.mapped("name")
            )
            if line.product_id.default_code:
                order_name = f"{order.name} - [{line.product_id.default_code}] {line.product_id.name} ({attribute_values})"
            elif line.product_id.product_template_attribute_value_ids:
                order_name = (
                    f"{order.name} - {line.product_id.name} ({attribute_values})"
                )
            else:
                order_name = f"{order.name} - {line.product_id.name}"
            self.env["sale.order.lost"].create(
                {
                    "create_order": order.date_order,
                    "team_id": order.team_id.id,
                    "name": order.name,
                    "user_id": order.user_id.id,
                    "partner_id": order.partner_id.id,
                    "order_line_product_name": order_name,
                    "product_code": line.product_id.default_code,
                    "product_id": line.product_id.id,
                    "product_tmpl_id": line.product_id.product_tmpl_id.id,
                    "brand_name": line.product_id.brand_id.name,
                    "family_name": line.product_id.family_id.name,
                    "subfamily_name": line.product_id.subfamily_id.name,
                    "product_uom_qty": line.product_uom_qty,
                    "sale_lost_reason_id": self.lost_reason.id,
                    "sale_lost_reason_obs": self.sale_lost_reason_obs,
                    "currency_id": order.currency_id.id,
                    "company_id": order.company_id.id,
                }
            )
