# -- coding: utf-8 --
##############################################################################
#
#   Copyright (c) 2022-2025 Eynes SRL (Eynes - Ingenieria del software)
#   License AGPL-3.0 or later (http://www.gnu.org/licenses/agpl.html).
#
##############################################################################

import logging
from odoo import models, _, api
from odoo.exceptions import ValidationError

_logger = logging.getLogger(__name__)

class PurchaseOrder(models.Model):
    _inherit = 'purchase.order'

    @api.model_create_multi
    def create(self, vals_list):
        rec = super().create(vals_list)
        if self.env.user.has_group('sia_custom_permission.group_purchase_readonly'):
            raise ValidationError(_("You are not allowed to create a purchase order"))
        return rec

    def write(self, vals):
        if self.env.user.has_group('sia_custom_permission.group_purchase_readonly'):
            raise ValidationError(_("You are not allowed to edit a purchase order"))
        return super().write(vals)

    def unlink(self):
        if self.env.user.has_group('sia_custom_permission.group_purchase_readonly'):
            raise ValidationError(_("You are not allowed to delete a purchase order"))
        return super().unlink()
