# -- coding: utf-8 --
##############################################################################
#
#   Copyright (c) 2022-2025 Eynes SRL (Eynes - Ingenieria del software)
#   License AGPL-3.0 or later (http://www.gnu.org/licenses/agpl.html).
#
##############################################################################

import logging
from odoo import models, _, api
from odoo.exceptions import ValidationError

_logger = logging.getLogger(__name__)

class ProductTemplate(models.Model):
    """Inherit res.config.settings to add a field for users to notify when a cost is determined."""

    _inherit = "product.template"

    @api.model_create_multi
    def create(self, vals_list):
        company = self.env["res.users"].browse(self._uid).company_id
        if not company.is_main_company:
            raise ValidationError(_("Only the main company can create products."))
        templates = super(ProductTemplate, self).create(vals_list)
        
        return templates

    def write(self, vals):
        """Override write method to update the classification visibility"""
        company = self.env["res.users"].browse(self._uid).company_id
        confirming_purchase_order = self.env.context.get('confirming_purchase_order', False)
        is_picking = self.env.context.get('is_picking', False)
        if not company.is_main_company and not confirming_purchase_order and not is_picking:
            raise ValidationError(_("Only the main company can modify products."))
        
        res = super(ProductTemplate, self).write(vals)
        
        return res

    def unlink(self):
        """Override unlink method to update the classification visibility"""
        company = self.env["res.users"].browse(self._uid).company_id
        if not company.is_main_company:
            raise ValidationError(_("Only the main company can delete products."))
        res = super(ProductTemplate, self).unlink()

        return res


class ProductProduct(models.Model):
    _inherit = 'product.product'

    def _meli_stock_moves_update( self ):
        # Agregar un registro en el contexto
        self = self.with_context(is_picking=True)
        
        super(ProductProduct, self)._meli_stock_moves_update()

    @api.model_create_multi
    def create(self, vals_list):
        company = self.env["res.users"].browse(self._uid).company_id
        if not company.is_main_company:
            raise ValidationError(_("Only the main company can create product variants."))
        products = super(ProductProduct, self).create(vals_list)
        
        return products

    def write(self, vals):
        company = self.env["res.users"].browse(self._uid).company_id
        confirming_purchase_order = self.env.context.get('confirming_purchase_order', False)
        is_picking = self.env.context.get('is_picking', False)
        if not company.is_main_company and not confirming_purchase_order and not is_picking:
            raise ValidationError(_("Only the main company can modify product variants."))
        res = super(ProductProduct, self).write(vals)
        
        return res

    def unlink(self):
        company = self.env["res.users"].browse(self._uid).company_id
        if not company.is_main_company:
            raise ValidationError(_("Only the main company can delete product variants."))
        res = super(ProductProduct, self).unlink()
        
        return res